/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include "pim/AppointmentBuilder.h"
#include "base/Log.h"

/*
* Shif the day of week (or a combination of days) for the recurrence
*/
int convertDayOfWeekMaskBackward(int dayOfWeekMaskSent) {   
    int dayOfWeekMask = 0;
    if ((dayOfWeekMaskSent % 2) != 0) {
        dayOfWeekMask = dayOfWeekMaskSent >> 1;    
        dayOfWeekMask += 64;
    }
    else {
        dayOfWeekMask = dayOfWeekMaskSent >> 1;
    }
    return dayOfWeekMask;
}
int convertDayOfWeekMaskForward(int dayOfWeekMaskSent) {   
    int dayOfWeekMask = 0;
    if (dayOfWeekMaskSent >= 64) {
        dayOfWeekMask = dayOfWeekMaskSent << 1;    
        dayOfWeekMask -= 127;
    }
    else {
        dayOfWeekMask = dayOfWeekMaskSent << 1;
    }
    return dayOfWeekMask;
}

/*
* Method to populate the appointment XML structure. 
* It get every property name and value and build the XML structure to exchange with server.
*
* @param appointmentStringItem : the string that will contain the xml structure
* @param pAppointment          : the appointment object that contain every value
*
*/
void populateAppointmentStringItem(string &appointmentStringItem, IAppointment *pAppointment) {
	
	BSTR element        = NULL      ;
	long elementLong    = 0         ;
	DATE startdate      = NULL      ;
    DATE enddate        = NULL      ;
    DATE date           = NULL      ;
    string element_s    = string()  ;
    
    //
    // FOR RECURRENCE
    // to get if a modification in the UTC goes into day before or after
    // store the initial value of day and check the last value
    //
    int x = 0, y = 0, z = 0;
    BOOL changeDay = FALSE, isBackward = FALSE;
    BOOL changeYearBackward = FALSE, changeYearForward = FALSE;


	VARIANT_BOOL variantBool = NULL; // 0==false -1==true
	wchar_t tempDate [DIM_LOCAL_TEMP];

	wchar_t* charBlob = NULL;
	
    appointmentStringItem = TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
	appointmentStringItem = appointmentStringItem + TEXT("<appointment>");
		
		// BSTR
		pAppointment->get_Subject(&element);
        element_s = string(element); encodeSpecialChar(element_s);  
		appointmentStringItem  = appointmentStringItem + TEXT("<Subject>") + element_s + TEXT("</Subject>");
		
		pAppointment->get_Location(&element);
        element_s = string(element); encodeSpecialChar(element_s);  
		appointmentStringItem  = appointmentStringItem + TEXT("<Location>") + element_s + TEXT("</Location>");
		
		pAppointment->get_Categories(&element);
        element_s = string(element); encodeSpecialChar(element_s);  
		appointmentStringItem  = appointmentStringItem + TEXT("<Categories>") + element_s + TEXT("</Categories>");

		pAppointment->get_ReminderSoundFile(&element);
        element_s = string(element); encodeSpecialChar(element_s);  
		appointmentStringItem  = appointmentStringItem + TEXT("<ReminderSoundFile>") + element_s + TEXT("</ReminderSoundFile>");
		
		pAppointment->get_Body(&element);
        removeEndCarriage(&element);
        element_s = string(element);  encodeSpecialChar(element_s);  
		appointmentStringItem  = appointmentStringItem + TEXT("<Body>") + element_s + TEXT("</Body>");									
		
		// DATE
		pAppointment->get_Start(&startdate);
        pAppointment->get_End(&enddate);

        /*
        * see the start and end date. If the start is 00:00 and end is 23:59 the appointment is decided to be 
        * an all day event. So the AllDayEvent tag is put to 'true' only in the SIF structure. The format of the date
        * is like the birthday: yyyy-MM-dd
        *
        */

        BOOL isAllDay = isAllDayEvent(startdate, enddate);

		wsprintf(tempDate, TEXT(""));		
		double toCompare = MAX_DATE_DOUBLE;
        double toCompare4000 = MAX_DATE_DOUBLE_4000;

		SYSTEMTIME st;

		if (startdate != toCompare) { 
            VariantTimeToSystemTime(startdate, &st);
            x = st.wHour;
            localTimeToUTC(st);
            y = st.wHour;
            if (isAllDay) {
                doubleToSystemTimeBirthday(tempDate, startdate);
            } else {
			    doubleToSystemTime(tempDate, startdate);			
            }
		}
		appointmentStringItem  = appointmentStringItem + TEXT("<Start>") + tempDate + TEXT("</Start>");		
		
		
		wsprintf(tempDate, TEXT(""));
		if (enddate != toCompare) { 
            if (isAllDay) {
                doubleToSystemTimeBirthday(tempDate, enddate);
            } else {
			    doubleToSystemTime(tempDate, enddate);
            }
		}
		appointmentStringItem  = appointmentStringItem + TEXT("<End>") + tempDate + TEXT("</End>");
		

        // VARIANT
		pAppointment->get_AllDayEvent(&variantBool);
		wsprintf(tempDate, TEXT("%i"), variantBool);
        if (isAllDay) {
            wsprintf(tempDate, TEXT("-1")); // TRUE
        }
		normalizeBoolean(tempDate);
		appointmentStringItem  = appointmentStringItem + TEXT("<AllDayEvent>") + tempDate + TEXT("</AllDayEvent>");	

		pAppointment->get_ReminderSet(&variantBool);
		wsprintf(tempDate, TEXT("%i"), variantBool);
		normalizeBoolean(tempDate);
		appointmentStringItem  = appointmentStringItem + TEXT("<ReminderSet>") + tempDate + TEXT("</ReminderSet>");		

		// long
		//pAppointment->get_Duration(&elementLong);		
		//wsprintf(tempDate, TEXT("%i"), elementLong);
		//appointmentStringItem  = appointmentStringItem + TEXT("<Duration>") + tempDate + TEXT("</Duration>");
		
		pAppointment->get_BusyStatus(&elementLong);		
		wsprintf(tempDate, TEXT("%i"), elementLong);
		appointmentStringItem  = appointmentStringItem + TEXT("<BusyStatus>") + tempDate + TEXT("</BusyStatus>");
		
		pAppointment->get_Sensitivity(&elementLong);		
		wsprintf(tempDate, TEXT("%i"), elementLong);
		appointmentStringItem  = appointmentStringItem + TEXT("<Sensitivity>") + tempDate + TEXT("</Sensitivity>");
		
		pAppointment->get_ReminderOptions(&elementLong);		
		wsprintf(tempDate, TEXT("%i"), elementLong);
		appointmentStringItem  = appointmentStringItem + TEXT("<ReminderOptions>") + tempDate + TEXT("</ReminderOptions>");
		
		pAppointment->get_ReminderMinutesBeforeStart(&elementLong);		
		wsprintf(tempDate, TEXT("%i"), elementLong);
		appointmentStringItem  = appointmentStringItem + TEXT("<ReminderMinutesBeforeStart>") + tempDate + TEXT("</ReminderMinutesBeforeStart>");
		

		/******************************************************/
		//Recurrence
		pAppointment->get_IsRecurring(&variantBool);		
		wsprintf(tempDate, TEXT("%i"), variantBool);
		normalizeBoolean(tempDate);
		appointmentStringItem  = appointmentStringItem + TEXT("<IsRecurring>") + tempDate + TEXT("</IsRecurring>");
		
		IRecurrencePattern *pRecurrence;
		pAppointment->GetRecurrencePattern (&pRecurrence);
        
        long            recurrenceType = 0,
                        interval       = 0,
                        monthOfYear    = 0,
                        dayOfMonth     = 0,
                        dayOfWeekMask  = 0,
                        instance       = 0;   

        pRecurrence->get_RecurrenceType(&recurrenceType);
        pRecurrence->get_Interval(&interval);			
	    pRecurrence->get_MonthOfYear(&monthOfYear);			
	    pRecurrence->get_DayOfMonth(&dayOfMonth);		      	
	    pRecurrence->get_Instance(&instance);
        pRecurrence->get_DayOfWeekMask(&dayOfWeekMask);
               
        //
        // Modification to get right information about changing day, month...
        //
        TIME_ZONE_INFORMATION tmz;            
        int k = GetTimeZoneInformation(&tmz);        
    
        z = (x-y) < 0 ? (x-y)*(-1) : (x-y) ;    
    
        //
        // Change day?
        //
        changeDay = z > 12 ? TRUE : FALSE;
        if (changeDay) {
            if (tmz.Bias < 0)
                isBackward = TRUE; 
            else 
                isBackward = FALSE;
        }
            
        //
        // RecurrenceType
        //
        if (recurrenceType < 0) {
            recurrenceType = 0;
        }
        wsprintf(tempDate, TEXT("%i"), recurrenceType);        
		appointmentStringItem  = appointmentStringItem + TEXT("<RecurrenceType>") + tempDate + TEXT("</RecurrenceType>");
        
        //
        // Interval
        //
        wsprintf(tempDate, TEXT("%i"), interval);
		appointmentStringItem  = appointmentStringItem + TEXT("<Interval>") + tempDate + TEXT("</Interval>");		
        
        //
        // Day of Month
        //
        if (dayOfMonth != 0) {
            if (changeDay) {
                if (isBackward) {
                    dayOfMonth = dayOfMonth - 1;
                    if (dayOfMonth == 0) {
                        if (monthOfYear == 5 || monthOfYear == 7 || monthOfYear == 10) { 
                            // if May, July, October the month before has 30 days
                            dayOfMonth = 30;
                        } else {
                            dayOfMonth = 31;
                        }
                        changeYearBackward = TRUE;  
                    }
                } else { // is 
                    dayOfMonth = dayOfMonth + 1;
                    if (dayOfMonth >  31  || 
                          (dayOfMonth == 31 && 
                            (monthOfYear == 4 || monthOfYear == 6 || monthOfYear == 9)
                           )
                        ) {
                        dayOfMonth = 1;
                        changeYearForward = TRUE;
                    }
                }
            }
            wsprintf(tempDate, TEXT("%i"), dayOfMonth);        
		    appointmentStringItem  = appointmentStringItem + TEXT("<DayOfMonth>") + tempDate + TEXT("</DayOfMonth>");	      	
        } else {
            appointmentStringItem  = appointmentStringItem + TEXT("<DayOfMonth>0</DayOfMonth>");
        }
        
        //
        // Month of Year
        //
        if (monthOfYear != 0) {
            if (changeYearBackward) {
                monthOfYear = monthOfYear - 1;
                if (monthOfYear == 0) {
                    monthOfYear = 12;
                }
            } else if (changeYearForward) {
                monthOfYear = monthOfYear + 1;
                if (monthOfYear > 12) {
                    monthOfYear = 1;
                }
            }        
            wsprintf(tempDate, TEXT("%i"), monthOfYear);
		    appointmentStringItem  = appointmentStringItem + TEXT("<MonthOfYear>") + tempDate + TEXT("</MonthOfYear>");		
        } else {
            appointmentStringItem  = appointmentStringItem + TEXT("<MonthOfYear>0</MonthOfYear>");
        }
        //
        // DayOfWeekMask
        //
        if (dayOfWeekMask != 0) {
            if (changeDay) {
                if (isBackward) {
                    dayOfWeekMask = convertDayOfWeekMaskBackward(dayOfWeekMask);
                } else {
                    dayOfWeekMask = convertDayOfWeekMaskForward(dayOfWeekMask);
                }
            }
            wsprintf(tempDate, TEXT("%i"), dayOfWeekMask);
		    appointmentStringItem  = appointmentStringItem + TEXT("<DayOfWeekMask>") + tempDate + TEXT("</DayOfWeekMask>");
        } else {
            appointmentStringItem  = appointmentStringItem + TEXT("<DayOfWeekMask>0</DayOfWeekMask>");
        }

        //
        // Instance
        //
        if (instance != 0) {
            wsprintf(tempDate, TEXT("%i"), instance);
		    appointmentStringItem  = appointmentStringItem + TEXT("<Instance>") + tempDate + TEXT("</Instance>");
        } else {
            appointmentStringItem  = appointmentStringItem + TEXT("<Instance>0</Instance>");
        }
        
        //
        // Pattern Start Date
        //
        pRecurrence->get_PatternStartDate(&date);
		wsprintf(tempDate, TEXT(""));		
		toCompare = MAX_DATE_DOUBLE;
		
		if (date != toCompare && date < toCompare4000 ) { 
			doubleToSystemTime(tempDate, date);			
		}
		appointmentStringItem  = appointmentStringItem + TEXT("<PatternStartDate>") + tempDate + TEXT("</PatternStartDate>");		
		
        //
        // No End Date
        //
		pRecurrence->get_NoEndDate(&variantBool);
		wsprintf(tempDate, TEXT("%i"), variantBool);
		normalizeBoolean(tempDate);
		appointmentStringItem  = appointmentStringItem + TEXT("<NoEndDate>") + tempDate + TEXT("</NoEndDate>");	

        //
        // Pattern End Date
        //
		pRecurrence->get_PatternEndDate(&date);
		wsprintf(tempDate, TEXT(""));		
		toCompare = MAX_DATE_DOUBLE;
		
		if (date != toCompare && date < toCompare4000 ) { 
			doubleToSystemTime(tempDate, date);		
        } 
		appointmentStringItem  = appointmentStringItem + TEXT("<PatternEndDate>") + tempDate + TEXT("</PatternEndDate>");
        
        //
        // Occurrences
        //

        pRecurrence->get_Occurrences(&elementLong);		
        if (elementLong < 0) { 
            elementLong = 0;
        }
		wsprintf(tempDate, TEXT("%i"), elementLong);
		appointmentStringItem  = appointmentStringItem + TEXT("<Occurrences>") + tempDate + TEXT("</Occurrences>");
		

        /*
		pRecurrence->get_RecurrenceType(&elementLong);		
		wsprintf(tempDate, TEXT("%i"), elementLong);
		appointmentStringItem  = appointmentStringItem + TEXT("<RecurrenceType>") + tempDate + TEXT("</RecurrenceType>");

		pRecurrence->get_Interval(&elementLong);		
		wsprintf(tempDate, TEXT("%i"), elementLong);
		appointmentStringItem  = appointmentStringItem + TEXT("<Interval>") + tempDate + TEXT("</Interval>");

		pRecurrence->get_MonthOfYear(&elementLong);		
		wsprintf(tempDate, TEXT("%i"), elementLong);
		appointmentStringItem  = appointmentStringItem + TEXT("<MonthOfYear>") + tempDate + TEXT("</MonthOfYear>");

		pRecurrence->get_DayOfMonth(&elementLong);				        
        wsprintf(tempDate, TEXT("%i"), elementLong);        
		appointmentStringItem  = appointmentStringItem + TEXT("<DayOfMonth>") + tempDate + TEXT("</DayOfMonth>");
		
		pRecurrence->get_DayOfWeekMask(&elementLong);	              
		wsprintf(tempDate, TEXT("%i"), elementLong);
		appointmentStringItem  = appointmentStringItem + TEXT("<DayOfWeekMask>") + tempDate + TEXT("</DayOfWeekMask>");
	
		pRecurrence->get_Instance(&elementLong);		
		wsprintf(tempDate, TEXT("%i"), elementLong);
		appointmentStringItem  = appointmentStringItem + TEXT("<Instance>") + tempDate + TEXT("</Instance>");
	
		pRecurrence->get_PatternStartDate(&date);
		wsprintf(tempDate, TEXT(""));		
		toCompare = MAX_DATE_DOUBLE;
		
		if (date != toCompare) { 
			doubleToSystemTime(tempDate, date);			
		}
		appointmentStringItem  = appointmentStringItem + TEXT("<PatternStartDate>") + tempDate + TEXT("</PatternStartDate>");		
		
		pRecurrence->get_NoEndDate(&variantBool);
		wsprintf(tempDate, TEXT("%i"), variantBool);
		normalizeBoolean(tempDate);
		appointmentStringItem  = appointmentStringItem + TEXT("<NoEndDate>") + tempDate + TEXT("</NoEndDate>");	

		pRecurrence->get_PatternEndDate(&date);

		wsprintf(tempDate, TEXT(""));		
		toCompare = MAX_DATE_DOUBLE;
		
		if (date != toCompare) { 
			doubleToSystemTime(tempDate, date);		
		}
		appointmentStringItem  = appointmentStringItem + TEXT("<PatternEndDate>") + tempDate + TEXT("</PatternEndDate>");		
		*/

		
		/******************************************* TO BE DEVELOPED *******************************
		// Irecipient
		IRecipients *pRecipients;
		IRecipient *pRecipient;
		recipients: to do... 

		pAppointment->get_Recipients(&pRecipients);
		int countRecipient = 0;
		pRecipients->get_Count(&countRecipient);

		// insert <recipients> tag 
		appointmentStringItem  = appointmentStringItem + TEXT("<Recipients>");
		
		
		
		for (int i = 0; i < countRecipient; i++) {
			appointmentStringItem  = appointmentStringItem + TEXT("<Recipient>");
			pRecipients->Item(i + 1, &pRecipient);
			pRecipient->get_Address(&element);
			appointmentStringItem  = appointmentStringItem + TEXT("<Address>") + element + TEXT("</Address>");
			pRecipient->get_Name(&element);
			appointmentStringItem  = appointmentStringItem + TEXT("<Name>") + element + TEXT("</Name>");
			appointmentStringItem  = appointmentStringItem + TEXT("</Recipient>");
			pRecipient->Release();
			

		}
		pRecipients->Release();
		appointmentStringItem  = appointmentStringItem + TEXT("</Recipients>");
		************************////////////////////////////////////
				
		//CEBLOB. Use body. BodyInk used only if Ink Control used.
		/*
		CEBLOB ceblob;
		appointmentStringItem  = appointmentStringItem + TEXT("<BodyInk>");
		pAppointment->get_BodyInk(&ceblob);
		wsprintf(temp, TEXT("%i"), ceblob.dwCount);
		MessageBox (NULL, temp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);		
		MultiByteToWideChar (CP_ACP, 0, (const char*)charBlob, ceblob.dwCount, (unsigned short* )ceblob.lpb, ceblob.dwCount);		
		appointmentStringItem  = appointmentStringItem + charBlob ;	
		appointmentStringItem  = appointmentStringItem + TEXT("</BodyInk>");
		*/

	appointmentStringItem = appointmentStringItem +  TEXT("</appointment>");
	
	if (dbg)
		MessageBox (NULL, appointmentStringItem, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

}


/*
* Method that complete an appointment object retrieving property value by an XML structure.
* It parse the XML structure, get the every property value using its name and set it into the object.
*
* @param pAppointment : the appointment object that contain every value
* @param ptrData      : the variable that contain the xml structure
*/

void completeAppointment(IAppointment *pAppointment, wchar_t *ptrData) {
		
    if (dbg) {
		MessageBox (NULL, TEXT ("in complete appointment"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    }

	wchar_t* dummy                  = NULL;
	wchar_t* dummyEvent             = NULL;		
	string   dummyEvent_s           = string();
	VARIANT_BOOL isNotAllDayEvent   = TRUE;	
    
    //
    // FOR RECURRENCE
    // to get if a modification in the UTC goes into day before or after
    // store the initial value of day and check the last value
    //
    int x = 0, y = 0, z = 0;
    BOOL changeDay = FALSE, isBackward = FALSE;
    BOOL changeYearBackward = FALSE, changeYearForward = FALSE;

	pAppointment->ClearRecurrencePattern ();

    //VARIANT_BOOL --- AllDayEvent
	dummyEvent = getElementContent(ptrData, TEXT ("AllDayEvent"), NULL);
	
	// with workaround normalizeAppointments this condition will be always false!!

	if (dummyEvent != NULL && 
        (wcscmp(dummyEvent, TEXT("1")) == 0 ||
         wcscmp(dummyEvent, TEXT("-1")) == 0 || // kept to be compliant with old client. Now TRUE is represented
         wcscmp(dummyEvent, TEXT("True")) == 0)) {  // with "1" only		 

		pAppointment->put_AllDayEvent(FALSE);
		double locDate = NULL;
		
		dummyEvent = getElementContent(ptrData, TEXT ("Start"), NULL);
               
        DATE st;
        SYSTEMTIME systemTime;
        BOOL isUTC = FALSE;
        BOOL isAllDayFormat = isDateForAllDayEventFormat(dummyEvent);
        
        if (isAllDayFormat) {
            systemTimeToDoubleBirthFormat(dummyEvent, &st, TEXT("START"));

        }  else { // the date is in the usual format
        
            isUTC = getDateAndTimeAsIs(dummyEvent, &st);
        
            VariantTimeToSystemTime(st, &systemTime);
            x = systemTime.wHour;           
            if (isUTC) {
                UTCToLocalTime(systemTime);
            }
            y = systemTime.wHour;
            systemTime.wHour = 0;
            systemTime.wMinute = 0;
            systemTime.wMilliseconds = 0;
            SystemTimeToVariantTime(&systemTime, &st);
        }
        pAppointment->put_Start(st);
        
        dummyEvent = getElementContent(ptrData, TEXT ("End"), NULL);
        isAllDayFormat = isDateForAllDayEventFormat(dummyEvent);

        if (isAllDayFormat) {
            systemTimeToDoubleBirthFormat(dummyEvent, &st, TEXT("END"));

        }  else { // the date is in the usual format
        
            isUTC = FALSE;
            isUTC = getDateAndTimeAsIs(dummyEvent, &st);
        
            VariantTimeToSystemTime(st, &systemTime);
        
            if (isUTC) {
                UTCToLocalTime(systemTime);
            }
  
            systemTime.wHour = 23;
            systemTime.wMinute = 59;
            systemTime.wMilliseconds = 0;
            SystemTimeToVariantTime(&systemTime, &st);  
        }

        pAppointment->put_End(st);
	
		delete dummyEvent;dummyEvent=NULL;
		isNotAllDayEvent = FALSE;
	}
	else {
		pAppointment->put_AllDayEvent(VARIANT_FALSE);	
		
	}		
	delete dummyEvent;dummyEvent=NULL;		
		
	//VARIANT_BOOL -- ReminderSet 
	dummyEvent = getElementContent(ptrData, TEXT ("ReminderSet"), NULL);
	

	if (dummyEvent != NULL && 
        (wcscmp(dummyEvent, TEXT("1")) == 0 || 
         wcscmp(dummyEvent, TEXT("-1")) == 0 ||     // kept to be compliant with old client. Now TRUE is represented
         wcscmp(dummyEvent, TEXT("True")) == 0)) {  // with "1" only
		pAppointment->put_ReminderSet(VARIANT_TRUE);

	}
	else {
		pAppointment->put_ReminderSet(VARIANT_FALSE);	
	}
	
	delete dummyEvent;dummyEvent=NULL;		
		
	// BSTR
	dummyEvent = getElementContent(ptrData, TEXT ("Subject"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = string(dummyEvent); decodeSpecialChar(dummyEvent_s);
		pAppointment->put_Subject ((wchar_t*)dummyEvent_s);
   		delete dummyEvent;dummyEvent=NULL;
    }
    
	dummyEvent = getElementContent(ptrData, TEXT ("Location"), NULL);
	if (dummyEvent != NULL) {
        dummyEvent_s = string(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pAppointment->put_Location ((wchar_t*)dummyEvent_s);
		delete dummyEvent;dummyEvent=NULL;
    }
    
	dummyEvent = getElementContent(ptrData, TEXT ("ReminderSoundFile"), NULL);
    if (dummyEvent != NULL) {      
        dummyEvent_s = string(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pAppointment->put_ReminderSoundFile  ((wchar_t*)dummyEvent_s);
		delete dummyEvent;dummyEvent=NULL;
    }
	dummyEvent = getElementContent(ptrData, TEXT ("Categories"), NULL);
    if (dummyEvent != NULL) {		
        dummyEvent_s = string(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pAppointment->put_Categories ((wchar_t*)dummyEvent_s);
		delete dummyEvent;dummyEvent=NULL;
    }

	dummyEvent = getElementContent(ptrData, TEXT ("Body"), NULL);
	if (dummyEvent != NULL) {
        dummyEvent_s = string(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pAppointment->put_Body  ((wchar_t*)dummyEvent_s);
		delete dummyEvent;dummyEvent=NULL;
    }		
	if (isNotAllDayEvent) {
		wchar_t localTemp[DIM_LOCAL_TEMP];
		//DATE
		dummyEvent = getElementContent(ptrData, TEXT ("Start"), NULL);

		if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
			;
		else {
			DATE st;
            SYSTEMTIME systemTime;
            //
            // to handle properly the RecurrencePattern
            //
            BOOL isUTC = FALSE;
            isUTC = getDateAndTimeAsIs(dummyEvent, &st);
            
            VariantTimeToSystemTime(st, &systemTime);
            x = systemTime.wHour;           
            if (isUTC) {
                UTCToLocalTime(systemTime);
            }
            y = systemTime.wHour;
            
			systemTimeToDouble(dummyEvent, &st, NULL);
			if (dbg) {
				wsprintf(localTemp, TEXT("st %f"), st);	
				MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
			}
		pAppointment->put_Start(st);
		delete dummyEvent;
		}

		dummyEvent = getElementContent(ptrData, TEXT ("End"), NULL);

		if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
			;
		else {
			DATE en;
            
			systemTimeToDouble(dummyEvent, &en, NULL);
			if (dbg) {
				wsprintf(localTemp, TEXT("en %f"), en);	
				MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
			}

		pAppointment->put_End(en);
		delete dummyEvent;
		}

	}
        
	//long
	//dummyEvent = getElementContent(ptrData, TEXT ("Duration"), NULL);
    //if (dummyEvent != NULL) {
	//	pAppointment->put_Duration(wcstol(dummyEvent, &dummy, 0));
	//	delete dummyEvent;dummyEvent=NULL;
    //}

	dummyEvent = getElementContent(ptrData, TEXT ("BusyStatus"), NULL);
    if (dummyEvent != NULL) {
		pAppointment->put_BusyStatus(wcstol(dummyEvent, &dummy, 0));
	    delete dummyEvent;dummyEvent=NULL;
    }

	dummyEvent = getElementContent(ptrData, TEXT ("Sensitivity"), NULL);
    if (dummyEvent != NULL) {
		pAppointment->put_Sensitivity(wcstol(dummyEvent, &dummy, 0));
		delete dummyEvent;dummyEvent=NULL;
    }

	dummyEvent = getElementContent(ptrData, TEXT ("ReminderOptions"), NULL);
    if (dummyEvent != NULL) {
		pAppointment->put_ReminderOptions (wcstol(dummyEvent, &dummy, 0));
		delete dummyEvent;dummyEvent=NULL;
    }

	dummyEvent = getElementContent(ptrData, TEXT ("ReminderMinutesBeforeStart"), NULL);
    if (dummyEvent != NULL) {
		pAppointment->put_ReminderMinutesBeforeStart  (wcstol(dummyEvent, &dummy, 0));
		delete dummyEvent;dummyEvent=NULL;
    }
				
	/****************************************************/
	
	//Recurrence
	dummyEvent = getElementContent(ptrData, TEXT ("IsRecurring"), NULL);
	
	if (dbg) {		
		MessageBox (NULL, dummyEvent, TEXT ("dummyEvent"), MB_SETFOREGROUND |MB_OK);
	}
    long            recurrenceType = 0,
                    interval       = 0,
                    monthOfYear    = 0,
                    dayOfMonth     = 0,
                    dayOfWeekMask  = 0,
                    instance       = 0;   

	if (dummyEvent != NULL &&
        (wcscmp(dummyEvent, TEXT("1")) == 0 ||
         wcscmp(dummyEvent, TEXT("-1")) == 0 ||    // kept to be compliant with old client. Now TRUE is represented
         wcscmp(dummyEvent, TEXT("True")) == 0)) { // with "1" only
				
		IRecurrencePattern *pRecurrence;
		pAppointment->GetRecurrencePattern (&pRecurrence);
        
        //
        // Modification to get right information about changing day, month...
        //
        TIME_ZONE_INFORMATION tmz;            
        int k = GetTimeZoneInformation(&tmz);        
    
        z = (x-y) < 0 ? (x-y)*(-1) : (x-y) ;    
    
        //
        // Change day? Backward and Forward are inverted because it is from utc to localtime
        //
        changeDay = z > 12 ? TRUE : FALSE;
        if (changeDay) {
            if (tmz.Bias < 0)
                isBackward = FALSE; 
            else 
                isBackward = TRUE;
        }

		dummyEvent = getElementContent(ptrData, TEXT ("RecurrenceType"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) >= 0) {
			pRecurrence->put_RecurrenceType(wcstol(dummyEvent, &dummy, 0));			
			delete dummyEvent;dummyEvent=NULL;
        }

		dummyEvent = getElementContent(ptrData, TEXT ("Interval"), NULL);		
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
            pRecurrence->put_Interval(wcstol(dummyEvent, &dummy, 0));		
			delete dummyEvent;dummyEvent=NULL;
        }
                
        //
        // Day of Month
        //
        dummyEvent = getElementContent(ptrData, TEXT ("DayOfMonth"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
            dayOfMonth = wcstol(dummyEvent, &dummy, 0);
			if (changeDay) {
                if (isBackward) {
                    dayOfMonth = dayOfMonth - 1;
                    if (dayOfMonth == 0) {
                        if (monthOfYear == 5 || monthOfYear == 7 || monthOfYear == 10) { 
                            // if May, July, October the month before has 30 days
                            dayOfMonth = 30;
                        } else {
                            dayOfMonth = 31;
                        }
                        changeYearBackward = TRUE;  
                    }
                } else { // is 
                    dayOfMonth = dayOfMonth + 1;
                    if (dayOfMonth >  31  || 
                          (dayOfMonth == 31 && 
                            (monthOfYear == 4 || monthOfYear == 6 || monthOfYear == 9)
                           )
                        ) {
                        dayOfMonth = 1;
                        changeYearForward = TRUE;
                    }
                }
            }            
            pRecurrence->put_DayOfMonth(dayOfMonth);			
			delete dummyEvent;dummyEvent=NULL;
        }
                
         //
        // Month of Year
        //
        dummyEvent = getElementContent(ptrData, TEXT ("MonthOfYear"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy , 0) > 0) {
            monthOfYear = wcstol(dummyEvent, &dummy , 0);
			if (changeYearBackward) {
                monthOfYear = monthOfYear - 1;
                if (monthOfYear == 0) {
                    monthOfYear = 12;
                }
            } else if (changeYearForward) {
                monthOfYear = monthOfYear + 1;
                if (monthOfYear > 12) {
                    monthOfYear = 1;
                }
            }       
            pRecurrence->put_MonthOfYear(monthOfYear);			
			delete dummyEvent;dummyEvent=NULL;
        }
       
        //
        // DayOfWeekMask
        //
        dummyEvent = getElementContent(ptrData, TEXT ("DayOfWeekMask"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
            dayOfWeekMask = wcstol(dummyEvent, &dummy, 0);
			if (changeDay) {
                if (isBackward) {
                    dayOfWeekMask = convertDayOfWeekMaskBackward(dayOfWeekMask);
                } else {
                    dayOfWeekMask = convertDayOfWeekMaskForward(dayOfWeekMask);
                }
            }
                        
            pRecurrence->put_DayOfWeekMask(dayOfWeekMask);			
			delete dummyEvent;dummyEvent=NULL;
        }       

		dummyEvent = getElementContent(ptrData, TEXT ("Instance"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
			pRecurrence->put_Instance(wcstol(dummyEvent, &dummy, 0));			
			delete dummyEvent;dummyEvent=NULL;
        }
		
		//Pattern Start Date
		dummyEvent = getElementContent(ptrData, TEXT ("PatternStartDate"), NULL);

		if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
			;
		else {
			DATE st;
            
			systemTimeToDouble(dummyEvent, &st, NULL);

			pRecurrence->put_PatternStartDate(st);
			delete dummyEvent;
		}

		//VARIANT_BOOL -- NoEndDate
		dummyEvent = getElementContent(ptrData, TEXT ("NoEndDate"), NULL);
		

		if (dummyEvent != NULL && 
            (wcscmp(dummyEvent, TEXT("1")) == 0 || 
             wcscmp(dummyEvent, TEXT("-1")) == 0 || 
			 wcscmp(dummyEvent, TEXT("True")) == 0)) {
			pRecurrence->put_NoEndDate(VARIANT_TRUE);

		} else {
			pRecurrence->put_NoEndDate(VARIANT_FALSE);

			//Pattern End Date
			dummyEvent = getElementContent(ptrData, TEXT ("PatternEndDate"), NULL);

			if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
				;
			else {
				DATE en;
                
				systemTimeToDouble(dummyEvent, &en, NULL);
	
				pRecurrence->put_PatternEndDate(en);
				delete dummyEvent;
			}

		}
        // Occurrences
        //
        dummyEvent = getElementContent(ptrData, TEXT ("Occurrences"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
            pRecurrence->put_Occurrences(wcstol(dummyEvent, &dummy , 0));

        }

	} else {
		;
	}

     //
    
		
	/*****************************RETRIEVE RECIPIENTS: delete current and write new!! ********
	todo...
	wchar_t localTemp [DIM_LOCAL_TEMP];		
	
	dummyEvent = getElementContent(ptrData, TEXT ("Recipients"), NULL);
			
	IRecipients *pRecipients;
	IRecipient *pRecipient;
	pAppointment->get_Recipients(&pRecipients);
	int countRecipient = 0;
	pRecipients->get_Count(&countRecipient);
	
	for (int i = 0; i < countRecipient; i++) {			
		pRecipients->Remove(i + 1);
	
	}


	wchar_t *pcRecipient = NULL;
	unsigned int pos = 0;
	size_t len = 0;
	pcRecipient = getElementContent(dummyEvent, TEXT ("Recipient"), NULL);
	
	int k = 0;
	
	while (pcRecipient != NULL) {
		
		dummy = getElementContent(pcRecipient, TEXT ("Name"), NULL);
		
		wsprintf(localTemp, TEXT("%s"), dummy );
		MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

		pRecipients->Add (dummy, &pRecipient);

		delete dummy; dummy = NULL;

		dummyEvent = dummyEvent + wcslen(pcRecipient) + wcslen(TEXT("<Recipient>")) + wcslen(TEXT("</Recipient>"));
		pcRecipient = getElementContent(dummyEvent, TEXT ("Recipient"), &pos);

		k++;

	}
	
	IPOlRecipient *pRecip2;
	VARIANT_BOOL bResolved = FALSE;
	int iRecipientCount = 0;
	pRecipients->get_Count (&iRecipientCount);
	// Count down so removing recipient does not reorder
	// remaining unresolved recipients.
	for (int j = iRecipientCount; j > 0; j--) {
		pRecipients->Item (j, &pRecipient);
		pRecipient->QueryInterface (IID_IPOlRecipient, (LPVOID*)&pRecip2);
		pRecip2->Resolve (TRUE, &bResolved);
		if (!bResolved) 
			pRecipients->Remove (j);
	}
	pRecip2->Release();

	****************************************************/
	
}

/*
* The date in all day event format is yyyy-MM-dd
* To discover if it is in this format find the '-' char inside the string
*/
BOOL isDateForAllDayEventFormat(wchar_t* stringDate) {
        
    BOOL ret = FALSE;
    
    wchar_t* found = NULL;
    found = wcschr(stringDate, '-');
    if (found != NULL) {
        ret = TRUE;
    }

    return ret;

}


BOOL isAllDayEvent(DATE startdate, DATE enddate) {
    
    SYSTEMTIME ststart, stend;
    VariantTimeToSystemTime(startdate, &ststart);
    VariantTimeToSystemTime(enddate, &stend);
    BOOL ret = FALSE;

    if (ststart.wHour == 0 && ststart.wMinute == 0 &&
        stend.wHour == 23 && stend.wMinute == 59) 
        
        ret = TRUE;

    return ret;

}




/*
* The method change the property AllDayEvent to False if set to true.
* This because changing AllDayEvent changes OID too. So 
* if AllDayEvent == YES (-1) ---> AllDayEvent = No, start = 0, end = 24
*
* @param pAppointment : the appointment object that contain every value
* @param dataType     : the data type to get retrieve appointments
*/

void normalizeAppointment(IPOutlookApp *polApp, int dataType) {


    IPOutlookItemCollection *pItems;
    IAppointment *pAppointment;
    IFolder *pFolder;

    VARIANT_BOOL variantBool = FALSE;
    wchar_t tempDate [30];
    DATE date = NULL;


    polApp->GetDefaultFolder (dataType, &pFolder);
    pFolder->get_Items (&pItems);

    int currentCountItems = 0;	
    
    pItems->get_Count(&currentCountItems);


    for (int i = 0; i < currentCountItems; i++) {
        pItems->Item (i + 1, (IDispatch**)&pAppointment);
        pAppointment->get_AllDayEvent(&variantBool);
        wsprintf(tempDate, TEXT("%i"), variantBool);

        if (wcscmp(tempDate, TEXT("-1")) == 0) {
            if (dbg)
                MessageBox (NULL, TEXT("In modifica AllDay"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

            SYSTEMTIME t;
            pAppointment->put_AllDayEvent(FALSE);
            // DATE
            // START****************
            pAppointment->get_Start(&date);

            VariantTimeToSystemTime(date, &t); 
            t.wHour = 0;
            t.wMinute = 0;
            SystemTimeToVariantTime(&t, &date);

            if (dbg) {
                wsprintf(tempDate, TEXT("%f"), date);
                MessageBox (NULL, tempDate, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            }
            pAppointment->put_Start(date);

            //END*************
            pAppointment->get_End(&date);

            VariantTimeToSystemTime(date, &t); 
            t.wHour = 23;
            t.wMinute = 59;
            SystemTimeToVariantTime(&t, &date);
            if (dbg) {
                wsprintf(tempDate, TEXT("%f"), date);
                MessageBox (NULL, tempDate, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            }					

            pAppointment->put_End(date);
            HRESULT  resu = pAppointment->Save();

            if (dbg) {
                wsprintf(tempDate, TEXT("Save new result(0 ok): %i"), resu);
                MessageBox (NULL, tempDate, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            }
        }

        pAppointment->Release ();
    }

    pFolder->Release();
    pItems->Release();
}

/*
* The method calculate the CRC hash value for the appointment item.
* It create the XML structure and calculate the crc value.
* Use to compare for understand the modified value!
*
* @param pAppointment : the appointment object that contain every value
*/

long calculateAppointmentHash (IAppointment* pAppointment) {

	CCrc32 cc = CCrc32();
	string p = string();
	BSTR element = NULL;	    

	populateAppointmentStringItem(p, pAppointment);	
    
    BSTR dummy = p;

    long l = cc.crc32String( &dummy);
	
    

	return l;
}

VObject* AppointmentToVObject(IAppointment *pAppointment) {

	BSTR element = NULL;
	DATE startdate = NULL;
    DATE enddate   = NULL;
	wchar_t* dateValue = NULL;
	VObject* vo = new VObject();
	
	vo->addProperty(new VProperty(TEXT("BEGIN"), TEXT("VCALENDAR")));
	vo->addProperty(new VProperty(TEXT("BEGIN"), TEXT("VEVENT")));
	
	pAppointment->get_Subject(&element);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("SUMMARY"), element));

	pAppointment->get_Location(&element);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("LOCATION"), element));

	pAppointment->get_Body(&element);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("DESCRIPTION"), element));
	
	pAppointment->get_Start(&startdate);
	dateValue = new wchar_t[20];
	doubleToSystemTime(dateValue, startdate);
	vo->addProperty(new VProperty(TEXT("DTSTART"), dateValue));
	delete [] dateValue; dateValue = NULL;

	pAppointment->get_End(&enddate);
	dateValue = new wchar_t[20];
	doubleToSystemTime(dateValue, enddate);
	vo->addProperty(new VProperty(TEXT("DTEND"), dateValue));
	delete [] dateValue; dateValue = NULL;
	
	pAppointment->get_Categories(&element);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("CATEGORIES"), element));

	long sensitivity;
	pAppointment->get_Sensitivity(&sensitivity);
	if(sensitivity == olPrivate)
		vo->addProperty(new VProperty(TEXT("CLASS"), TEXT("PRIVATE")));
	else
		vo->addProperty(new VProperty(TEXT("CLASS"), TEXT("PUBLIC")));

	IRecipients* pRecipients;
	pAppointment->get_Recipients(&pRecipients);
	int countRecipient = 0;
	pRecipients->get_Count(&countRecipient);

	for(int i = 0; i < countRecipient; i++) {
		IRecipient* pRecipient;
		pRecipients->Item(i + 1, &pRecipient);

		wchar_t* attedee = new wchar_t[100];
		pRecipient->get_Name(&element);
		wcscpy(attedee, element);
		wcscpy(attedee, TEXT(" "));
		pRecipient->get_Address(&element);
		wcscpy(attedee, element);

		vo->addProperty(new VProperty(TEXT("ATTENDEE"), attedee));

		delete [] attedee; attedee = NULL;
		pRecipient->Release();
	}

	pRecipients->Release();

	VARIANT_BOOL bReminder;
	pAppointment->get_ReminderSet(&bReminder);

	if(bReminder == VARIANT_TRUE) {
		long minBefore;
		pAppointment->get_ReminderMinutesBeforeStart(&minBefore);
		double minStartDate = startdate * 1440;
		//subtract the alarm 
		minStartDate -= minBefore;

		dateValue = new wchar_t[30];
		doubleToSystemTime(dateValue, minStartDate/1440);
		
		wcscat(dateValue, TEXT(";;0;reminder"));
		vo->addProperty(new VProperty(TEXT("AALARM"), dateValue));
		delete [] dateValue; dateValue = NULL;
	}

	//recurrence
	VARIANT_BOOL isRec;
	pAppointment->get_IsRecurring(&isRec);

	if(isRec == VARIANT_TRUE) {
		IRecurrencePattern* pRecPat;
		pAppointment->GetRecurrencePattern(& pRecPat);
		wchar_t* recRule = NULL;
		recRule = extractRrule(pRecPat);
		if(recRule)
			vo->addProperty(new VProperty(TEXT("RRULE"), recRule));
		pRecPat->Release();
	}	
	
	vo->addProperty(new VProperty(TEXT("END"), TEXT("VEVENT")));
	vo->addProperty(new VProperty(TEXT("END"), TEXT("VCALENDAR")));
	
	return vo; 
}

void VObjectToAppointment(IAppointment *pAppointment, VObject *vo) {

	int i = 0;

	pAppointment->ClearRecurrencePattern ();

	if(vo->containsProperty(TEXT("SUMMARY")) && vo->getProperty(TEXT("SUMMARY"))->getValue()) 
		pAppointment->put_Subject(vo->getProperty(TEXT("SUMMARY"))->getValue());

	if(vo->containsProperty(TEXT("LOCATION")) && vo->getProperty(TEXT("LOCATION"))->getValue()) 
		pAppointment->put_Location(vo->getProperty(TEXT("LOCATION"))->getValue());

	if(vo->containsProperty(TEXT("DESCRIPTION")) && vo->getProperty(TEXT("DESCRIPTION"))->getValue()) 
		pAppointment->put_Body(vo->getProperty(TEXT("DESCRIPTION"))->getValue());
	
	DATE startdate = NULL;
    DATE enddate = NULL;

	if(vo->containsProperty(TEXT("DTSTART")) && vo->getProperty(TEXT("DTSTART"))->getValue()) {
		
		systemTimeToDouble(vo->getProperty(TEXT("DTSTART"))->getValue(), &startdate, NULL);
		pAppointment->put_Start(startdate);
		}

	if(vo->containsProperty(TEXT("DTEND")) && vo->getProperty(TEXT("DTEND"))->getValue()) {
		
		systemTimeToDouble(vo->getProperty(TEXT("DTEND"))->getValue(), &enddate, NULL);
		pAppointment->put_End(enddate);
		}

	BOOL isAllDay = isAllDayEvent(startdate, enddate);

	if(isAllDay)
		pAppointment->put_AllDayEvent(VARIANT_TRUE);
	else
		pAppointment->put_AllDayEvent(VARIANT_FALSE);

	if(vo->containsProperty(TEXT("CATEGORIES")) && vo->getProperty(TEXT("CATEGORIES"))->getValue()) 
		pAppointment->put_Categories(vo->getProperty(TEXT("CATEGORIES"))->getValue());


	if(vo->containsProperty(TEXT("CLASS")) && vo->getProperty(TEXT("CLASS"))->getValue()) {
		wchar_t* calClass = new wchar_t[20];
		wcscpy(calClass, vo->getProperty(TEXT("CLASS"))->getValue());
		if(!wcscmp(calClass, TEXT("PRIVATE")) || !wcscmp(calClass, TEXT("CONFIDENTIAL")))
			pAppointment->put_Sensitivity(olPrivate);
		else
			pAppointment->put_Sensitivity(olNormal);

		delete [] calClass;
	}
	else
		pAppointment->put_Sensitivity(olNormal);

	//ATTENDEE;ROLE=OWNER;STATUS=CONFIRMED:John Smith <jsmith@host1.com>
	//ATTENDEE;ROLE=ATTENDEE;STATUS=CONFIRMED;VALUE=URL;TYPE=VCARD:http://www.xyz.com/~myvcard.vcf
	IRecipients* pRecipients;
	pAppointment->get_Recipients(&pRecipients);
	int countRecipient = 0;
	pRecipients->get_Count(&countRecipient);
	
	for (i = 0; i < countRecipient; i++) {			
		pRecipients->Remove(i + 1);
	}
	
	for(i = 0; i<vo->propertiesCount(); i++) 
		if(!wcscmp(vo->getProperty(i)->getName(), TEXT("ATTENDEE")) &&
			vo->getProperty(i)->getValue() &&
			!vo->getProperty(i)->containsParameter(TEXT("VALUE"))) {

			wchar_t* attendee = new wchar_t[200];
			wcscpy(attendee, vo->getProperty(i)->getValue());

			wchar_t separator[] = TEXT("<");
			wchar_t* pos = NULL;

			pos = wcsstr(attendee, separator);

			if(pos) {
				wchar_t* name = new wchar_t[100];
				wcscpy(name, TEXT(""));
				if((pos - attendee)-1 > 0) {
					wcsncpy(name, attendee, (pos - attendee)-1);
					name[pos - attendee] = TEXT('\0');
				}

				wchar_t* addr = new wchar_t[100];
				wcscpy(addr, pos+1);
				addr[wcslen(addr) - 1] = TEXT('\0');
				
				IRecipient* pRecipient;
				pRecipients->Add(name, &pRecipient);
				pRecipient->put_Address(addr);
				pRecipient->Release();

				delete [] name;
				delete [] addr;
			}			
		}

	pRecipients->Release();

	//alarm
	//The value consists of: Run Time, Snooze Time; Repeat Count, Audio Content
	if(vo->containsProperty(TEXT("AALARM")) && vo->getProperty(TEXT("AALARM"))->getValue()) {
		
		DATE runTime;
		systemTimeToDouble(vo->getProperty(TEXT("AALARM"))->getPropComponent(1), &runTime, NULL);

		long minBeforeEvent = long((startdate * 86400 - runTime * 86400) / 60);

		pAppointment->put_ReminderSet(VARIANT_TRUE);
		pAppointment->put_ReminderMinutesBeforeStart(minBeforeEvent);

	}

	//recurrence 
	if((vo->containsProperty(TEXT("RRULE")) && vo->getProperty(TEXT("RRULE"))->getValue())) {

		pAppointment->ClearRecurrencePattern();
		
		IRecurrencePattern* pRecPat;
		pAppointment->GetRecurrencePattern(& pRecPat);
		
		
		DATE startDate;
		pAppointment->get_Start(&startDate);
		bool ret = convertRrule(pRecPat, vo->getProperty(TEXT("RRULE"))->getValue(), startDate);

		if(!ret)
			pAppointment->ClearRecurrencePattern();
		
		pRecPat->Release();
	
	}
	
}
