/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//
#include <windows.h>
#include "Winbase.h"
#include "pim/Utils.h"
#include "base/util/utils.h"
#include "pim/ActionLog.h"
#include "pim/FILEFileManagement.h"
#include "pim/SettingFunctions.h"

///// **********************************
//#include "RichInk.h"
// Defines and typedefs needed for the conversion:

#define EM_STREAMIN          (WM_USER + 73)
#define EM_STREAMOUT         (WM_USER + 74)

typedef DWORD (CALLBACK *EDITSTREAMCALLBACK)
      (DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb);

typedef struct _editstream
{
    DWORD dwCookie; /* user value passed to callback as first parameter */
    DWORD dwError;  /* last error */
    EDITSTREAMCALLBACK pfnCallback;
} EDITSTREAM;

typedef struct tagCOOKIE
{
    HANDLE hFile;
    LPBYTE pbStart;
    LPBYTE pbCur;
    LONG  bCount;
    DWORD dwError;
} COOKIE, * PCOOKIE;

// Stream formats:
#define SF_TEXT         0x0001
#define SF_RTF          0x0002
#define SF_UNICODE      0x0010      // Unicode data of some kind
#define SF_UTEXT        SF_TEXT | SF_UNICODE   // Unicode text file
#define SFF_PWI         0x0800      
#define SF_PWI          ( SF_RTF | SFF_PWI | 0x010000 ) // PWI format

static int recursiveLevel = 1;
static int currentLevel   = 0;


static DWORD CALLBACK ReadCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb) {

    PCOOKIE pCookie = (PCOOKIE)dwCookie;
    // Zero the bytes read.

    *pcb = 0;
    
    // If there is an error return it.
    if (ReadFile(pCookie->hFile, pbBuff, cb, (unsigned long *)pcb, 0) == 0)

    return (pCookie->dwError = GetLastError());
  
    // Return no error.
    return 0;

} // ReadCallback()

static DWORD CALLBACK  WriteCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb) {
    PCOOKIE pCookie = (PCOOKIE)dwCookie;
    if (WriteFile(pCookie->hFile, pbBuff, cb,(unsigned long *) pcb, 0) == 0) {
        if (dbg)
            MessageBox (NULL, TEXT("Errore in writeCallback"), TEXT ("lunghezza"), MB_SETFOREGROUND |MB_OK);	
        return (pCookie->dwError = GetLastError()); 
    } else {
        if (dbg)
            MessageBox (NULL, TEXT("No Errore in writeCallback"), TEXT ("lunghezza"), MB_SETFOREGROUND |MB_OK);	
    }

    return 0;
} // WriteCallback()


static DWORD CALLBACK  BufferReadCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
{
    PCOOKIE pCookie = (PCOOKIE)dwCookie;
    LONG   bytesLeft;
    LONG   bytesRead;

    // Calc the bytes read.
    bytesRead = pCookie->pbCur - pCookie->pbStart;
    
    // Calc bytes left to read.
    if (bytesRead < pCookie->bCount) {
      // Calc the bytes left to read.
        bytesLeft = pCookie->bCount - bytesRead;
    } else{
        bytesLeft = 0;
    }
    
    if (cb > bytesLeft) 
        cb = bytesLeft;    
    
    *pcb = cb;
    // Copy any bytes.
    if (cb) {
        memcpy(pbBuff, pCookie->pbCur, cb);
        pCookie->pbCur += cb;
    }

    return 0;
} // BufferReadCallback()

static DWORD CALLBACK BufferWriteCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG *pcb)
{   
    PCOOKIE pCookie = (PCOOKIE)dwCookie;
    // Do not overwrite the end of the buffer.
    // If there is no output buffer, then were
    // only here to determine the space required
    // for the stream out.

    if (pCookie->pbStart && (pCookie->pbCur + cb > pCookie->pbStart + pCookie->bCount)) {
        // Writing all this data would overflow the buffer.
        // So only write what will fit.
        cb = pCookie->pbStart + pCookie->bCount - pCookie->pbCur;
    }

    *pcb = cb;
    
    if (pCookie->pbStart)
        memcpy(pCookie->pbCur, pbBuff, cb); pCookie->pbCur += cb;
    return 0;
} // BufferWriteCallback()




int writeTextToFile(wchar_t* filename, wchar_t* text) {   
       
    HINSTANCE   hLib;
    HWND   hwndInk = NULL;
    LPTSTR lpszClass = TEXT("richink") ;
    EDITSTREAM   es;
    COOKIE      cookie;
    HLOCAL sz;
    UINT cbSz;
    
    hLib = LoadLibrary( TEXT("richink.dll") );
    if (!hLib)
      return -1;

    hwndInk = CreateWindow(lpszClass, NULL, WS_BORDER, 0, 0, 0, 0, NULL, 0, NULL, NULL);
    if (!hwndInk)
      return -1;    
    
    
/* write back to note */    

    HANDLE file = CreateFile(filename, GENERIC_WRITE, 0, //FILE_SHARE_READ | FILE_SHARE_WRITE,
                              NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);   

    memset(&cookie, 0, sizeof(cookie));
    cookie.dwError      = 0;
    cookie.pbStart      = (LPBYTE)text;
    cookie.pbCur        = cookie.pbStart;
    cookie.bCount       =  wcslen(text) * sizeof(wchar_t);
    // cookie.bCount   =  5 * sizeof(wchar_t);
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = BufferReadCallback;       
    SendMessage (hwndInk, EM_STREAMIN, (WPARAM)SF_UTEXT, (LPARAM)&es);
      
    cookie.dwError      = 0;
    cookie.pbStart      = 0;
    cookie.pbCur        = 0;
    cookie.bCount       = 0;
    cookie.hFile        = file;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = BufferWriteCallback;
    SendMessage (hwndInk, EM_STREAMOUT, (WPARAM)SF_PWI, (LPARAM)&es);   
         
    sz = (LPWSTR)LocalAlloc(LPTR, (cbSz = cookie.pbCur - cookie.pbStart) + 2);     

    cookie.dwError      = 0;
    cookie.pbStart      = (LPBYTE)sz;
    cookie.pbCur        = cookie.pbStart;
    cookie.bCount       = cbSz;
    //cookie.hFile = file;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = WriteCallback;
    SendMessage (hwndInk, EM_STREAMOUT, (WPARAM)SF_PWI, (LPARAM)&es);
     
    CloseHandle(file);
    
    return 0;
}


wchar_t* readTextFromArray(byte* ptrFileContent, int numBytes) {      
    
    HINSTANCE   hLib;
    HWND        hwndInk = NULL;
    LPTSTR      lpszClass = TEXT("richink") ;
    EDITSTREAM  es;
    COOKIE      cookie;
    HLOCAL      sz;
    UINT        cbSz;
    wchar_t*    ret = NULL;

    hLib = LoadLibrary( TEXT("richink.dll") );
    if (!hLib)
      return ERROR;
    
    hwndInk = CreateWindow(lpszClass, NULL, WS_BORDER, 0,
       0, 0, 0, NULL, 0, NULL, NULL);
    if (!hwndInk)
      return ERROR;    

    // Stream in the Ink Data from a memory block.
    memset(&cookie, 0, sizeof(cookie));
    cookie.dwError      = 0;
    cookie.pbStart      = ptrFileContent;// (LPBYTE)pb;      // Pointer to memory block
    cookie.pbCur        = cookie.pbStart;
    cookie.bCount       = numBytes;// cb;         // Size of ink data
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = BufferReadCallback;
    SendMessage (hwndInk, EM_STREAMIN, (WPARAM)SF_PWI, (LPARAM)&es);
    
    // Next call stream out to get the size of the Unicode text.
    cookie.dwError      = 0;
    cookie.pbStart      = 0;
    cookie.pbCur        = 0;
    cookie.bCount       = 0;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = BufferWriteCallback;
    SendMessage (hwndInk, EM_STREAMOUT, (WPARAM)SF_TEXT, (LPARAM)&es);

    
    // Allocate memory and stream out the Unicode text to it. 
    sz = (LPWSTR)LocalAlloc(LPTR, (cbSz = cookie.pbCur - cookie.pbStart) + 2);
    
    // Check for alloc err.
    cookie.dwError      = 0;
    cookie.pbStart      = (LPBYTE)sz;
    cookie.pbCur        = cookie.pbStart;
    cookie.bCount       = cbSz;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = BufferWriteCallback;
    SendMessage (hwndInk, EM_STREAMOUT, (WPARAM)SF_TEXT, (LPARAM)&es);    
       
        
    // ret = char2wchar((char*)sz);
    ret = utf82wc((char*)sz);

    if (dbg)
        MessageBox(NULL, ret, TEXT("ret from read text from array"),MB_OK);
    
    return ret;
  
}

/*
* Return the content of a binary (pwi as notes format) or text file.
* It reads contents directly from file in filesystem if exists
*/


wchar_t* readTextFromBinaryFile(wchar_t* filename) {   
       
    HINSTANCE   hLib;
    HWND   hwndInk = NULL;
    LPTSTR lpszClass = TEXT("richink") ;
    EDITSTREAM   es;
    COOKIE      cookie;
    HLOCAL sz;
    UINT cbSz;
    wchar_t* ret = NULL;

    // open file if only if exists
    HANDLE hfile = CreateFile(filename, GENERIC_READ, 0, 
                              NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    
    hLib = LoadLibrary( TEXT("richink.dll") );
    if (!hLib)
      return NULL;

    hwndInk = CreateWindow(lpszClass, NULL, WS_BORDER, 0, 0, 0, 0, NULL, 0, NULL, NULL);
    
    if (!hwndInk)
      return NULL;    

    // Stream in the Ink Data from a memory block.
    memset (&cookie, 0, sizeof(cookie));
    cookie.dwError      = 0;
    cookie.pbStart      = 0; 
    cookie.pbCur        = cookie.pbStart;
    cookie.bCount       = 0;
    cookie.hFile        = hfile;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = ReadCallback;
    SendMessage (hwndInk, EM_STREAMIN, (WPARAM)SF_PWI, (LPARAM)&es);
    
    // Next call stream out to get the size of the Unicode text.
    cookie.dwError      = 0;
    cookie.pbStart      = 0;
    cookie.pbCur        = 0;
    cookie.bCount       = 0;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = BufferWriteCallback;    
    SendMessage (hwndInk, EM_STREAMOUT, (WPARAM)SF_TEXT, (LPARAM)&es);
    
    // Allocate memory and stream out the Unicode text to it. 
    sz = (LPWSTR)LocalAlloc(LPTR, (cbSz = cookie.pbCur - cookie.pbStart) + 2);
    
    // Check for alloc err.
    cookie.dwError      = 0;
    cookie.pbStart      = (LPBYTE)sz;
    cookie.pbCur        = cookie.pbStart;
    cookie.bCount       = cbSz;
    es.dwCookie         = (DWORD)&cookie;
    es.dwError          = 0;
    es.pfnCallback      = BufferWriteCallback;    
    SendMessage (hwndInk, EM_STREAMOUT, (WPARAM)SF_TEXT, (LPARAM)&es); 
        
    // ret = char2wchar((char*)sz);
    ret = utf82wc((char*)sz); 
   
    if (dbg)
        MessageBox(NULL, ret, TEXT("ret text from file"), MB_OK);
    CloseHandle(hfile);
    
    return ret;
}


void writeTextToTextFile(wchar_t* fName, wchar_t* text) {
    
    FILE* f;
	
    // specific for favorites. If file doesn't end with .url, it is a directory. So create it.
    if (wcsstr(fName, URL_EXTENSION) == NULL) {
        if (dbg)
            MessageBox(NULL, TEXT("createDirectory"), TEXT("writeTextToTextFile"), MB_OK);
        if (CreateDirectory(fName, NULL) == TRUE) {
            if (dbg)
             MessageBox(NULL, TEXT("Dir correctly created"), TEXT("writeTextToTextFile"), MB_OK);
        } else {
            if (dbg)
             MessageBox(NULL, TEXT("Dir NOT correctly created"), TEXT("writeTextToTextFile"), MB_OK);
        }
        return;
    }

	f = _wfopen(fName, TEXT("w+"));
	
    if( f != NULL ) {

       fwprintf(f, text); 
       if (dbg)
           MessageBox(NULL, text, TEXT("writeTextToTextFile"), MB_OK);
    } else {
        if (dbg)
            MessageBox(NULL, TEXT("Error in writeTextToTextFile"), TEXT("writeTextToTextFile"), MB_OK);

            wchar_t localDir     [DIM_LOCAL_TEMP];
            wchar_t internalPath [DIM_LOCAL_TEMP];
            getDirName(fName, localDir);
            
            if (dbg)
                MessageBox(NULL, localDir, TEXT("writeTextToTextFile"), MB_OK);

            SHGetSpecialFolderPath(NULL, internalPath, CSIDL_FAVORITES, FALSE);
            wcscat(internalPath, TEXT("\\"));        
            wcscat(internalPath, localDir);
            if (dbg)
                MessageBox(NULL, internalPath, TEXT("writeTextToTextFile"), MB_OK);

            CreateDirectory(internalPath, NULL);

            f = _wfopen(fName, TEXT("w+"));
            fwprintf(f, text); 

            if (dbg)
                MessageBox(NULL, text, TEXT("writeTextToTextFile"), MB_OK);

    }
   
	fflush(f);
	fclose(f);

}


void writeByteToFile(wchar_t* fName, byte *ptr, int numBytes) {
    
    wchar_t wfilename[DIM_FILE];
	wchar_t* pt = NULL;
	char filename [DIM_FILE];
	BOOL usedDefaultChar;

	wsprintf(wfilename, fName);	

	if (dbg) {		
		MessageBox (NULL, wfilename, TEXT ("pathFile"), MB_SETFOREGROUND |MB_OK);	
	}
	
	WideCharToMultiByte (CP_ACP, 0, wfilename, DIM_FILE -1, filename, DIM_FILE -1, "?", &usedDefaultChar);

    int numwritten = 0;    
    FILE* f;
	wchar_t tmp[DIMLINE];
		
	f = fopen(filename, "w+b");
	
    if( f != NULL ) {

        numwritten = fwrite( ptr, sizeof( byte ), numBytes, f );
        
        if (dbg) {	
            wsprintf(tmp, TEXT("Number of items written = %d"), numwritten);
		    MessageBox (NULL, tmp, TEXT ("numwritten"), MB_SETFOREGROUND |MB_OK);	

	    }
        
    }
   
	fflush(f);
	fclose(f);


}

void readByteFromFile(wchar_t* fName, byte *ptr, int* numBytes) {
	
	wchar_t wfilename[DIM_FILE];
	wchar_t* pt = NULL;
	char filename [DIM_FILE];
	BOOL usedDefaultChar;

	wsprintf(wfilename, fName);	

	if (dbg) {		
		MessageBox (NULL, wfilename, TEXT ("path read"), MB_SETFOREGROUND |MB_OK);	
	}
	
	WideCharToMultiByte (CP_ACP, 0, wfilename, DIM_FILE -1, filename, DIM_FILE -1, "?", &usedDefaultChar);

	wchar_t* element = NULL;

	byte line[DIMLINE];    
    int numread = 0;
    int total = 0;
    FILE* f;
	wchar_t tmp[DIMLINE];
	
	f = fopen(filename, "r+b");
	
    while( !feof( f ) ) {

        numread = fread( line, sizeof( byte ), DIMLINE, f ); 
                
        memcpy(ptr + total, line, numread);
        total = total + numread;
    }
    if (dbg) {	
            wsprintf(tmp, TEXT("Number of items total read = %d"), total);
		    MessageBox (NULL, tmp, TEXT ("numReadTotal"), MB_SETFOREGROUND |MB_OK);	

	}
	fflush(f);
	fclose(f);
    
    *numBytes = total;
    	
    if (dbg) {	
        // element = char2wchar((char*)ptr);
        element = utf82wc((char*)ptr);
		MessageBox (NULL, element, TEXT ("FileContent"), MB_SETFOREGROUND |MB_OK);	
        delete element;

	}        

}

/*
* Method to write current file name and time of writing in file to use to
* compare with others in order to search for modified files...
*/


void writeCurrentFileItems(int dataType, wchar_t* path, wchar_t* dir) {
    
    wchar_t localTemp[DIM_LOCAL_TEMP];
    wchar_t wfilename[DIM_FILE];	
	wchar_t internalPath[DIM_LOCAL_TEMP];
    wchar_t filterPattern[256];

    wchar_t** fileList = NULL;
    int count = 0;
    int i = 0;
    FILE* f;	
    
    BOOL recursiveInDirectory = FALSE;
    filterPattern [0] = 0;

    if (dbg)
		MessageBox (NULL, TEXT("in writeCurrentFileItems"), TEXT ("FileContent"), MB_SETFOREGROUND |MB_OK);	       
        
    if (dataType == constFiles) {
        wsprintf(internalPath, TEXT("%s%s"), dir, TEXT("\\"));
        wsprintf(wfilename, TEXT("%s%s"), path, TEXT(FILENAME_FILES));
    }
    else if (dataType == constNotes) {
        wsprintf(internalPath, TEXT("%s%s"), dir, TEXT("\\"));
        wsprintf(wfilename, TEXT("%s%s"), path, TEXT(FILENAME_NOTES));
        wsprintf(filterPattern, PWI_EXTENSION_PATTERN);
    }
    else if (dataType == constFavorites) {
        SHGetSpecialFolderPath(NULL, localTemp, CSIDL_FAVORITES, FALSE);
        wsprintf(internalPath, TEXT("%s%s"), localTemp, TEXT("\\"));        
        wsprintf(wfilename, TEXT("%s%s"), path, TEXT(FILENAME_FAVORITES));
        recursiveInDirectory = TRUE;
    }
                	
    f = _wfopen(wfilename, TEXT("w+"));

    getFileList(internalPath, fileList, &count, TRUE, recursiveInDirectory, filterPattern);    
    fileList = new wchar_t*[count]; 
    int m = 0;
    for (m = 0; m < count ; m++) {
        fileList[m] = NULL; 
    }
    // to retrieve fileName
    getFileList(internalPath, fileList, &count, FALSE, recursiveInDirectory, filterPattern);   
    
    wsprintf(localTemp, TEXT("%f\n"), getSystemTime());
    fwprintf(f, localTemp);
    
    for (i = 0; i < count; i++) {
        
        fwprintf(f, fileList[i]); 
        fwprintf(f, TEXT("\n")); 
        if (dbg)
            MessageBox (NULL, fileList[i], TEXT ("FileName"), MB_SETFOREGROUND |MB_OK);	
         

    }        
    fflush(f);
	fclose(f);
    
     
    for (i = 0; i < count; i++) {
        if (fileList[i] != NULL)
            delete fileList[i]; 
    }
    
    if (fileList != NULL)
        delete [] fileList;
    
	
}

int readFilenameFromFile(int dataType, wchar_t* path, wchar_t*** ptrArrayFilename) {

	wchar_t wfilename [DIM_FILE];
    wchar_t pt        [DIM_FILE];
    wchar_t** ptr     = NULL;
	int i             = 0;

	if (dataType == constFiles) 
        wsprintf(pt, TEXT(FILENAME_FILES));	
    else if (dataType == constNotes) 
        wsprintf(pt, TEXT(FILENAME_NOTES));	
    else if (dataType == constFavorites) 
        wsprintf(pt, TEXT(FILENAME_FAVORITES));	

	wsprintf(wfilename, TEXT("%s%s"),  path, pt);
	
	if (dbg) {
		MessageBox (NULL, path, TEXT ("readFilenameFromFile"), MB_SETFOREGROUND |MB_OK);
		MessageBox (NULL, wfilename, TEXT ("readFilenameFromFile"), MB_SETFOREGROUND |MB_OK);	
	}	

	wchar_t* element = NULL;
	wchar_t line[256];
	FILE* f;
	wchar_t* tmp = NULL;

	f = _wfopen(wfilename, TEXT("r"));
	
    if (f == NULL) {        
        return 0;

    }

	while(fgetws(line, 255, f) != NULL) {
	    i++;		
	}
	fflush(f);
    
    if (dbg) {
		wsprintf(pt, TEXT("%i"), i);
		MessageBox (NULL, pt, TEXT ("readFilenameFromFile"), MB_SETFOREGROUND |MB_OK);	
	}	
   
    ptr = new wchar_t* [i];
    
    i = 0;
    f = _wfreopen(wfilename, TEXT("r"), f);
    if (dbg) {
	
		MessageBox (NULL, TEXT("dopo reopen"), TEXT ("readFilenameFromFile"), MB_SETFOREGROUND |MB_OK);	
	}	

    while(fgetws(line, 255, f) != NULL) {
	   
        ptr[i] = new wchar_t[255];
        wsprintf(ptr[i], line);
        
        //
        // to remove carriage/return at the end of file. CarriageReturn was introduced
        // by method writeCurrentFileItems to insert new line
        //
        removeEndCarriage(&ptr[i]);

        if (dbg) {	
		    MessageBox (NULL, ptr[i], TEXT ("readFilenameFromFile"), MB_SETFOREGROUND |MB_OK);	
	    }	
        i++;
	}
    fflush(f);
	fclose(f);
    
    *ptrArrayFilename = ptr;
    return i - 1;
}
/*
* get file size
*/

BOOL getFileSize(wchar_t* filename, unsigned long *size) {


   HANDLE file = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
                              NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

   DWORD lpFileSizeHigh;
   DWORD dwSize;
      
   if( file ) {
        dwSize = GetFileSize(file, &lpFileSizeHigh);    
   
        if (dwSize != 0xFFFFFFFF) {
            if (dbg) {	
     	        MessageBox (NULL, TEXT("size ok"), TEXT ("lunghezza"), MB_SETFOREGROUND |MB_OK);	
            }
            *size = dwSize;

        }
        else {
            if (dbg) {	
     	        MessageBox (NULL, TEXT("size not ok"), TEXT ("lunghezza"), MB_SETFOREGROUND |MB_OK);	
            }
            *size = 0;
        }
   
        if (dbg) {	
            wchar_t tmp[100];
            wsprintf(tmp, TEXT("lunghezza dwSize= %d"), dwSize);
		    MessageBox (NULL, tmp, TEXT ("lunghezza"), MB_SETFOREGROUND |MB_OK);	

	    }   

   }
   else
   {
       if (dbg)
		    MessageBox (NULL, TEXT("Error in handle getFileSize..."), TEXT ("Size Error"), MB_SETFOREGROUND |MB_OK);
   }
   if( file ) CloseHandle( file );

   return TRUE;
}

/*
* 
*/

BOOL getLastWriteTime(wchar_t* filename, wchar_t* wtime, DATE* dtime) {


    HANDLE file = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
                              NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    // FILETIME time;

    FILETIME ftCreate, ftAccess, ftWrite, stUTC;
    SYSTEMTIME  stLocal;
    BOOL isUTC = TRUE;

    //wchar_t propertyValue[10];
    //getClientConfigurationInternal (NULL, PROPERTY_SOURCEPROP_SPDM_SYSTEMTIME_TO_LOCALTIME, propertyValue, FALSE); 
    
    //if (wcscmp(propertyValue, TEXT("1")) == 0) {
    //    isUTC = TRUE;
    //}

    if( file ) {
        if (dbg)
		    MessageBox (NULL, TEXT("OK in Handle Time file..."), TEXT ("Time"), MB_SETFOREGROUND |MB_OK);
        
        if (!GetFileTime(file, &ftCreate, &ftAccess, &ftWrite)) {
            if (dbg)
		        MessageBox (NULL, TEXT("Error GetFileTime..."), TEXT ("Time"), MB_SETFOREGROUND |MB_OK);
            return FALSE;
        }
        else {
            if (dbg)
		        MessageBox (NULL, TEXT("GetFileTime correct!"), TEXT ("Time"), MB_SETFOREGROUND |MB_OK);
        }                        

        if (isUTC) {
            FileTimeToSystemTime(&ftWrite, &stLocal);            
        }
        else {
            FileTimeToLocalFileTime(&ftWrite, &stUTC);                    
            FileTimeToSystemTime   (&stUTC, &stLocal);                
        }
        
        wchar_t month [10];
        wchar_t day   [10];
        wchar_t hour  [10];
        wchar_t minute[10];
        wchar_t second[10];
    
        // month
        if (stLocal.wMonth < 10)
            wsprintf(month, TEXT("0%i"), stLocal.wMonth);
        else
            wsprintf(month, TEXT("%i"), stLocal.wMonth);
    
        // day
        if (stLocal.wDay  < 10)
            wsprintf(day, TEXT("0%i"), stLocal.wDay);
        else
            wsprintf(day, TEXT("%i"), stLocal.wDay);
    
        // hour
        if (stLocal.wHour  < 10)
            wsprintf(hour, TEXT("0%i"), stLocal.wHour);
        else
            wsprintf(hour, TEXT("%i"), stLocal.wHour);

        // minute
        if (stLocal.wMinute < 10)
            wsprintf(minute, TEXT("0%i"), stLocal.wMinute);
        else
            wsprintf(minute, TEXT("%i"), stLocal.wMinute);

        // second
        if (stLocal.wSecond < 10)
            wsprintf(second, TEXT("0%i"), stLocal.wSecond);
        else
            wsprintf(second, TEXT("%i"), stLocal.wSecond);

        
        if (isUTC) {
            wsprintf(wtime, TEXT("%i%s%sT%s%s%sZ"), stLocal.wYear, month, day, hour, minute, second);
        } else {
            wsprintf(wtime, TEXT("%i%s%sT%s%s%s"), stLocal.wYear, month, day, hour, minute, second);
        }                       
    
        if (dbg)
		    MessageBox (NULL, wtime, TEXT ("WTime"), MB_SETFOREGROUND |MB_OK);   
        
        if (dtime != NULL)
            systemTimeToDouble(wtime, dtime, NULL);
   
   }
   else {
       
       if (dbg)
		    MessageBox (NULL, TEXT("Error in Handle Time file..."), TEXT ("Time Error"), MB_SETFOREGROUND |MB_OK);
   }
   if( file ) CloseHandle( file );

   return TRUE;
}

BOOL getFileList(wchar_t *dir, wchar_t** ptrArray, int *count, BOOL onlyFilesNumber, 
                 BOOL isRecursiveInDirectory) {

    return getFileList(dir, ptrArray, count, onlyFilesNumber, isRecursiveInDirectory, NULL);
}

/*
* method to retrieve number or filename directory dir.
* it may be recursive if isRecursiveDirectory is TRUE.
* if onlyFilesNumber is TRUE, count contains the number of file to process
*/

BOOL getFileList(wchar_t *dir, wchar_t** ptrArray, int *count, BOOL onlyFilesNumber, 
                 BOOL isRecursiveInDirectory, wchar_t* filter) {

    WIN32_FIND_DATA FileData;
    
    HANDLE hFind;
    wchar_t localTemp [512];
    wchar_t toFind    [512];
    wchar_t szNewPath [512];
    wchar_t filterPattern [256];
    
    BOOL ret = TRUE;
    szNewPath[0] =   0;
    int lung = 0;
    
    DWORD dwAttrs;

    BOOL fFinished = FALSE;
    
    
    if (filter == NULL || wcslen(filter) == 0)
        wsprintf(filterPattern, TEXT("*.*"));
    else
        wsprintf(filterPattern, filter);


    wsprintf(toFind, TEXT("%s%s"), dir, filterPattern);
    if (dbg)
        MessageBox (NULL, toFind, TEXT ("getFileList"), MB_SETFOREGROUND |MB_OK);

    hFind = FindFirstFile(toFind, &FileData);
    
    int i = 0;

    if (hFind == INVALID_HANDLE_VALUE)
    {
        if (dbg)
		    MessageBox (NULL, TEXT("Invalid handle for retrieve files"), TEXT ("RetrieveFiles"), MB_SETFOREGROUND |MB_OK);
        return FALSE;
    }
    else
    {   
        if (onlyFilesNumber == TRUE)  {
            //only to retrieve number of files
		    while (!fFinished)
		    {	
                wsprintf(localTemp, TEXT("%s%s"), dir, FileData.cFileName);
                
                dwAttrs = GetFileAttributes(localTemp);
                
                // if (dwAttrs & FILE_ATTRIBUTE_DIRECTORY) {
                if (dwAttrs == FILE_ATTRIBUTE_DIRECTORY) {
                    
                    if (isRecursiveInDirectory == TRUE && currentLevel < recursiveLevel) {
                        
                        currentLevel++;

                        wsprintf(localTemp, TEXT("%s%s%s"), dir, FileData.cFileName, TEXT("\\") );
  
                        int k = 0;
                        getFileList(localTemp, NULL, &k, TRUE, isRecursiveInDirectory, filterPattern);                              
                        i = i + k + 1; // + 1 is to count directory too!!

                        currentLevel--;
                                          
                    }
                    
                }    
                else {
                    i++;                                        
                }
			    if (!FindNextFile(hFind, &FileData))
			    {
				    if (GetLastError() == ERROR_NO_MORE_FILES)	 {
					    fFinished = TRUE;
				    } else {   
                        if (dbg)
                            MessageBox (NULL, TEXT("Couldn't find next file."), TEXT ("RetrieveFiles"), MB_SETFOREGROUND |MB_OK);					   
					    return FALSE;
				    }
			    }
		    }
            if (dbg) {
                wsprintf(localTemp, TEXT("%i"), i);
                MessageBox (NULL, localTemp, TEXT ("Totale number of files"), MB_SETFOREGROUND |MB_OK);
            }
            *count = i;
        }        

        else {

            while (!fFinished)
		    {
                
                wsprintf(szNewPath, TEXT("%s%s"), dir, FileData.cFileName);
			    
                dwAttrs = GetFileAttributes(szNewPath);
                
                if (dwAttrs == FILE_ATTRIBUTE_DIRECTORY) {
                   
                    if (isRecursiveInDirectory == TRUE && currentLevel < recursiveLevel) {
                        
                        currentLevel++;

                        wsprintf(localTemp, TEXT("%s%s%s"), dir, FileData.cFileName, TEXT("\\") );                                           
                                            
                        int j = 0;
                        ptrArray[i] = new wchar_t[256];
                        wsprintf(ptrArray[i], localTemp);
                        i++;
                        getFileList(localTemp, &ptrArray[i], &j, FALSE, isRecursiveInDirectory);
                        i = i + j;

                        currentLevel--;
                        
                    }
                   
                   
                }    
                else {
                    ptrArray[i] = new wchar_t[256];
                    wsprintf(ptrArray[i], szNewPath);
                    i++;
                }     
                
                wsprintf(szNewPath,TEXT(""));
                
			    
                if (!FindNextFile(hFind, &FileData))
		            {
				    if (GetLastError() == ERROR_NO_MORE_FILES)
				    {

					    fFinished = TRUE;
				    }
				    else
				    {
					    if (dbg)
                            MessageBox (NULL, TEXT("Couldn't find next file."), TEXT ("RetrieveFiles"), MB_SETFOREGROUND |MB_OK);	
					    return FALSE;
				    }
			    }
                *count = i;
		    }
        }

    // Close the search handle.

	FindClose(hFind);
    }
    return ret;

}

/*
* The key can be a file name so it can contain some special char to be encoded.
* The method get an array of wchar and overwrite the current key with the encoded one.
*/
void encodeKey(wchar_t* key) {
    
    if (IS_KEY_BASE64 == FALSE)
        return;

    wchar_t* encoded = NULL;
    encoded = encodeBase64UTF8(key);
    
    wsprintf(key, encoded);

    if (encoded != NULL)
        delete encoded;

}

/*
* The key can be a file name so it can contain some special char to be encoded.
* The method get an array of wchar and overwrite the current key with the encoded one.
*/
void decodeKey(wchar_t* key) {
    
    if (IS_KEY_BASE64 == FALSE)
        return;

    wchar_t* decoded = NULL;
    decoded = decodeBase64WUTF8(key);
    
    wsprintf(key, decoded);
    
    if (decoded != NULL)
        delete decoded;

}
