/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pim/FileObjectSyncSource.h"
#include "spds/SyncItemStatus.h"
#include "base/util/utils.h"
#include "base/Log.h"
#include "pim/FILEClient2Server.h"
#include "pim/FILEServer2Client.h"
#include "pim/FILEFileManagement.h"

static int call;
static int cnew;
static int ckey;
static int cupdated;
static int cdeleted;

FileObjectSyncSource::FileObjectSyncSource(const wchar_t* name) : SyncSource(name) {
    c = NULL;    
    setPath(TEXT("\\"));
}

FileObjectSyncSource::~FileObjectSyncSource() {
    if (c) 
        delete c; 
    c = NULL;
}

void FileObjectSyncSource::setType(SourceType t) {
    type = t;
}

SourceType FileObjectSyncSource::getType() {
    return type;
}

void FileObjectSyncSource::setPath(wchar_t* p) {
    if (p == NULL)
        path = stringdup(TEXT(""));
    else
        path = stringdup(p);
}

wchar_t* FileObjectSyncSource::getPath() {
    return path;
}

void FileObjectSyncSource::setDir(wchar_t* p) {
    if (p == NULL)
        dir = stringdup(TEXT("\\"));
    else
        dir = stringdup(p);
}

wchar_t* FileObjectSyncSource::getDir() {
    return dir;
}
/*
* Return the first SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
*/

SyncItem* FileObjectSyncSource::getFirstItem() {
        
    if (c == NULL) {
        c = new Container();
    }

    setAllItemsFILE(c, getType(), TEXT(""), getDir()); // 
    if (c->getAllItemsSize() == 0) {
        return NULL;
    }        
    call = 0;
    ArrayList* list = c->getAllItems();    
    return (SyncItem*)list->get(call);    

}

/*
* Return the next SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
*/

SyncItem* FileObjectSyncSource::getNextItem() {        
    if (c == NULL)
        return NULL;

    if (c->getAllItemsSize() == 0)
        return NULL;

    call++;
    if (call == c->getAllItemsSize()) {
        return NULL;
    }        
    return (SyncItem*)c->getAllItems()->get(call);  
}

SyncItem* FileObjectSyncSource::getFirstItemKey() {
    if (c == NULL) {
        c = new Container();
    }

    setAllItemsFILEKey(c, getType(), TEXT(""), getDir()); // 
    if (c->getAllItemsSize() == 0) {
        return NULL;
    }        
    ckey = 0;
    ArrayList* list = c->getAllItems();    
    return (SyncItem*)list->get(ckey)->clone(); 
}

SyncItem* FileObjectSyncSource::getNextItemKey() {

    if (c == NULL)
        return NULL;

    if (c->getAllItemsSize() == 0)
        return NULL;

    ckey++;
    if (ckey == c->getAllItemsSize()) {
        return NULL;
    }        
    return (SyncItem*)c->getAllItems()->get(ckey)->clone(); 

}

/*
* This method is always called by the sync manager. So the setModifiedItemsPOOM is always called
*/

SyncItem* FileObjectSyncSource::getFirstNewItem() {
    if (c == NULL) {
        c = new Container();
    }

    setModifiedItemsFILE(c, getType(), getPath(), getDir() ); 
   
    if (c->getNewItemsSize() == 0) {
        return NULL;
    }    
    cnew = 0;
    ArrayList* list = c->getNewItems();

    return (SyncItem*)list->get(cnew);    

}

SyncItem* FileObjectSyncSource::getNextNewItem() {    
   if (c == NULL)
        return NULL;

    cnew++;
    if (cnew == c->getNewItemsSize()) {
        return NULL;
    }        
    return (SyncItem*)c->getNewItems()->get(cnew);  
}

SyncItem* FileObjectSyncSource::getFirstUpdatedItem() {
    if (c == NULL) {
        c = new Container();
    }

    if (c->getUpdatedItemsSize() == 0) {
        return NULL;
    }    
    cupdated = 0;
    ArrayList* list = c->getUpdatedItems();

    return (SyncItem*)list->get(cupdated);          
}

SyncItem* FileObjectSyncSource::getNextUpdatedItem() {    
    if (c == NULL)
        return NULL;

    cupdated++;
    if (cupdated == c->getUpdatedItemsSize()) {
        return NULL;
    }        
    return (SyncItem*)c->getUpdatedItems()->get(cupdated);  
}

SyncItem* FileObjectSyncSource::getFirstDeletedItem() {
   
    if (c == NULL) {
        c = new Container();
    }

    if (c->getDeletedItemsSize() == 0) {
        return NULL;
    }    
    cdeleted = 0;
    ArrayList* list = c->getDeletedItems();

    return (SyncItem*)list->get(cdeleted);      
}

SyncItem* FileObjectSyncSource::getNextDeletedItem() {    
    if (c == NULL)
        return NULL;

    cdeleted++;
    if (cupdated == c->getDeletedItemsSize()) {
        return NULL;
    }        
    return (SyncItem*)c->getDeletedItems()->get(cdeleted);    
}

void FileObjectSyncSource::setItemStatus(const wchar_t* key, int status) {
    wchar_t tmp[256];
    wsprintf(tmp, TEXT("key: %s, status: %i"), key, status);
    LOG.debug(tmp);
}

int FileObjectSyncSource::addItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    wchar_t* oid = FILEmanageNewItems(&item, getType(), getPath(), getDir());
        
    if (oid != NULL) {        
        ret = STC_ITEM_ADDED;
        wchar_t tmp[128];
        wsprintf(tmp, TEXT("Added item: %s LUID: %s"), item.getKey(), oid);
        LOG.debug(tmp);
        wsprintf(tmp, TEXT("%i"), oid);
        item.setKey(oid);   

        delete [] oid; oid = NULL;
    }

    return ret;
}

int FileObjectSyncSource::updateItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    long h = FILEmanageUpdatedItems(&item, getType(), getPath(), getDir());
    
    if (h == 0) {
        ret = STC_OK;
        wchar_t tmp[256];
        wsprintf(tmp, TEXT("updated item: %s"), item.getKey());
        LOG.debug(tmp);
    }
    return ret;
}

int FileObjectSyncSource::deleteItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    long h = FILEmanageDeletedItems(&item, getType(), getPath(), getDir());
    
    if (h == 0) {
        ret = STC_OK;
        wchar_t tmp[256];
         wsprintf(tmp, TEXT("deleted item: %s"), item.getKey());
        LOG.debug(tmp);
    }
    return ret;    
}

int FileObjectSyncSource::beginSync() {
    //
    // Nothing to do
    //
    return 0;
}

int FileObjectSyncSource::endSync() {
        
    writeCurrentFileItems(getType(), getPath(), getDir());
    return 0;
}

void FileObjectSyncSource::assign(FileObjectSyncSource& s) {
    setSyncMode(s.getSyncMode());
    setLastSync(s.getLastSync());
    setNextSync(s.getNextSync());
    setRemoteURI(s.getRemoteURI());
    setType(s.getType());
    setLastAnchor(s.getLastAnchor());
    setNextAnchor(s.getNextAnchor());
    setErrorHandler(s.getErrorHandler());
    setFilter(s.getFilter());
    setPath(getPath());
    setType(getType());
    setDir(getDir());
}