/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include <oleauto.h>
#include "initguid.h"
#include "pimstore.h"
#include <stdio.h>


#include "base/startcmd.h"
#include "pim/maincpp.h"
#include "pim/Utils.h"
#include "pim/SettingFunctions.h"
#include "client/Sync4jClient.h"
#include "pim/ContactSyncSource.h"
#include "pim/CalendarSyncSource.h"
#include "pim/NoteSyncSource.h"
#include "pim/TaskSyncSource.h"
#include "pim/BriefcaseSyncSource.h"

#include "pim/mail/MailSyncSource.h"
#include "client/MailSourceManagementNode.h"
#include "filter/SourceFilter.h"
#include "filter/ClauseUtil.h"

#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "client/SourceManagementNode.h"

/*
* Used to sync the email only. It sync alway the email without 
*/

DWORD WINAPI synchronizationMailOnly () {
    
    wchar_t MAILS       [15];
    wchar_t tmp[15];
    wsprintf(tmp, TEXT("%s"), TEXT("mail"));
    wchar_t tmp2[15];
    wcscpy(tmp2, MAILS_NAME);
    int ret = 0;
    wcscpy(MAILS,         MAILS_NAME       );

    wchar_t path [256];
    wmemset(path, 0, 256);
    SyncSource** ssArray = new SyncSource*[2];
       
    getClientConfigurationInternal (NULL, PROPERTY_SPDM_PATH_INFO, path, NULL);
    
    // mail
    MailSourceManagementNode m = MailSourceManagementNode(SOURCES_CONTEXT, TEXT("mails"));   
    MailSyncSourceConfig sc;    
    wchar_t t[12];
    sc = m.getMailSourceConfig(true);    
            
    getFolderToSync(sc, t);    

    int days = sc.getDownloadAge();
    wchar_t* downloadAge = getDayBefore(days);

    wchar_t ttt[15];
    wsprintf(ttt, TEXT("%d"), days);
    
    SourceFilter* filter = 
        ClauseUtil::createSourceFilter(downloadAge, sc.getBodySize(),sc.getAttachSize());
        
    MailSyncSource     mailSource     = MailSyncSource    (MAILS);      
    mailSource.setPath(path);
    mailSource.setFolderToSync(t);
    mailSource.setFilter(filter);
    
    ssArray[0] = &mailSource;
    ssArray[1] = NULL;
    
    Sync4jClient& s4j = Sync4jClient::getInstance();
    s4j.setDMConfig(APPLICATION_URI);    
    LogLevel ll = getLOGLevel();
    LOG.setLevel(ll);

    ret = s4j.sync(ssArray); 
    
    //
    // If there is a mail in Outbox folder, the sync is performed twice.
    //
    if (mailSource.getIsMailInOutbox() && ret == 0) {
        ret = s4j.sync(ssArray);
    }
    
    if (mailSource.getIsMailInInbox()) {
        MessageBeep(MB_ICONASTERISK);
    }

    Sync4jClient::dispose();

    return ret;
}

/*      
 * Entry point from VB application. 
 * Accept and manage Variant array of String, long and int. 
 * The entry array is a String array in which elements rapresent SyncSource name.
 * Now accepted and managed element names are "contacts" and "appointments"
 * Array contains at least one element
 * @param vbErrorMsg   : the variable to store the message to return to vb GUI interface
 * @param path       : the install path of the SyncClient PIM application on the device
 */

DWORD WINAPI synchronization (wchar_t* vbErrorMsg, wchar_t* path) {
    
    wchar_t CONTACTS    [15];
    wchar_t APPOINTMENTS[15];       
    wchar_t TASKS       [15];
    wchar_t FAVORITES   [15];
    wchar_t FILES       [15];
    wchar_t NOTES       [15];
    wchar_t MAILS       [15];    

    wchar_t errorMessage[1024];
    int rc          = 0;
    lastErrorCode   = 0;
    ArrayList syncSources;
    wchar_t* value;
    errorMessage[0] = 0;
    SyncSource** ssArray = NULL;
    int ret = 0;
    BOOL isMailToSync = FALSE;

    wsprintf(CONTACTS,      CONTACTS_NAME    );
    wsprintf(APPOINTMENTS,  APPOINTMENTS_NAME);
    wsprintf(TASKS,         TASKS_NAME       );
    wsprintf(FAVORITES,     FAVORITES_NAME   );
    wsprintf(FILES,         FILES_NAME       );
    wsprintf(NOTES,         NOTES_NAME       );
    wsprintf(MAILS,         MAILS_NAME       );

    
    
    // for remember code only...
    // int constFolderCalendar  =  9; 
    // int constFolderContacts  = 10;
    // int constFolderTasks     = 13;

    //
    // custom values
    //
    // int constFiles       = 600;
    // int constNotes       = 601;
    // int constFavorites   = 602; 
           
    // 
    // The list of the sync sources to sync
    //
    DMTree* dmt = DMTreeFactory::getDMTree(TEXT(""));		
	ManagementNode* node = NULL;
    
    // contact
    node = dmt->getManagementNode(CONTACTS_CONTEXT);
	
	value = node->getPropertyValue(TEXT("sync"));
    if (wcscmp(value, TEXT("none")) != 0) {
        ContactSyncSource  contactSource  = ContactSyncSource (CONTACTS, OL_CONTACTS);     
        
        contactSource.setPath(path);
        syncSources.add(contactSource);
    }
    if (node) { delete node ; node = NULL; }
    // calendar
    node = dmt->getManagementNode(APPOINTMENTS_CONTEXT);	
	value = node->getPropertyValue(TEXT("sync"));
    if (wcscmp(value, TEXT("none")) != 0) {
        CalendarSyncSource  s  = CalendarSyncSource (APPOINTMENTS,     OL_CALENDAR);         
        s.setPath(path);
        syncSources.add(s);
    }
    if (node) { delete node ; node = NULL; }

    // task
    node = dmt->getManagementNode(TASKS_CONTEXT);
	
	value = node->getPropertyValue(TEXT("sync"));
    if (wcscmp(value, TEXT("none")) != 0) {
        TaskSyncSource  s  = TaskSyncSource (TASKS,     OL_TASK);         
        s.setPath(path);
        syncSources.add(s);
    }
    if (node) { delete node ; node = NULL; }

     // briefcase
    node = dmt->getManagementNode(BRIEFCASE_CONTEXT);
	
	value = node->getPropertyValue(TEXT("sync"));
    if (wcscmp(value, TEXT("none")) != 0) {
        BriefcaseSyncSource  s  = BriefcaseSyncSource (FILES,     OL_BRIEFCASE);         
        s.setPath(path);
        if (node) { delete node ; node = NULL; }

        node = dmt->getManagementNode(BRIEFCASE_SOURCEPROP_CONTEXT);
        value = node->getPropertyValue(TEXT("dir"));
        s.setDir(value);
        syncSources.add(s);
    }
    if (node) { delete node ; node = NULL; }

     // note
    node = dmt->getManagementNode(NOTES_CONTEXT);
	
	value = node->getPropertyValue(TEXT("sync"));
    if (wcscmp(value, TEXT("none")) != 0) {
        NoteSyncSource  s  = NoteSyncSource (NOTES,     OL_NOTES);         
        s.setPath(path);
        if (node) { delete node ; node = NULL; }
        node = dmt->getManagementNode(NOTES_SOURCEPROP_CONTEXT);
        value = node->getPropertyValue(TEXT("dir"));
        s.setDir(value);
        syncSources.add(s);
    }
    if (node) { delete node ; node = NULL; }


    // mail
    MailSourceManagementNode m = MailSourceManagementNode(SOURCES_CONTEXT, TEXT("mails"));   
    MailSyncSourceConfig sc;    
    wchar_t t[12];
    sc = m.getMailSourceConfig(true);    
    MailSyncSource  mailSource  = MailSyncSource    (MAILS); 
    
    if (wcscmp(sc.getSync(), TEXT("none")) != 0) {
        getFolderToSync(sc, t);    
    
        int days = sc.getDownloadAge();
        wchar_t* downloadAge = getDayBefore(days);

        SourceFilter* filter = 
            ClauseUtil::createSourceFilter(downloadAge, sc.getBodySize(),sc.getAttachSize());

             
        mailSource.setPath(path);
        mailSource.setFolderToSync(t);
        mailSource.setFilter(filter);

        syncSources.add(mailSource);
        isMailToSync = TRUE;
        
    }   

    if (syncSources.size() == 0) {
      wsprintf(errorMessage, TEXT("No sources to sync"));
    
    } else {
        Sync4jClient& s4j = Sync4jClient::getInstance();
        s4j.setDMConfig(APPLICATION_URI);   
        LogLevel ll = getLOGLevel();
        LOG.setLevel(ll);

        //ret = s4j.sync(syncSources);         
        
        int l = syncSources.size();
        const int ml = 10;        
        SyncSource* array[ml];    
        for (int i=0; (i<l) && (i<ml-1); ++i) {
            array[i] = (SyncSource*)syncSources[i];
        }
        array[l] = NULL;
        ret = s4j.sync(array);
        
        //
        // If there is a mail in Outbox folder, the sync is performed twice.
        //
        if (isMailToSync && mailSource.getIsMailInOutbox() && ret == 0) {
            syncSources.clear();
            //syncSources.add(mailSource);
            //ret = s4j.sync(syncSources);
            //int l = syncSources.size();
            //const int ml = 10;        
            SyncSource** array = new SyncSource*[2];    
            
            array[0] = &mailSource;            
            array[l] = NULL;
            ret = s4j.sync(array);
        
         }
        
        if (isMailToSync && mailSource.getIsMailInInbox()) {
            MessageBeep(MB_ICONASTERISK);
        }
        Sync4jClient::dispose();
    }
    
    //
    // NOTE: when passing a string from VB to a DLL, it looks like
    // VB does not want that the last chararcter in the string is
    // the terminator. Therefore we change it to a space. However,
    // IT IS VERY IMPORTANT that the string allocated by the caller
    // less than the length of errorMessage. 
    //
    wsprintf(vbErrorMsg, errorMessage);
    vbErrorMsg[wcslen(vbErrorMsg)] = L' ';
    vbErrorMsg[wcslen(vbErrorMsg)+1] = 0;    
    LOG.info(TEXT("Sync ended."));
    
    return ret;

}

/*      
 * Entry point to kickoff the synchronization process, specifying which
 * sources to synchronize.
 *
 * @param path    : the install path of the SyncClient PIM application on 
 *                  the device
 * @param sources : NULL terminated array of strings containing the source
 *                  name of the sources to synchronize
 */

DWORD WINAPI synchronizeMultipleSources (const wchar_t* path, const wchar_t** sources) {
    
    int ret = 0;
    if (sources == 0) {
        return 0;
    }
    BOOL isMailToSync = FALSE;
    ArrayList syncSources;
    DMTree* dmt = DMTreeFactory::getDMTree(TEXT(""));		
	ManagementNode* node = NULL;
    int i=0;
    MailSyncSource     mailSource     = MailSyncSource    (MAILS_NAME); 

    while (sources[i]) {
        if (wcscmp(sources[i], CONTACTS_NAME) == 0) {
            ContactSyncSource s = ContactSyncSource (CONTACTS_NAME, OL_CONTACTS);
            s.setPath(path);
            syncSources.add(s);
        } else if (wcscmp(sources[i], APPOINTMENTS_NAME) == 0) {
            CalendarSyncSource s = CalendarSyncSource (APPOINTMENTS_NAME, OL_CALENDAR);
            s.setPath(path);
            syncSources.add(s);
        } else if (wcscmp(sources[i], TASKS_NAME)        == 0) {

            TaskSyncSource  s  = TaskSyncSource (TASKS_NAME,     OL_TASK);         
            s.setPath(path);
            syncSources.add(s);

        } else if (wcscmp(sources[i], FAVORITES_NAME)    == 0) {
/*
            FavoritesSyncSource s = FavoritesSyncSource (FAVORITES_NAME, OL_FAVORITES);
            s.setPath(path);
            syncSources.add(s);
*/
        } else if (wcscmp(sources[i], FILES_NAME)        == 0) {
            
            BriefcaseSyncSource  s  = BriefcaseSyncSource (FILES_NAME, OL_BRIEFCASE);         
            s.setPath((wchar_t*)path);
            
            node = dmt->getManagementNode(BRIEFCASE_SOURCEPROP_CONTEXT);
            wchar_t* value = node->getPropertyValue(TEXT("dir"));
            s.setDir(value);
            syncSources.add(s);
            if (node) { delete node ; node = NULL; }

        } else if (wcscmp(sources[i], NOTES_NAME)        == 0) {

            NoteSyncSource  s  = NoteSyncSource (NOTES_NAME,     OL_NOTES);         
            s.setPath((wchar_t*)path);
            
            node = dmt->getManagementNode(NOTES_SOURCEPROP_CONTEXT);
            wchar_t* value = node->getPropertyValue(TEXT("dir"));
            s.setDir(value);
            syncSources.add(s);
            if (node) { delete node ; node = NULL; }

        } else if (wcscmp(sources[i], MAILS_NAME)         == 0) {
            MailSourceManagementNode m = MailSourceManagementNode(SOURCES_CONTEXT, TEXT("mails"));   
            MailSyncSourceConfig sc;    
            wchar_t t[12];
            sc = m.getMailSourceConfig(true);    
       
            getFolderToSync(sc, t);    
    
            int days = sc.getDownloadAge();
            wchar_t* downloadAge = getDayBefore(days);

            SourceFilter* filter = 
                ClauseUtil::createSourceFilter(downloadAge, sc.getBodySize(),sc.getAttachSize());

            //MailSyncSource     mailSource     = MailSyncSource    (MAILS_NAME);      
            mailSource.setPath((wchar_t*)path);
            mailSource.setFolderToSync(t);
            mailSource.setFilter(filter);

            syncSources.add(mailSource);
            isMailToSync = TRUE;
        }

        ++i;
    }

    //
    // Now we can create the Sync4jClient object and kick off the sync
    //
    Sync4jClient& s4j = Sync4jClient::getInstance();
    s4j.setDMConfig(APPLICATION_URI);
    LogLevel ll = getLOGLevel();
    LOG.setLevel(ll);

    //ret = s4j.sync(syncSources); 
    int l = syncSources.size();
    const int ml = 10;        
    SyncSource* array[ml];    
    for (i=0; (i<l) && (i<ml-1); ++i) {
        array[i] = (SyncSource*)syncSources[i];
    }
    array[l] = NULL;
    ret = s4j.sync(array);

    if (isMailToSync && mailSource.getIsMailInOutbox() && ret == 0) {
        SyncSource* array[2];    
        array[0] = &mailSource;
        array[1] = NULL;
        ret = s4j.sync(array);            
    }
    if (isMailToSync && mailSource.getIsMailInInbox()) {
        MessageBeep(MB_ICONASTERISK);
    }
    Sync4jClient::dispose();

    return ret;

}

BOOL startPolling()
{    
    return startcmd(SYNCAPP, TEXT("poll mail"));
}

/*
 * Return 0: process terminated successfully
 *        >0: error code from child process    
 *       -1: no such process/invalid handle
 *       -2: timeout
 *       -3: can't get child exit code
 *       -5: can't run the app
 */
int startSync(time_t timeout)
{
    if (!timeout)
        timeout=5*60; // default timeout 5 minutes

    unsigned long pid = startcmd(SYNCAPP, TEXT("once"));

    if (pid) {
        return waitProcess(pid, timeout*1000);
    }
    else {
        return -5;
    }
}

/*      
 * Entry point to kickoff the synchronization process, both specifying the 
 * sources or not. The default is to make all the configured sources.
 *
 * @param path    : the install path of the SyncClient PIM application on 
 *                  the device
 * @param sources : NULL terminated array of strings containing the source
 *                  name of the sources to synchronize
 */

DWORD WINAPI synchronize (wchar_t *errmsg, const wchar_t* path, const wchar_t** sources)
{
    wcscpy(errmsg, TEXT("Ok."));

    if (sources)
        return synchronizeMultipleSources(path, sources);
    else
        return synchronization(errmsg, (wchar_t *)path);
}

/*
    LOG_LEVEL_NONE  = 0
    LOG_LEVEL_INFO  = 1
    LOG_LEVEL_DEBUG = 2
*/

LogLevel getLOGLevel() {
    
    wchar_t level[64];
    wmemset(level, 0, 64);
    LogLevel ret = LOG_LEVEL_NONE;
    long l = 0;

    getClientConfigurationInternal(NULL, PROPERTY_SPDM_LOG_LEVEL, level, FALSE);
    
    if (wcslen(level) != 0)
        l = wcstol(level, NULL, 0);
    
    ret = (LogLevel)l;

    return ret;    
}


DWORD WINAPI settings() {
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("deviceId"       ), TEXT("sc-pim-ppc"                          ) );          
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("username"       ), TEXT("marco.magistrali"                               ) );     
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("password"       ), TEXT("marco"                               ) );     
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("syncUrl"        ), TEXT("http://192.168.0.22:8080/sync4j/sync") );   
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("serverName"     ), TEXT("http://192.168.0.22:8080"            ) );   
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("beginTimestamp" ), TEXT("0"                                   ) );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("endTimestamp"   ), TEXT("0"                                   ) );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("sourceCount"    ), TEXT("0"                                   ) );                                                                                                                          
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("responseTimeout"), TEXT("0"                                   ) );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("checkConn"      ), TEXT("1"                                   ) );                                                                                                                   
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("firstTimeSyncMode"), TEXT("0")                                  );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("proxyHost"        ), TEXT("" )                                  );       
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("proxyPort"        ), TEXT("" )                                  );       
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("useProxy"         ), TEXT("" )                                  );       
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("clientNonce"          ), TEXT("cGlwcG8="                  )     );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("serverNonce"          ), TEXT("VGB8YHQ6U25lWmAuR3Z3bw=="  )     );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("serverID"             ), TEXT("sync4j"                    )     );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("serverPWD"            ), TEXT("sync4j"                    )     );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("isServerAuthRequired" ), TEXT("F"                         )     );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("clientAuthType"       ), TEXT("syncml:auth-basic"         )     );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("serverAuthType"       ), TEXT("syncml:auth-basic"         )     );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("maxMsgSize"           ), TEXT("12288"                     )     );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\syncml"), TEXT("maxModPerMsg"         ), TEXT("8"                         )     );

setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\contacts"), TEXT("sync"     ), TEXT("none"              )            );                              
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\contacts"), TEXT("last"     ), TEXT("0"                    )            );                 
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\contacts"), TEXT("name"     ), TEXT("contact"              )            );                
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\contacts"), TEXT("type"     ), TEXT("text/x-s4j-sifc"      )            );                        
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\contacts"), TEXT("syncModes"), TEXT("slow,two-way,refresh" )            );              
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\contacts"), TEXT("uri"      ), TEXT("scard"                )            );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\contacts"), TEXT("name"), TEXT("contact" )                          );               
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\contacts"), TEXT("show"), TEXT("1"       )                          );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\contacts"), TEXT("useSIF"), TEXT("1"       )                          );

setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\appointments"), TEXT("sync"     ), TEXT("none"              )            );                              
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\appointments"), TEXT("last"     ), TEXT("0"                    )            );                 
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\appointments"), TEXT("name"     ), TEXT("calendar"             )            );                
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\appointments"), TEXT("type"     ), TEXT("text/x-s4j-sife"      )            );                        
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\appointments"), TEXT("syncModes"), TEXT("slow,two-way,refresh" )            );              
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\appointments"), TEXT("uri"      ), TEXT("scal"                 )            );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\appointments"), TEXT("name"), TEXT("calendar" )                          );               
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\appointments"), TEXT("show"), TEXT("1"       )                          );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\appointments"), TEXT("useSIF"), TEXT("1"       )                          );

setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\tasks"), TEXT("sync"     ), TEXT("none"              )            );                              
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\tasks"), TEXT("last"     ), TEXT("0"                    )            );                 
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\tasks"), TEXT("name"     ), TEXT("task"             )            );                
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\tasks"), TEXT("type"     ), TEXT("text/x-s4j-sift"      )            );                        
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\tasks"), TEXT("syncModes"), TEXT("slow,two-way,refresh" )            );              
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\tasks"), TEXT("uri"      ), TEXT("stask"                 )            );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\tasks"), TEXT("name"), TEXT("calendar" )                          );               
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\tasks"), TEXT("show"), TEXT("1"       )                          );


setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\notes"), TEXT("sync"     ), TEXT("none"              )            );                              
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\notes"), TEXT("last"     ), TEXT("0"                    )            );                 
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\notes"), TEXT("name"     ), TEXT("note"             )            );                
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\notes"), TEXT("type"     ), TEXT("text/x-s4j-sifn"      )            );                        
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\notes"), TEXT("syncModes"), TEXT("slow,two-way,refresh" )            );              
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\notes"), TEXT("uri"      ), TEXT("snote"                 )            );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\notes"), TEXT("name"), TEXT("note" )                          );               
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\notes"), TEXT("show"), TEXT("1"       )                          );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\notes"), TEXT("dir"), TEXT("\\My Documents"       )                          );

setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\briefcase"), TEXT("sync"     ), TEXT("none"              )            );                              
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\briefcase"), TEXT("last"     ), TEXT("0"                    )            );                 
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\briefcase"), TEXT("name"     ), TEXT("briefcase"             )            );                
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\briefcase"), TEXT("type"     ), TEXT("text/x-s4j-sifn"      )            );                        
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\briefcase"), TEXT("syncModes"), TEXT("slow,two-way,refresh" )            );              
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\briefcase"), TEXT("uri"      ), TEXT("./briefcase"                 )            );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\briefcase"), TEXT("name"), TEXT("briefcase" )                          );               
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\briefcase"), TEXT("show"), TEXT("1"       )                          );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\briefcase"), TEXT("dir"), TEXT("\\My Documents"       )                          );

// mail registry
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\mails"), TEXT("sync"     ), TEXT("two-way"              )            );                              
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\mails"), TEXT("last"     ), TEXT("0"                    )            );                 
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\mails"), TEXT("name"     ), TEXT("mail"              )            );                
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\mails"), TEXT("type"     ), TEXT("application/vnd.omads-email+xml"      )            );                        
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\mails"), TEXT("syncModes"), TEXT("slow,two-way,refresh" )            );              
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\mails"), TEXT("uri"      ), TEXT("./imapEmail"                )            );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\mails"), TEXT("Inbox"      ), TEXT("1"                )            );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\mails"), TEXT("Outbox"      ), TEXT("1"                )            );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\mails"), TEXT("Trash"      ), TEXT("0"                )            );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\mails"), TEXT("Sent"      ), TEXT("1"                )            );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sources\\mails"), TEXT("Draft"      ), TEXT("0"                )            );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\mails"), TEXT("name"), TEXT("mail" )                          );               
setProperty(TEXT("Software\\Funambol\\SyncclientPIM\\spds\\sourcesprop\\mails"), TEXT("show"), TEXT("1"       )                          );

//setProperty(TEXT("Software\\Funambol\\SyncclientPIM"), TEXT("Name"), TEXT("TransportDemo"));
setProperty(TEXT("Software\\Funambol\\SyncclientPIM"), TEXT("logLevel"),           TEXT("1")                                             );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM"), TEXT("encrypt"),            TEXT("0")                                             );
setProperty(TEXT("Software\\Funambol\\SyncclientPIM"), TEXT("path"),               TEXT("0")                                             );

// related to create an account programmatically
setProperty(TEXT("Software\\Microsoft\\Inbox\\Svc\\Sync4j"), TEXT("Name"), TEXT("Sync4j"));
setProperty(TEXT("Software\\Microsoft\\Inbox\\Svc\\Sync4j"), TEXT("DLL"),  TEXT("syncmltransport.dll"));
setProperty(TEXT("Software\\Microsoft\\Inbox\\Svc\\Sync4j"), TEXT("Port"), TEXT("0"));

return 0;
}


