/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>

#include "base/Log.h"
#include "spds/spdsutils.h"
#include "spds/Constants.h"

#include "pim/LongObj.h"

#include "pim/Utils.h"
#include "pim/ContactBuilder.h"
#include "pim/AppointmentBuilder.h"
#include "pim/TaskBuilder.h"
#include "pim/POOMFileManagement.h"
#include "pim/POOMClient2Server.h"

void setAllItemsPOOM(Container* c, int dataType) {          
    
    IPOutlookApp* polApp = getOutlookApp();
    if (polApp == NULL)
        return;


    if (dataType == olFolderCalendar)
        normalizeAppointment(polApp, dataType);

    wchar_t localTemp [DIM_LOCAL_TEMP];

    IPOutlookItemCollection *pItems;
    IFolder *pFolder;

    IContact     *pContact;
    IAppointment *pAppointment;
    ITask        *pTask;

    polApp->GetDefaultFolder (dataType, &pFolder);

    pFolder->get_Items (&pItems);

    int countItems = 0;
    objlist currentOid  = objlist(true);
    objlist currentHash = objlist(true);

    pItems->get_Count(&countItems);
	bool isSIF = checkSIF(dataType);

    SyncItem* syncItem = NULL;
    for (int j=0; j<countItems; j++) {
        string stringItem = string();
        long oid = 0;

        // Item values start from element number 1 not 0!!!!!
        // from dataType choose if a Contact or a Appointment
        // leave 2 populateString to semplify
        if (dataType == olFolderContacts) {
            pItems->Item (j + 1, (IDispatch**)&pContact);
			/*
			if(isSIF)
				populateContactStringItem(stringItem, pContact);
			else {
				VObject* vo = ContactToVObject(pContact);
				if(vo) {
					stringItem = vo->toString();
					delete vo;
				}
			}
            */
            pContact->get_Oid(&oid);
            
            long currentLongHash = calculateContactHash(pContact);                  
            // set these objlist to write to file...
            add(currentOid, new LongObj(oid));
            add(currentHash, new LongObj(currentLongHash));
                
        }
        else if (dataType == olFolderCalendar){
            pItems->Item (j + 1, (IDispatch**)&pAppointment);

			/* if(isSIF)
				populateAppointmentStringItem(stringItem, pAppointment);
			else
			{
				VObject* vo = AppointmentToVObject(pAppointment);
				if(vo) {
					stringItem = vo->toString();
					LOG.debug(TEXT("Vobject to string ..."));
					LOG.debug(vo->toString());
					delete vo;
				}
			} */
            pAppointment->get_Oid(&oid);
			
			long currentLongHash = calculateAppointmentHash(pAppointment);
			            
            // set these objlist to write to file...
            add(currentOid, new LongObj(oid));
            add(currentHash, new LongObj(currentLongHash));
                
        }
        else if (dataType == olFolderTasks){
            pItems->Item (j + 1, (IDispatch**)&pTask);
            //populateTaskStringItem(stringItem, pTask);
            pTask->get_Oid(&oid);

            long currentLongHash = calculateTaskHash(pTask);
            
            // set these objlist to write to file...
            add(currentOid, new LongObj(oid));
            add(currentHash, new LongObj(currentLongHash));
                
        }
        wsprintf(localTemp, TEXT("%i"), oid);                                   

        syncItem = new SyncItem(localTemp);
        
        // encode base64
        /*
        wchar_t* t = NULL;
        
        if(isSIF) {
            t = encodeBase64UTF8((wchar_t*)stringItem);
            stringItem = t;
        }
        syncItem->setData((wchar_t*)stringItem, ((length(stringItem))*sizeof(wchar_t))); 
        */
        c->addItemToAllItems(syncItem);

        if (syncItem) {
            delete syncItem; 
            syncItem = NULL;
        }

        //if (t != NULL)
        //    delete t;
                
    }    

    // Release objects
    if (countItems != 0){
        if (dataType == olFolderContacts)
            pContact->Release ();
        else if (dataType == olFolderCalendar)
            pAppointment->Release ();
        else if (dataType == olFolderTasks)
            pTask->Release ();
    }
    pItems->Release ();
    pFolder->Release ();
    
    releaseOutlookApp(polApp);

    wsprintf(localTemp, SET_ALL_ITEMS, countItems); 
    LOG.debug(TEXT("Exiting from setAllItems... "));
 }

 /*
Set modified Items for FAST_SYNC from PPC to server
*/

void setModifiedItemsPOOM(Container* c, int dataType, wchar_t* path) {
        
    LOG.debug(TEXT("FAST: in setModifiedItems... "));                        
    wchar_t localTemp [DIM_LOCAL_TEMP];    
    
    IPOutlookApp* polApp = getOutlookApp();
    if (polApp == NULL)
        return;

    IContact     *pContact;
    IAppointment *pAppointment;
    ITask        *pTask;   

    int countItems = 0;
    
    objlist newOid     = objlist(true);
    objlist updatedOid = objlist(true);
    objlist deletedOid = objlist(true);
    bool isSIF = checkSIF(dataType);
    
    // only if dataType is appointment
    if (dataType == olFolderCalendar)
            normalizeAppointment(polApp, dataType);

    fillModifiedOidArray (newOid, updatedOid, deletedOid, polApp, dataType, path) ;
            
    // SyncItem** newItems = new SyncItem* [length(newOid)];
    // SyncItem** updatedItems = new SyncItem* [length(updatedOid)];
    // SyncItem** deletedItems = new SyncItem* [length(deletedOid)];
    
    SyncItem* syncItem = NULL;

    // for every oid array...
    int i = 0;
    long oid = 0;
    for (i = 0; i < length(newOid); i++) {
            
        string stringItem = string();
        LongObj* longObj = (LongObj*) get(newOid, i);
                
        oid = longObj->getOID();                
        wsprintf(localTemp, TEXT("%i"), oid);
        

        if (dataType == olFolderContacts) { // it's a contact
            polApp->GetItemFromOid (oid, (IDispatch**)&pContact);
            
            if(isSIF)
				populateContactStringItem(stringItem, pContact);        
			else {
				VObject* vo = ContactToVObject(pContact);
				if(vo) {
					stringItem = vo->toString();
					delete vo;
				}
			}
            
            
        }
        else if (dataType == olFolderCalendar){ // it's an appointments
        
            polApp->GetItemFromOid (oid, (IDispatch**)&pAppointment);
			
			if(isSIF)
				populateAppointmentStringItem(stringItem, pAppointment);
			else {
				VObject* vo = AppointmentToVObject(pAppointment);
				if(vo) {
					stringItem = vo->toString();
					delete vo;
				}
			}
                                
        }
        else if (dataType == olFolderTasks){ // it's an task
            
            polApp->GetItemFromOid (oid, (IDispatch**)&pTask);
			if(isSIF)
				populateTaskStringItem(stringItem, pTask);
			else {
				VObject* vo = TaskToVObject(pTask);
				if(vo) {
					stringItem = vo->toString();
					delete vo;
				}
			}
                                    
        }
        // Set the SyncItem
            
        // newItems[i] = new SyncItem(localTemp);                          
        syncItem = new SyncItem(localTemp);                          
        
        // encode base64
        
        wchar_t* t = NULL;        
        if (isSIF) {
        t = encodeBase64UTF8((wchar_t*)stringItem);
        stringItem = t;
        }
        // newItems[i]->setData((wchar_t*)stringItem, ((length(stringItem) + 1)*sizeof(wchar_t))); 
        syncItem->setData((wchar_t*)stringItem, ((length(stringItem) + 1)*sizeof(wchar_t))); 
        
        syncItem->setState(SYNC_STATE_NEW);
        
        c->addItemToNewItems(syncItem);
        
        if (syncItem) {
            delete syncItem; syncItem = NULL;
        }
        if (t != NULL)
            delete t;
        
    }
        
    i=0;
    oid = 0;
    for (i = 0; i < length(updatedOid); i++) {
                
        string stringItem = string();
        LongObj* longObj = (LongObj*) get(updatedOid, i);
                
        oid = longObj->getOID();                
        wsprintf(localTemp, TEXT("%i"), oid);
            

        if (dataType == olFolderContacts) { // it's a contact
                polApp->GetItemFromOid (oid, (IDispatch**)&pContact);
                
                if(isSIF)
					populateContactStringItem(stringItem, pContact);                        
			    else {
				    VObject* vo = ContactToVObject(pContact);
				    if(vo) {
					    stringItem = vo->toString();
					    delete vo;
				    }
			    }
                
                // populateContactStringItem(stringItem, pContact);                        
        }
        else if (dataType == olFolderCalendar){ // it's an appointments
                polApp->GetItemFromOid (oid, (IDispatch**)&pAppointment);
				if(isSIF)
					populateAppointmentStringItem(stringItem, pAppointment);
				else {
					VObject* vo = AppointmentToVObject(pAppointment);
					if(vo) {
						stringItem = vo->toString();
						delete vo;
					}
				}
        }
        else if (dataType == olFolderTasks){ // it's a task
                polApp->GetItemFromOid (oid, (IDispatch**)&pTask);
                populateTaskStringItem(stringItem, pTask);                                      
        }
        // Set the SyncItem
        // updatedItems[i] = new SyncItem(localTemp);              
        syncItem = new SyncItem(localTemp);
        
        // encode base64
        
        wchar_t* t = NULL;
        if (isSIF) {
        t = encodeBase64UTF8((wchar_t*)stringItem);
        stringItem = t;
        }
        // updatedItems[i]->setData((wchar_t*)stringItem, ((length(stringItem) + 1)*sizeof(wchar_t)));
        syncItem->setData((wchar_t*)stringItem, ((length(stringItem) + 1)*sizeof(wchar_t)));
        
        syncItem->setState(SYNC_STATE_UPDATED);
        c->addItemToUpdatedItems(syncItem);
        
        if (syncItem) {
            delete syncItem; syncItem = NULL;
        }
        if (t != NULL)
            delete t;
    }
        
    i=0;
    oid = 0;
    for (i = 0; i < length(deletedOid); i++) {
            
        string stringItem = string();
        LongObj* longObj = (LongObj*) get(deletedOid, i);
                
        oid = longObj->getOID();                
        wsprintf(localTemp, TEXT("%i"), oid);
            
        if (dataType == olFolderContacts) { // it's a contact           
        }
        else if (dataType == olFolderCalendar){ // it's an appointments                                 
        }
        else if (dataType == olFolderTasks) {                                   
        }
        // Set the SyncItem
        // deletedItems[i] = new SyncItem(localTemp);         
        
        syncItem = new SyncItem(localTemp);
        
        c->addItemToDeletedItems(syncItem);

        if (syncItem) {
            delete syncItem; syncItem = NULL;
        }
                    
    }

    // now we have to disctinct if generic Items are IContact or ICalendar
    if ((length(newOid) != 0)  || (length(updatedOid) != 0) ) {
        if (dataType == olFolderContacts)
                pContact->Release();
        else if (dataType == olFolderCalendar)
                pAppointment->Release ();
        else if (dataType == olFolderTasks)
                pTask->Release ();
    }           

    releaseOutlookApp(polApp);

    LOG.debug(TEXT("end setModifiedItems... "));
    
}

/*
fill array new, update, delete with OID of element to modify. This function can be used by every source.
This function call funcion that use the alghorytm to retrieve OID modified
*/
void fillModifiedOidArray (objlist& newOid, objlist& updateOid, objlist& deleteOid, 
                                                   IPOutlookApp *polApp, int dataType, wchar_t* path) {
        
        
    wchar_t localTemp [DIM_LOCAL_TEMP];

    IPOutlookItemCollection *pItems;
    IFolder *pFolder;

    IContact     *pContact;
    IAppointment *pAppointment;
    ITask        *pTask;

    polApp->GetDefaultFolder (dataType, &pFolder);

    pFolder->get_Items (&pItems);

    int currentCountItems = 0;
    objlist currentOid  = objlist(true);
    objlist currentHash = objlist(true);

    pItems->Sort(TEXT ("[Oid]"), FALSE);
    pItems->get_Count(&currentCountItems);
    
    if (dbg) {
            wsprintf(localTemp, TEXT("In device: Item's number is: %i"), currentCountItems);
            MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    }
    long previousCountItems = 0;
    objlist previousOid  = objlist(true);
    objlist previousHash = objlist(true);


    readFromFile(&previousCountItems, previousOid, previousHash, dataType, path);

    if (dbg) {
            wsprintf(localTemp, TEXT("%i"), previousCountItems);
            MessageBox (NULL, localTemp, TEXT ("Previoes count item"), MB_SETFOREGROUND |MB_OK);
    }
  
    int i = 0;

    int j = 0;
    int lastFound = -1;

    long currentLongOid = 0;
    long currentLongHash = 0;
    
    for (i = 0; i < currentCountItems; i++) {

        if (dataType == olFolderContacts) {
            currentLongOid = 0;
            pItems->Item (i + 1, (IDispatch**)&pContact);
            
            pContact->get_Oid(&currentLongOid);                     
            currentLongHash = calculateContactHash(pContact);
            // set these objlist to write to file...
            add(currentOid, new LongObj(currentLongOid));
            add(currentHash, new LongObj(currentLongHash));

        }
        else if (dataType == olFolderCalendar){ // for appointment
            long internalOid = 0;
            
            pItems->Item (i + 1, (IDispatch**)&pAppointment);
            
            pAppointment->get_Oid(&internalOid);
            currentLongHash = calculateAppointmentHash(pAppointment);
            currentLongOid = internalOid;
            // set these objlist to write to file...
            add(currentOid, new LongObj(currentLongOid));
            add(currentHash, new LongObj(currentLongHash));
                
        }
        else if (dataType == olFolderTasks){ // for appointment
            long internalOid = 0;                   
            pItems->Item (i + 1, (IDispatch**)&pTask);                      
            pTask->get_Oid(&internalOid);
            currentLongHash = calculateTaskHash(pTask);
            currentLongOid = internalOid;
            // set these objlist to write to file...
            add(currentOid, new LongObj(currentLongOid));
            add(currentHash, new LongObj(currentLongHash));
                
        }
        int itemFound = 0;
        j = lastFound + 1;
        
        if (dbg) {
            wsprintf(localTemp, TEXT("i: %i - j: %i"), i,j);
            MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
        while ((j < previousCountItems && itemFound == 0)) {
            LongObj* longObjOid = (LongObj*) get(previousOid, j);
            long previousLongOid = longObjOid->getOID();    


            if (currentLongOid == previousLongOid) {
                itemFound = 1;
                lastFound = j;
                
                LongObj* longObjHash = (LongObj*) get(previousHash, j);
                long previousLongHash = longObjHash->getOID();  

                if (currentLongHash != previousLongHash) {
                    // if different hash then into updateOid to pass to setModifiedItems
                    if (dbg)
                            MessageBox (NULL, TEXT("Update"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
                    add(updateOid, new LongObj(currentLongOid));
                } else {
                    // nothing to do...
                }
            }
            j++;
        }
        if (itemFound == 0) {
            if (dbg)
                MessageBox (NULL, TEXT("New"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            add(newOid, new LongObj(currentLongOid));
        }
                

    } // fine for

    i = 0;
    lastFound = -1;

    for (j = 0; j < previousCountItems; j++) {

        int itemFound = 0;
        i = lastFound + 1;
        
        LongObj* longObjOid = (LongObj*) get(previousOid, j);
        long previousLongOid = longObjOid->getOID();    

        
        while ((i < currentCountItems && itemFound == 0)) {
            LongObj* longObjOidC = (LongObj*) get(currentOid, i);
            long currentLongOid = longObjOidC->getOID();    

            if (currentLongOid == previousLongOid) {
                itemFound = 1;
                lastFound = i;
            }
            i++;
        }

        if (itemFound == 0) {
            if (dbg)
                MessageBox (NULL, TEXT("Delete"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
            add(deleteOid, new LongObj(previousLongOid));
        }

    } // fine for
        
    if (dbg)
        MessageBox (NULL, TEXT("Prima di write"), TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
    

    // Release objects
    if (currentCountItems != 0){
        if (dataType == olFolderContacts)
            pContact->Release ();
        else if (dataType == olFolderCalendar)
            pAppointment->Release ();
        else if (dataType == olFolderTasks)
            pTask->Release ();
    }
    
    pFolder->Release ();
    pItems->Release ();
 
}

void fillSyncItem(SyncItem* syncItem, int dataType) {
    
    IPOutlookApp* polApp = getOutlookApp();
    if (polApp == NULL)
        return;

    IContact     *pContact;
    IAppointment *pAppointment;
    ITask        *pTask; 
    
    bool isSIF = checkSIF(dataType);
    string stringItem = string();
    long oid = wcstol(syncItem->getKey(), NULL, 10);

    if (dataType == olFolderContacts) { // it's a contact
        polApp->GetItemFromOid (oid, (IDispatch**)&pContact);
        if(isSIF)
			populateContactStringItem(stringItem, pContact);
		else {
			VObject* vo = ContactToVObject(pContact);
			if(vo) {
				stringItem = vo->toString();
				delete vo;
			}
		}
                
    }
    else if (dataType == olFolderCalendar){ // it's an appointments
    
        polApp->GetItemFromOid (oid, (IDispatch**)&pAppointment);

		if(isSIF)
			populateAppointmentStringItem(stringItem, pAppointment);
		else {
			VObject* vo = AppointmentToVObject(pAppointment);
			if(vo) {
				stringItem = vo->toString();
				delete vo;
			}
		}
                            
    }
    else if (dataType == olFolderTasks){ // it's an task
        
        polApp->GetItemFromOid (oid, (IDispatch**)&pTask);
		if(isSIF)
			populateTaskStringItem(stringItem, pTask);
		else {
			VObject* vo = TaskToVObject(pTask);
			if(vo) {
				stringItem = vo->toString();
				delete vo;
			}
		}
                                
    }

    // encode base64    
    wchar_t* t = NULL;
    
    if(isSIF) {
        t = encodeBase64UTF8((wchar_t*)stringItem);
        stringItem = t;
    }
    syncItem->setData((wchar_t*)stringItem, ((length(stringItem) + 1)*sizeof(wchar_t))); 
    
    // Release objects
   
        if (dataType == olFolderContacts)
            pContact->Release ();
        else if (dataType == olFolderCalendar)
            pAppointment->Release ();
        else if (dataType == olFolderTasks)
            pTask->Release ();
   
   
    releaseOutlookApp(polApp);

}
