/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include <windows.h>
#include <oleauto.h>

#include "pimstore.h"
#include <stdio.h>
#include "pim/LongObj.h"

#include "base/Log.h"

#include "pim/Utils.h"
#include "pim/ActionLog.h"
#include "pim/POOMFileManagement.h"
#include "pim/SettingFunctions.h"

/*
*
*   FILE MANAGEMENT
*
*
*/

void writeToFile(objlist &currentOid, objlist &currentHash, int dataType, wchar_t* path) {
                
    wchar_t wfilename[DIM_FILE];
    wchar_t* pt = NULL;
    char filename [DIM_FILE];
    BOOL usedDefaultChar;
        
    if (dataType == olFolderContacts) 
        pt = TEXT(FILENAME_CONTACTS);
    else if (dataType == olFolderCalendar) 
        pt = TEXT(FILENAME_APPOINTMENTS);
    else if (dataType == olFolderTasks) 
        pt = TEXT(FILENAME_TASKS);
    wsprintf(wfilename, TEXT("%s%s"),path, pt);
    

    if (dbg) {
        MessageBox (NULL, path, TEXT ("path"), MB_SETFOREGROUND |MB_OK);
        MessageBox (NULL, wfilename, TEXT ("path"), MB_SETFOREGROUND |MB_OK);   
    }
        
    WideCharToMultiByte (CP_ACP, 0, wfilename, 255, filename, 255, "?", &usedDefaultChar);

    FILE* f;
    if (dataType == olFolderContacts) {
        f = fopen(filename, "w+");
        fwprintf(f, TEXT("<contacts>\n"));
    }
    else if (dataType == olFolderCalendar){
        f = fopen(filename, "w+");
        fwprintf(f, TEXT("<appointments>\n"));
    }
    else if (dataType == olFolderTasks){
        f = fopen(filename, "w+");
        fwprintf(f, TEXT("<tasks>\n"));
    }
        
        
    fwprintf(f, TEXT("<num>%i</num>\n"), length(currentOid));
    int oidLength = length(currentOid);

    for (int i = 0; i < oidLength; i++) {
        LongObj* longObjC = (LongObj*) get(currentOid, i);
        long currentLongOid = longObjC->getOID();       
        LongObj* longObjH = (LongObj*) get(currentHash, i);
        long currentLongHash = longObjH->getOID();      

        fwprintf(f, TEXT("<item>\n"));
        fwprintf(f, TEXT("<oid>%i</oid>\n"), currentLongOid);
        fwprintf(f, TEXT("<hash>%i</hash>\n"), currentLongHash);
        fwprintf(f, TEXT("</item>\n"));
    }
        
    if (dataType == olFolderContacts) {
        fwprintf(f, TEXT("</contacts>\n"));
    }
    else if (dataType == olFolderCalendar){
        fwprintf(f, TEXT("</appointments>\n"));
    }
    else if (dataType == olFolderTasks){
        fwprintf(f, TEXT("</tasks>\n"));
    }
    
    fflush(f);
    fclose(f);
}


// method used to check if file with previous appointments or contacts exist! if not exists force slow sync
BOOL existsFile(int dataType, wchar_t* path) {
        
    wchar_t wfilename[DIM_FILE];
    wchar_t* pt = NULL;
    char filename [DIM_FILE];
    BOOL usedDefaultChar;
    BOOL exists = FALSE;
    
    if (dataType == olFolderContacts) 
        pt = TEXT(FILENAME_CONTACTS);
    else if (dataType == olFolderCalendar) 
        pt = TEXT(FILENAME_APPOINTMENTS);
    else if (dataType == olFolderTasks) 
        pt = TEXT(FILENAME_TASKS);
    else if (dataType == constFiles) 
        pt = TEXT(FILENAME_FILES);
    else if (dataType == constNotes) 
        pt = TEXT(FILENAME_NOTES);
    else if (dataType == constFavorites) 
        pt = TEXT(FILENAME_FAVORITES);

    wsprintf(wfilename, TEXT("%s%s"),path, pt);     

    if (dbg) {
        MessageBox (NULL, path, TEXT ("path read"), MB_SETFOREGROUND |MB_OK);
        MessageBox (NULL, wfilename, TEXT ("path read"), MB_SETFOREGROUND |MB_OK);      
    }
    
    WideCharToMultiByte (CP_ACP, 0, wfilename, DIM_FILE -1, filename, DIM_FILE -1, "?", &usedDefaultChar);
    
    FILE* f = NULL;
    f = fopen(filename, "r");

    if (f != NULL) {
        fflush(f);
        fclose(f);
        exists = TRUE;
    }

    return exists;  

}


void readFromFile(long* previousCountItem, objlist &previousOid, objlist &previousHash, int dataType, wchar_t* path) {
        
    wchar_t wfilename[DIM_FILE];
    wchar_t* pt = NULL;
    char filename [DIM_FILE];
    BOOL usedDefaultChar;
    
    if (dataType == olFolderContacts) 
        pt = TEXT(FILENAME_CONTACTS);
    else if (dataType == olFolderCalendar) 
        pt = TEXT(FILENAME_APPOINTMENTS);
    else if (dataType == olFolderTasks) 
        pt = TEXT(FILENAME_TASKS);


    wsprintf(wfilename, TEXT("%s%s"),path, pt);
    

    if (dbg) {
            MessageBox (NULL, path, TEXT ("path read"), MB_SETFOREGROUND |MB_OK);
            MessageBox (NULL, wfilename, TEXT ("path read"), MB_SETFOREGROUND |MB_OK);      
    }
    
    WideCharToMultiByte (CP_ACP, 0, wfilename, DIM_FILE -1, filename, DIM_FILE -1, "?", &usedDefaultChar);


    wchar_t* element = NULL;
    wchar_t line[100];
    FILE* f;
    wchar_t* tmp = NULL;
    if (dataType == olFolderContacts) 
        f = fopen(filename, "r");
    else if (dataType == olFolderCalendar) 
        f = fopen(filename, "r");
    else if (dataType == olFolderTasks) 
        f = fopen(filename, "r");
    
    while(fgetws(line, 99, f) != NULL) {
            
        if ((element = getElementContent(line, TEXT("oid"), NULL)) != NULL) {            
            add(previousOid, new LongObj(wcstol(element, &tmp, 0)));
        }

        else if ((element = getElementContent(line, TEXT("hash"), NULL)) != NULL) {            
            add(previousHash, new LongObj(wcstol(element, &tmp, 0)));
        }

        else if ((element = getElementContent(line, TEXT("num"), NULL)) != NULL) {                          
            (*previousCountItem) = wcstol(element, &tmp, 0);
        }        
        delete element;

    }
        fflush(f);
        fclose(f);
}
