/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include "spdm/Constants.h"

#include "pim/SettingFunctions.h"
#include <winioctl.h>
#include "ptypes/ptypes.h"



#define IOCTL_HAL_GET_DEVICEID CTL_CODE(FILE_DEVICE_HAL, 21, METHOD_BUFFERED, FILE_ANY_ACCESS)

extern "C" __declspec(dllimport) BOOL KernelIoControl(
                                                        DWORD dwIoControlCode, 
                                                        LPVOID lpInBuf, 
                                                        DWORD nInBufSize, 
                                                        LPVOID lpOutBuf, 
                                                        DWORD nOutBufSize, 
                                                        LPDWORD lpBytesReturned
);

USING_PTYPES

/*
* Get the unique Device Identifier from PPC 2002 - 2003
* see more at http://www.pocketpcdn.com/articles/serial_number2002.html
*
* @param deviceId     : the retrieved device id
*/

void GetSerialNumberFromKernelIoControl(wchar_t* deviceId) {
        
    DWORD dwOutBytes = 0;
    const int nBuffSize = 4096;
    byte arrOutBuff[nBuffSize];
    BOOL bRes = TRUE; 
    
    bRes = KernelIoControl(IOCTL_HAL_GET_DEVICEID, 0, 0, arrOutBuff, nBuffSize, &dwOutBytes);

    if (bRes) {
        string strDeviceInfo = string();
        for (unsigned int i = 0; i<dwOutBytes; i++) {                   
            wchar_t tmp [10];
            wsprintf(tmp, TEXT("%02X"), arrOutBuff[i]);
            strDeviceInfo = strDeviceInfo + tmp;

        }
        del(strDeviceInfo, 0, 40);
        del(strDeviceInfo, 2, 3);
        del(strDeviceInfo, 11, 16);
        del(strDeviceInfo, 17, 4);
        
        wsprintf(deviceId, TEXT("fwm-%s"), strDeviceInfo);
                        
    } else {
        deviceId[0] = 0;        
    }               
}

/* 
* Method to get communication settings from device. 
* @param name  : name of the property to retrieve value of
* @param value : value of the property
*/

DWORD WINAPI getConfigParameter (wchar_t* name, wchar_t* value) {
        
    wchar_t tmp[512];       
    wchar_t spds_context [128];
    wsprintf(spds_context, SPDS_CONTEXT);
    
    if      (wcscmp(name, SPDM_SYNCURL   ) == 0) {        
        wmemset(tmp, 0, 512);
        getProperty(spds_context, PROPERTY_SYNC_URL, tmp, 512);                   
    }
    
    else if (wcscmp(name, SPDM_DEVICEID) == 0) {
        
        wmemset(tmp, 0, 512);
        getProperty(spds_context, PROPERTY_DEVICE_ID, tmp, 512);        

        if (wcscmp(tmp, TEXT("")) == 0) {
            GetSerialNumberFromKernelIoControl(tmp);
            GetSerialNumberFromKernelIoControl(tmp);            
        }
    }
    
    else if (wcscmp(name, SPDM_PASSWORD  ) == 0) {        
        wmemset(tmp, 0, 512);
        getProperty(spds_context, PROPERTY_PASSWORD, tmp, 512);
    }
    else if (wcscmp(name, SPDM_USERNAME  ) == 0) {  
        wmemset(tmp, 0, 512);
        getProperty(spds_context, PROPERTY_USERNAME, tmp, 512);
    }
    else if (wcscmp(name, SPDM_SERVERNAME) == 0) {  
        wmemset(tmp, 0, 512);
        getProperty(spds_context, PROPERTY_SERVER_NAME, tmp, 512);
    }    
        
    // overwrite the terminator char \0 of wchar_t* array to read value properly

    wcscpy(value, tmp);    
    value[wcslen(value)] = L' ';

    return 0;
}

/* 
* Method to set communication settings into device. The parameters are retrieved by Visual Basic interface
*
* @param username     : user name
* @param password     : user password
* @param syncURL      : synchronization url
* @param serverName   : the server uri used to address the sync server
* @param errorMsg     : the returned message to vb interface
*/

DWORD WINAPI setConfigParameters(wchar_t *username, wchar_t* password, 
                                 wchar_t* syncURL, wchar_t* serverName, wchar_t* errorMsg) {
        
    wchar_t deviceId [DIM_DEVICE_ID];       
    
    GetSerialNumberFromKernelIoControl(deviceId);
    
    /*
    * Repeat function call GetSerialNumberFromKernelIoControl to fix
    * retrieve serial number from Asus MyPal 620.
    * It's seems this function doesn't work properly if the function
    * is called only one time. 
    */

    GetSerialNumberFromKernelIoControl(deviceId);

    if (wcsstr(syncURL, HTTP_HEADER) == NULL && wcsstr(syncURL, HTTPS_HEADER) == NULL ) {
        wchar_t tmp[256];
        wsprintf(tmp, HTTP_HEADER);
        wcscat(tmp, syncURL);
        wsprintf(syncURL, tmp);    
    }
    wchar_t spds_context[128];
    wsprintf(spds_context, SPDS_CONTEXT);
    
    setProperty(spds_context, PROPERTY_SYNC_URL , syncURL);    
    setProperty(spds_context, PROPERTY_USERNAME , username); 
    setProperty(spds_context, PROPERTY_PASSWORD , password); 
    setProperty(spds_context, PROPERTY_SERVER_NAME , serverName); 
    /*
    wchar_t* p = NULL;
    p = TEXT("syncUrl"); 
    setProperty(spds_context, p , syncURL);    

    p = TEXT("username");
    setProperty(spds_context,  p, username); 

    p = TEXT("password") ;    
    setProperty(spds_context, p, password); 

    p = TEXT("serverName");    
    setProperty(spds_context, p , serverName); 
    */
    // 
    // check if deviceId has arleady a value
    //
    wchar_t previousDeviceId[512];    
    getProperty(spds_context, PROPERTY_DEVICE_ID, previousDeviceId, 511);    

    if (wcscmp(previousDeviceId, TEXT("")) == 0) {
        setProperty(spds_context, PROPERTY_DEVICE_ID, deviceId);     
    }

    wsprintf(errorMsg, TEXT("success"));     
    
    // overwrite the terminator char \0 of wchar_t* array to read value properly
    errorMsg[wcslen(errorMsg)] = L' ';
    
    return 0;
}

/*
* Method to get the synchronization setting parameters.
* 
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property
*/

DWORD WINAPI getSourceParameter (LPWSTR sourceName, LPWSTR propertyName,  LPWSTR propertyValue) {

    wchar_t rootContext [DIM_MANAGEMENT_PATH];
    wchar_t rootContextProps [DIM_MANAGEMENT_PATH];
    wchar_t tmp[128];
           
    if (wcscmp(sourceName, SYNC_APPOINTMENTS) == 0) {          
            wsprintf(rootContext, APPOINTMENTS_CONTEXT);      
            wsprintf(rootContextProps, APPOINTMENTS_SOURCEPROP_CONTEXT);     
    }
    else if (wcscmp(sourceName, SYNC_CONTACTS) == 0) {
            wsprintf(rootContext, CONTACTS_CONTEXT);     
            wsprintf(rootContextProps, CONTACTS_SOURCEPROP_CONTEXT);   
    }
    else if (wcscmp(sourceName, SYNC_TASKS) == 0) {
            wsprintf(rootContext, TASKS_CONTEXT);   
            wsprintf(rootContextProps, TASKS_SOURCEPROP_CONTEXT);
        }
    else if (wcscmp(sourceName, SYNC_NOTES) == 0) {
            wsprintf(rootContext, NOTES_CONTEXT);  
            wsprintf(rootContextProps, NOTES_SOURCEPROP_CONTEXT);            
        }
    else if (wcscmp(sourceName, SYNC_FILES) == 0) {
            wsprintf(rootContext, BRIEFCASE_CONTEXT);   
            wsprintf(rootContextProps, BRIEFCASE_SOURCEPROP_CONTEXT);                      
        }
    else if (wcscmp(sourceName, SYNC_FAVORITES) == 0) {
            wsprintf(rootContext, FAVORITES_CONTEXT);     
        }
    else if (wcscmp(sourceName, SYNC_MAILS) == 0) {
            wsprintf(rootContext, MAILS_CONTEXT);     
            wsprintf(rootContextProps, MAILS_SOURCEPROP_CONTEXT);
        }
    
    if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SYNC) == 0)  {          // sync
        getProperty(rootContext, PROPERTY_SOURCE_SYNC, tmp, 127);        
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_LAST) == 0) {  
        getProperty(rootContext, PROPERTY_SOURCE_LAST_SYNC, tmp, 127);     
    }
     else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_URI) == 0) {    // uri
        getProperty(rootContext, PROPERTY_SOURCE_URI, tmp, 127);
        
    }  
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SYNC_MODES) == 0) {   // sync modes      
       getProperty(rootContext, PROPERTY_SOURCE_SYNC_MODES, tmp, 127);        
    } 

    /*
    * Properties into rootContextProp
    */
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DIR) == 0) {    
         // dir. It is the parameter only for briefcase and notes
        getProperty(rootContextProps, PROPERTY_SOURCE_DIR, tmp, 127); 
        
        wchar_t* loc = wcschr(tmp, L'\\');
        if (loc != NULL)
            wsprintf(tmp, loc);
        else {
            SHGetSpecialFolderPath(NULL, tmp, CSIDL_PERSONAL, FALSE);                
        }
    }  
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_INBOX) == 0) {    
         getProperty(rootContext, PROPERTY_SOURCE_INBOX, tmp, 127); 
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_OUTBOX) == 0) {    
         getProperty(rootContext, PROPERTY_SOURCE_OUTBOX, tmp, 127); 
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_TRASH) == 0) {    
         getProperty(rootContext, PROPERTY_SOURCE_TRASH, tmp, 127); 
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SENT) == 0) {    
         getProperty(rootContext, PROPERTY_SOURCE_SENT, tmp, 127); 
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DRAFT) == 0) {    
         getProperty(rootContext, PROPERTY_SOURCE_DRAFT, tmp, 127); 
    } 
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_ATTACH_SIZE) == 0) {
        getProperty(rootContext, PROPERTY_SOURCE_ATTACH_SIZE, tmp, 127);         
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DOWNLOAD_AGE) == 0) {
        getProperty(rootContext, PROPERTY_SOURCE_DOWNLOAD_AGE, tmp, 127);         
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_BODY_SIZE) == 0) {
        getProperty(rootContext, PROPERTY_SOURCE_BODY_SIZE, tmp, 127);         
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_USE_SIF) == 0) {    
         getProperty(rootContextProps, PROPERTY_SOURCEPROP_USE_SIF, tmp, 127); 
    } 

    // overwrite the terminator char \0 of wchar_t* array to read value properly
    wcscpy(propertyValue, tmp);
    propertyValue[wcslen(propertyValue)] = L' ';
      
    return 0;
}

/*
* Method to set the synchronization setting parameters.
* 
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property
* @param errorMsg       : returned error message
*/

DWORD WINAPI setSourceParameters (wchar_t* sourceName, wchar_t* propertyName, 
                                  wchar_t* propertyValue, wchar_t* errorMsg) {

    wchar_t rootContext [DIM_MANAGEMENT_PATH];
    wsprintf(rootContext, ROOT_CONTEXT);
    wchar_t rootContextProps [DIM_MANAGEMENT_PATH];

    
    if (wcscmp(sourceName, SYNC_APPOINTMENTS) == 0) {          
            wsprintf(rootContext, APPOINTMENTS_CONTEXT);               
            wsprintf(rootContextProps, APPOINTMENTS_SOURCEPROP_CONTEXT);     
    }
    else if (wcscmp(sourceName, SYNC_CONTACTS) == 0) {
            wsprintf(rootContext, CONTACTS_CONTEXT);     
            wsprintf(rootContextProps, CONTACTS_SOURCEPROP_CONTEXT);     
    }
    else if (wcscmp(sourceName, SYNC_TASKS) == 0) {
            wsprintf(rootContext, TASKS_CONTEXT);     
            wsprintf(rootContextProps, TASKS_SOURCEPROP_CONTEXT);    
    }
    else if (wcscmp(sourceName, SYNC_NOTES) == 0) {
            wsprintf(rootContext, NOTES_CONTEXT);  
            wsprintf(rootContextProps, NOTES_SOURCEPROP_CONTEXT);            
    }
    else if (wcscmp(sourceName, SYNC_FILES) == 0) {
            wsprintf(rootContext, BRIEFCASE_CONTEXT);   
            wsprintf(rootContextProps, BRIEFCASE_SOURCEPROP_CONTEXT);                      
    }
    else if (wcscmp(sourceName, SYNC_FAVORITES) == 0) {
            wsprintf(rootContext, FAVORITES_CONTEXT);     
    }
    else if (wcscmp(sourceName, SYNC_MAILS) == 0) {
            wsprintf(rootContext, MAILS_CONTEXT);    
            wsprintf(rootContextProps, MAILS_SOURCEPROP_CONTEXT);
        }
   
    if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SYNC) == 0) {     
        setProperty(rootContext, PROPERTY_SOURCE_SYNC, propertyValue);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_LAST) == 0) {     
        setProperty(rootContext, PROPERTY_SOURCE_LAST_SYNC, propertyValue);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_URI) == 0) {
        setProperty(rootContext, PROPERTY_SOURCE_URI, propertyValue);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DIR) == 0) {

        wchar_t t [256];
        wsprintf(t, propertyValue);

        wchar_t* loc = wcschr(t, L'\\');
        if (loc != NULL)
            wsprintf(t, loc);
        else
            wsprintf(t, TEXT("\\"));
        setProperty(rootContextProps, PROPERTY_SOURCE_DIR, propertyValue);
                
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_INBOX) == 0) {
        setProperty(rootContext, PROPERTY_SOURCE_INBOX, propertyValue);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_OUTBOX) == 0) {
        setProperty(rootContext, PROPERTY_SOURCE_OUTBOX, propertyValue);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_SENT) == 0) {
        setProperty(rootContext, PROPERTY_SOURCE_SENT, propertyValue);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DRAFT) == 0) {
        setProperty(rootContext, PROPERTY_SOURCE_DRAFT, propertyValue);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_TRASH) == 0) {
        setProperty(rootContext, PROPERTY_SOURCE_TRASH, propertyValue);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_ATTACH_SIZE) == 0) {
        setProperty(rootContext, PROPERTY_SOURCE_ATTACH_SIZE, propertyValue);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_DOWNLOAD_AGE) == 0) {
        setProperty(rootContext, PROPERTY_SOURCE_DOWNLOAD_AGE, propertyValue);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_BODY_SIZE) == 0) {
        setProperty(rootContext, PROPERTY_SOURCE_BODY_SIZE, propertyValue);
    }
    else if (wcscmp(propertyName, PROPERTY_SOURCE_SPDM_USE_SIF) == 0) {
        setProperty(rootContextProps, PROPERTY_SOURCEPROP_USE_SIF, propertyValue);
    }
    
    
    wsprintf(errorMsg, TEXT("success"));
    errorMsg[wcslen(errorMsg)] = L' ';
    
    return 0;

}

DWORD WINAPI setClientConfiguration(wchar_t* sourceName, wchar_t* propertyName, 
                                     wchar_t* propertyValue, wchar_t* errorMsg)  {
    
    wchar_t root_context[256];
    wmemset(root_context, 0, 256);
    wcscpy(root_context, ROOT_CONTEXT);

    if (sourceName == NULL || wcscmp(sourceName, TEXT("")) == 0) {
        
        if (wcscmp(propertyName, PROPERTY_SPDM_LOG_LEVEL) == 0)
            setProperty(root_context, PROPERTY_LOG_LEVEL, propertyValue);
        else  if (wcscmp(propertyName, PROPERTY_SPDM_ENCRYPT) == 0)
            setProperty(root_context, PROPERTY_ENCRYPT, propertyValue);
        else  if (wcscmp(propertyName, PROPERTY_SPDM_PATH_INFO) == 0)
            setProperty(root_context, PROPERTY_PATH_INFO, propertyValue);
        else if (wcscmp(propertyName, PROPERTY_SPDM_PUSH_NOTIFICATION) == 0)
            setProperty(root_context, PROPERTY_PUSH_NOTIFICATION, propertyValue);
        else if (wcscmp(propertyName, PROPERTY_SPDM_POLLING_NOTIFICATION) == 0)
            setProperty(root_context, PROPERTY_POLLING_NOTIFICATION, propertyValue);
         else if (wcscmp(propertyName, PROPERTY_SPDM_SERVER_NOTIFIED) == 0)
            setProperty(root_context, PROPERTY_SERVER_NOTIFIED, propertyValue);
                                   
    }
    return 0;
}

/*
* The method recall the other getClientConfiguration and set the variable isFromDB as true.
* This variable means that the caller is the VB and so it's necessary remove the final char '\0' 
* to permit the vb to understand the return string properly
*/

DWORD WINAPI getClientConfiguration (wchar_t* sourceName, wchar_t* propertyName, 
                                     wchar_t* propertyValue) {   

    return getClientConfigurationInternal (sourceName, propertyName, propertyValue, TRUE);
}

/*
* Method to get the client configurations. These property are set by installing process.
* These are extra parameters not used in synchronization process 
* but are needed for example to show or hide some fields in GUI interface.
* 
* @param sourceName     : source name of which get parameter's value
* @param propertyName   : name of property
* @param propertyValue  : value of property
*/

DWORD getClientConfigurationInternal (wchar_t* sourceName, wchar_t* propertyName, 
                                     wchar_t* propertyValue, BOOL isFromVB) {    

    const int VALUESIZE = 1024;
    int nc = 0;
    wchar_t context[DIM_MANAGEMENT_PATH];

    wchar_t root_context[256];
    wmemset(root_context, 0, 256);
    wcscpy(root_context, ROOT_CONTEXT);

    wchar_t buf[1024]; 

    wsprintf(context, TEXT("%s/%s"), ROOT_CONTEXT, CONTEXT_SPDS_SOURCESPROP);
       
    /*
    * if sourceName is empty then is a property of root context
    */
    if (sourceName == NULL || wcscmp(sourceName, TEXT("")) == 0) {
        
        if (wcscmp(propertyName, PROPERTY_SPDM_REMOTE_SETTINGS) == 0)
            getProperty(root_context, PROPERTY_REMOTE_SETTINGS, buf, VALUESIZE); 
        else if (wcscmp(propertyName, PROPERTY_SPDM_LOG_LEVEL) == 0)
            getProperty(root_context, PROPERTY_LOG_LEVEL, buf, VALUESIZE); 
        else if (wcscmp(propertyName, PROPERTY_SPDM_ENCRYPT) == 0)
            getProperty(root_context, PROPERTY_ENCRYPT, buf, VALUESIZE); 
        else if (wcscmp(propertyName, PROPERTY_SPDM_PATH_INFO) == 0)
            getProperty(root_context, PROPERTY_PATH_INFO, buf, VALUESIZE); 
        else if (wcscmp(propertyName, PROPERTY_SPDM_PUSH_NOTIFICATION) == 0)
            getProperty(root_context, PROPERTY_PUSH_NOTIFICATION, buf, VALUESIZE);
        else if (wcscmp(propertyName, PROPERTY_SPDM_POLLING_NOTIFICATION) == 0)
            getProperty(root_context, PROPERTY_POLLING_NOTIFICATION, buf, VALUESIZE);
        else if (wcscmp(propertyName, PROPERTY_SPDM_SERVER_NOTIFIED) == 0)
            getProperty(root_context, PROPERTY_SERVER_NOTIFIED, buf, VALUESIZE);
        
        else
            wsprintf(buf, TEXT(""));
                   
        wcscpy(propertyValue, buf);                                
        
        if (isFromVB != NULL && isFromVB == TRUE)
            propertyValue[wcslen(propertyValue)] = L' ';
        
        return 0;
    }               

    if (wcscmp(sourceName, SYNC_APPOINTMENTS) == 0) {
        wsprintf(context,APPOINTMENTS_SOURCEPROP_CONTEXT);        
    }
    else if (wcscmp(sourceName, SYNC_CONTACTS) == 0) {
        wsprintf(context,CONTACTS_SOURCEPROP_CONTEXT);
    }
    else if (wcscmp(sourceName, SYNC_TASKS) == 0) {
        wsprintf(context,TASKS_SOURCEPROP_CONTEXT);       
    }
    else if (wcscmp(sourceName, SYNC_FILES) == 0) {
        wsprintf(context,BRIEFCASE_SOURCEPROP_CONTEXT); 
    }
    else if (wcscmp(sourceName, SYNC_NOTES) == 0) {
       wsprintf(context,NOTES_SOURCEPROP_CONTEXT); 
    }
    else if (wcscmp(sourceName, SYNC_FAVORITES) == 0) {
        wsprintf(context,FAVORITES_SOURCEPROP_CONTEXT);       
    }
    else if (wcscmp(sourceName, SYNC_MAILS) == 0) {
        wsprintf(context,MAILS_SOURCEPROP_CONTEXT);       
    }
    
    if (wcscmp(propertyName, PROPERTY_SOURCEPROP_SPDM_SHOW) == 0) {       
        getProperty(context, PROPERTY_SOURCEPROP_SHOW, buf, VALUESIZE); 
    }
    else {
        wsprintf(buf, TEXT(""));                   
    }

    wcscpy(propertyValue, buf);                                
    propertyValue[wcslen(propertyValue)] = L' ';
     
    return 0;    
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////             READ/WRITE FUNCTION INTO DM REGISTRY            /////////////////////////////////     
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void convertSlashes(wchar_t* str) {
    int i, l;

    l = wcslen(str);

    for (i=0; i<l; ++i) {
        if (str[i] == L'/') {
            str[i] = L'\\';
        }
    }
}

void normalizeFullContext(wchar_t* str) {

    convertSlashes(str);
    if (wcsstr(str, TEXT("Software")) == NULL) {

        wchar_t s[512];
        wsprintf(s, TEXT("Software\\")); 
        wcscat(s, str);
        wmemset(str, 0, wcslen(str));
        wcscpy(str, s);

    }
}


/*
 * Returns the value of the given property
 * if buf is NULL, the value is returned as a new wchar_t array and must be fred by the user
 *
 * @param property - the property name
 * @param buf - the buffer for the value
 * @param len - the buffer size
 */
wchar_t* getProperty(wchar_t* fullContext, wchar_t* property, wchar_t* buf, int size) {
    HKEY key = NULL;    
    DWORD res;	
    long err = 0;
    
    ULONG dim = 512;   

    normalizeFullContext(fullContext);

    RegCreateKeyEx(
        HKEY_LOCAL_MACHINE,
        fullContext,
        0,
        NULL,
        REG_OPTION_NON_VOLATILE,
        KEY_ALL_ACCESS,
        NULL,
        &key,
        &res
    );

    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);
        goto finally;
    }

    err = RegQueryValueEx(
        key,
        property,
        NULL,
        NULL,  // we currently support only strings
        (UCHAR*)buf,
        &dim
    );
    /*
    if (buf == NULL) {
        buf = new wchar_t[dim + 1];
    
        err = RegQueryValueEx(
            key,
            property,
            NULL,
            NULL,  // we currently support only strings
            (UCHAR*)buf,
            &dim
        );
    }
    */
    
    if (err == ERROR_SUCCESS) {
        ; //MessageBox(NULL,  TEXT("Correct"), TEXT("getConfigParameter"), MB_OK);
    } else {		
        ;//MessageBox(NULL,  TEXT("Error"), TEXT("getConfigParameter"), MB_OK);
        wsprintf(buf, TEXT(""));
    }

    finally:

    if (key != 0) {
       RegCloseKey(key);
    }   
    return NULL;
}


/*
 * Sets a property value.
 *
 * @param property - the property name
 * @param value - the property value (zero terminated string)
 */
void setProperty(wchar_t* fullContext, wchar_t* property, wchar_t* value) {
    
    HKEY key;
    DWORD res;
    normalizeFullContext(fullContext);

    RegCreateKeyEx(
        HKEY_LOCAL_MACHINE,
        fullContext,
        0,
        NULL,
        REG_OPTION_NON_VOLATILE,
        KEY_ALL_ACCESS,
        NULL,
        &key,
        &res
    );


    if (key == 0) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, TEXT("Invalid context path: %s"), fullContext);
        goto finally;
    }

    RegSetValueEx(
        key,
        property,
        NULL,
        REG_SZ,  // we currently support only strings
        (UCHAR*)value,
        (wcslen(value)+1)*sizeof(wchar_t*)
    );


    finally:

    if (key != 0) {
       RegCloseKey(key);
    }
}
