/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//
// @author Marco Magistrali
//

#include "pim/TaskBuilder.h"

/*
* Method to populate the task XML structure. 
* It get every property name and value and build the XML structure to exchange with server.
*
* @param taskStringItem : the string that will contain the xml structure
* @param pTask          : the task object that contain every value
*
*/

void populateTaskStringItem (string &taskStringItem, ITask *pTask) {

    BSTR element = NULL;
    long elementLong = 0;
    DATE date = NULL;
    string element_s = string();

    VARIANT_BOOL variantBool = NULL; // 0==false -1==true
    wchar_t tempDate [DIM_LOCAL_TEMP];

    wchar_t* charBlob = NULL;
        
    taskStringItem = TEXT("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    taskStringItem = taskStringItem + TEXT("<task>");
            
        // BSTR
        pTask->get_Subject(&element);
        element_s = string(element); encodeSpecialChar(element_s);
        taskStringItem  = taskStringItem + TEXT("<Subject>") + element_s + TEXT("</Subject>");
                    
        pTask->get_Categories(&element);
        element_s = string(element); encodeSpecialChar(element_s);
        taskStringItem  = taskStringItem + TEXT("<Categories>") + element_s + TEXT("</Categories>");

        pTask->get_ReminderSoundFile(&element);
        element_s = string(element); encodeSpecialChar(element_s);
        taskStringItem  = taskStringItem + TEXT("<ReminderSoundFile>") + element_s + TEXT("</ReminderSoundFile>");
    
        pTask->get_Body(&element);
        removeEndCarriage(&element);
        element_s = string(element); encodeSpecialChar(element_s);
        taskStringItem  = taskStringItem + TEXT("<Body>") + element_s + TEXT("</Body>");                  
                    

        // VARIANT_BOOL
        pTask->get_ReminderSet(&variantBool);
        wsprintf(tempDate, TEXT("%i"), variantBool);
        normalizeBoolean(tempDate);
        taskStringItem  = taskStringItem + TEXT("<ReminderSet>") + tempDate + TEXT("</ReminderSet>");   

        pTask->get_Complete(&variantBool);
        wsprintf(tempDate, TEXT("%i"), variantBool);
        normalizeBoolean(tempDate);
        taskStringItem  = taskStringItem + TEXT("<Complete>") + tempDate + TEXT("</Complete>"); 

        pTask->get_TeamTask(&variantBool);
        wsprintf(tempDate, TEXT("%i"), variantBool);
        normalizeBoolean(tempDate);
        taskStringItem  = taskStringItem + TEXT("<TeamTask>") + tempDate + TEXT("</TeamTask>"); 
                
                        
        // DATE
        pTask->get_StartDate(&date);

        wsprintf(tempDate, TEXT(""));           
        double toCompare = MAX_DATE_DOUBLE;
        double toCompare4000 = MAX_DATE_DOUBLE_4000;
        
        if (date != toCompare) { 
                doubleToSystemTime(tempDate, date);

        }
        taskStringItem  = taskStringItem + TEXT("<StartDate>") + tempDate + TEXT("</StartDate>");               


        pTask->get_DueDate(&date);

        wsprintf(tempDate, TEXT(""));
        if (date != toCompare) { 
                doubleToSystemTime(tempDate, date);
        }
        taskStringItem  = taskStringItem + TEXT("<DueDate>") + tempDate + TEXT("</DueDate>");
        
        /*
        pTask->get_DateCompleted(&date);

        wsprintf(tempDate, TEXT(""));
        if (date != toCompare) { 
                doubleToSystemTime(tempDate, date);
        }
        taskStringItem  = taskStringItem + TEXT("<DateCompleted>") + tempDate + TEXT("</DateCompleted>");
        */

        pTask->get_ReminderTime(&date);

        wsprintf(tempDate, TEXT(""));
        if (date != toCompare) { 
                doubleToSystemTime(tempDate, date);
        }
        taskStringItem  = taskStringItem + TEXT("<ReminderTime>") + tempDate + TEXT("</ReminderTime>");
        

        // long 
        
        pTask->get_Sensitivity(&elementLong);           
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<Sensitivity>") + tempDate + TEXT("</Sensitivity>");
        
        pTask->get_ReminderOptions(&elementLong);               
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<ReminderOptions>") + tempDate + TEXT("</ReminderOptions>");
                
        pTask->get_Importance(&elementLong);            
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<Importance>") + tempDate + TEXT("</Importance>");

        /******************************************************/
        //Recurrence
        pTask->get_IsRecurring(&variantBool);           
        wsprintf(tempDate, TEXT("%i"), variantBool);
        normalizeBoolean(tempDate);
        taskStringItem  = taskStringItem + TEXT("<IsRecurring>") + tempDate + TEXT("</IsRecurring>");
        
        IRecurrencePattern *pRecurrence;
        pTask->GetRecurrencePattern (&pRecurrence);

        pRecurrence->get_RecurrenceType(&elementLong);                  
        if (elementLong < 0) {
            elementLong = 0;
        }
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<RecurrenceType>") + tempDate + TEXT("</RecurrenceType>");

        pRecurrence->get_Interval(&elementLong);                
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<Interval>") + tempDate + TEXT("</Interval>");

        pRecurrence->get_MonthOfYear(&elementLong);             
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<MonthOfYear>") + tempDate + TEXT("</MonthOfYear>");

        pRecurrence->get_DayOfMonth(&elementLong);              
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<DayOfMonth>") + tempDate + TEXT("</DayOfMonth>");
        
        pRecurrence->get_DayOfWeekMask(&elementLong);           
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<DayOfWeekMask>") + tempDate + TEXT("</DayOfWeekMask>");

        pRecurrence->get_Instance(&elementLong);                
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<Instance>") + tempDate + TEXT("</Instance>");

        pRecurrence->get_PatternStartDate(&date);

        wsprintf(tempDate, TEXT(""));           
        toCompare = MAX_DATE_DOUBLE;
        
        if (date != toCompare && date < toCompare4000) { 
                doubleToSystemTime(tempDate, date);                     
        }
        taskStringItem  = taskStringItem + TEXT("<PatternStartDate>") + tempDate + TEXT("</PatternStartDate>");         
        
        pRecurrence->get_NoEndDate(&variantBool);
        wsprintf(tempDate, TEXT("%i"), variantBool);
        normalizeBoolean(tempDate);
        taskStringItem  = taskStringItem + TEXT("<NoEndDate>") + tempDate + TEXT("</NoEndDate>");       

        pRecurrence->get_PatternEndDate(&date);

        wsprintf(tempDate, TEXT(""));           
        toCompare = MAX_DATE_DOUBLE;

        if (date != toCompare && date < toCompare4000 ) { 
                doubleToSystemTime(tempDate, date);             
        }
        taskStringItem  = taskStringItem + TEXT("<PatternEndDate>") + tempDate + TEXT("</PatternEndDate>");             
        
        pRecurrence->get_Occurrences(&elementLong);          
        
        if (elementLong < 0) {
            elementLong = 0;
        }
        wsprintf(tempDate, TEXT("%i"), elementLong);
        taskStringItem  = taskStringItem + TEXT("<Occurrences>") + tempDate + TEXT("</Occurrences>");

    taskStringItem = taskStringItem +  TEXT("</task>");
            
    if (dbg)
            MessageBox (NULL, taskStringItem, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);

}


/*
* Method that complete an task object retrieving property value by an XML structure.
* It parse the XML structure, get the every property value using its name and set it into the object.
*
* @param pTask      : the task object that contain every value
* @param ptrData    : the variable that contain the xml structure
*/

void completeTask  (ITask *pTask, wchar_t * ptrData) {

    wchar_t* dummy = NULL;
    wchar_t* dummyEvent = NULL;             
    string   dummyEvent_s           = string();
    pTask->ClearRecurrencePattern ();
    
    //VARIANT_BOOL -- ReminderSet 
    dummyEvent = getElementContent(ptrData, TEXT ("ReminderSet"), NULL);            
    if (dummyEvent != NULL && 
        (wcscmp(dummyEvent, TEXT("1")) == 0 ||
         wcscmp(dummyEvent, TEXT("-1")) == 0 || 
         wcscmp(dummyEvent, TEXT("True")) == 0)) {
        pTask->put_ReminderSet(VARIANT_TRUE);
    }
    else {
        pTask->put_ReminderSet(VARIANT_FALSE);  
    }               
    if (dummyEvent != NULL)
    delete dummyEvent;
    dummyEvent=NULL;            
    
    dummyEvent = getElementContent(ptrData, TEXT ("Complete"), NULL);           
    if (dummyEvent != NULL && 
        (wcscmp(dummyEvent, TEXT("1")) == 0 ||
         wcscmp(dummyEvent, TEXT("-1")) == 0 || 
         wcscmp(dummyEvent, TEXT("True")) == 0)) {
        pTask->put_Complete(VARIANT_TRUE);
    }
    else {
        pTask->put_Complete(VARIANT_FALSE);     
    }

    if (dummyEvent != NULL)
        delete dummyEvent;dummyEvent=NULL;      
    
    dummyEvent = getElementContent(ptrData, TEXT ("TeamTask"), NULL);           
    if (dummyEvent != NULL && 
        (wcscmp(dummyEvent, TEXT("1")) == 0 ||
         wcscmp(dummyEvent, TEXT("-1")) == 0 || 
         wcscmp(dummyEvent, TEXT("True")) == 0)) {
        pTask->put_TeamTask(VARIANT_TRUE);
    }
    else {
        pTask->put_TeamTask(VARIANT_FALSE);     
    }
    if (dummyEvent != NULL)
        delete dummyEvent; dummyEvent=NULL;

        
    // BSTR
    dummyEvent = getElementContent(ptrData, TEXT ("Subject"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = string(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pTask->put_Subject ((wchar_t*)dummyEvent_s);
        delete dummyEvent;  dummyEvent=NULL;
    }
        
    dummyEvent = getElementContent(ptrData, TEXT ("Categories"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = string(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pTask->put_Categories ((wchar_t*)dummyEvent_s);
        delete dummyEvent; dummyEvent=NULL;
    }
    
    dummyEvent = getElementContent(ptrData, TEXT ("ReminderSoundFile"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = string(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pTask->put_ReminderSoundFile  ((wchar_t*)dummyEvent_s);
        delete dummyEvent; dummyEvent=NULL;
    }
    
    dummyEvent = getElementContent(ptrData, TEXT ("Body"), NULL);
    if (dummyEvent != NULL) {
        dummyEvent_s = string(dummyEvent); decodeSpecialChar(dummyEvent_s);
        pTask->put_Body  ((wchar_t*)dummyEvent_s);   
        delete dummyEvent;  dummyEvent=NULL;                                    
    }   

    wchar_t localTemp[DIM_LOCAL_TEMP];
    //DATE
    dummyEvent = getElementContent(ptrData, TEXT ("StartDate"), NULL);
    if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
        ;
    else {
        DATE st;
        systemTimeToDouble(dummyEvent, &st, NULL);
        if (dbg) {
                wsprintf(localTemp, TEXT("st %f"), st); 
                MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
    pTask->put_StartDate(st);
    delete dummyEvent;
    }
    
    dummyEvent = getElementContent(ptrData, TEXT ("DueDate"), NULL);
    if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
        ;
    else {
        DATE dd;
        systemTimeToDouble(dummyEvent, &dd, NULL);
        if (dbg) {
                wsprintf(localTemp, TEXT("dd %f"), dd); 
                MessageBox (NULL, localTemp, TEXT ("Debug eVC"), MB_SETFOREGROUND |MB_OK);
        }
    pTask->put_DueDate(dd);
    delete dummyEvent;
    }



        //long
    dummyEvent = getElementContent(ptrData, TEXT ("Importance"), NULL);
    if (dummyEvent != NULL) {
        pTask->put_Importance(wcstol(dummyEvent, &dummy, 0));    
        delete dummyEvent; dummyEvent=NULL;
    }

    dummyEvent = getElementContent(ptrData, TEXT ("Sensitivity"), NULL);
    if (dummyEvent != NULL) {
        pTask->put_Sensitivity(wcstol(dummyEvent, &dummy, 0));    
        delete dummyEvent; dummyEvent=NULL;
    }


    dummyEvent = getElementContent(ptrData, TEXT ("ReminderOptions"), NULL);
    if (dummyEvent != NULL) {
        pTask->put_ReminderOptions (wcstol(dummyEvent, &dummy, 0));
        delete dummyEvent; dummyEvent=NULL;
    }
        
    /****************************************************/
    
    //Recurrence
    dummyEvent = getElementContent(ptrData, TEXT ("IsRecurring"), NULL);
    
    if (dbg) {                      
        MessageBox (NULL, dummyEvent, TEXT ("Is Task Recurring"), MB_SETFOREGROUND |MB_OK);
    }

    if (dummyEvent != NULL && (wcscmp(dummyEvent, TEXT("1")) == 0 ||
                               wcscmp(dummyEvent, TEXT("-1")) == 0 ||
                               wcscmp(dummyEvent, TEXT("True")) == 0)) {                    
        
        IRecurrencePattern *pRecurrence;
        pTask->GetRecurrencePattern (&pRecurrence);

        dummyEvent = getElementContent(ptrData, TEXT ("RecurrenceType"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) >= 0) {
            pRecurrence->put_RecurrenceType(wcstol(dummyEvent, &dummy, 0));                 
            delete dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Interval"), NULL);       
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) >= 0) {
            pRecurrence->put_Interval(wcstol(dummyEvent, &dummy, 0));               
            delete dummyEvent;dummyEvent=NULL;
        }
            
        dummyEvent = getElementContent(ptrData, TEXT ("MonthOfYear"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
            pRecurrence->put_MonthOfYear(wcstol(dummyEvent, &dummy, 0));                    
            delete dummyEvent;dummyEvent=NULL;
        }
                
        dummyEvent = getElementContent(ptrData, TEXT ("DayOfMonth"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
            pRecurrence->put_DayOfMonth(wcstol(dummyEvent, &dummy, 0));                     
            delete dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("DayOfWeekMask"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
            pRecurrence->put_DayOfWeekMask(wcstol(dummyEvent, &dummy, 0));                  
            delete dummyEvent;dummyEvent=NULL;
        }

        dummyEvent = getElementContent(ptrData, TEXT ("Instance"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) > 0) {
            pRecurrence->put_Instance(wcstol(dummyEvent, &dummy, 0));                       
            delete dummyEvent;dummyEvent=NULL;
        }

                
        //Pattern Start Date
        dummyEvent = getElementContent(ptrData, TEXT ("PatternStartDate"), NULL);

        if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
            ;
        else {
            DATE st;
            systemTimeToDouble(dummyEvent, &st, NULL);

            pRecurrence->put_PatternStartDate(st);
            delete dummyEvent;
        }

        //VARIANT_BOOL -- NoEndDate
        dummyEvent = getElementContent(ptrData, TEXT ("NoEndDate"), NULL);
        

        if (dummyEvent != NULL && (wcscmp(dummyEvent, TEXT("1")) == 0 ||
                                   wcscmp(dummyEvent, TEXT("-1")) == 0 || 
                                   wcscmp(dummyEvent, TEXT("True")) == 0)) {
            pRecurrence->put_NoEndDate(VARIANT_TRUE);

        } else {
            pRecurrence->put_NoEndDate(VARIANT_FALSE);

            //Pattern End Date
            dummyEvent = getElementContent(ptrData, TEXT ("PatternEndDate"), NULL);

            if (dummyEvent == NULL || (wcscmp(dummyEvent, TEXT("")) == 0))
                ;
            else {
                DATE en;
                systemTimeToDouble(dummyEvent, &en, NULL);

                pRecurrence->put_PatternEndDate(en);
                delete dummyEvent;
            }

        }
        dummyEvent = getElementContent(ptrData, TEXT ("Occurrences"), NULL);
        if (dummyEvent != NULL && wcstol(dummyEvent, &dummy, 0) >= 0) {
            pRecurrence->put_Occurrences(wcstol(dummyEvent, &dummy, 0));                 
            delete dummyEvent;dummyEvent=NULL;
        }

    } else {
            ;// task is not recurring...
    }

}

/*
* The method calculate the CRC hash value for the task item.
* It create the XML structure and calculate the crc value.
* Use to compare for understand the modified value!
*
* @param pTask : the task object that contain every value
*/

long calculateTaskHash (ITask *pTask) {

    CCrc32 cc = CCrc32();
    string p = string();
    BSTR element = NULL;    

    populateTaskStringItem(p, pTask);       
    
    BSTR dummy = p;

    long l = cc.crc32String( &dummy);
        
    return l;
}

VObject* TaskToVObject(ITask *pTask) {
	VObject* vo = new VObject();

	wchar_t* dateValue;
	DATE startdate;
	DATE duedate;
	DATE completeddate;
	BSTR element;

	vo->addProperty(new VProperty(TEXT("BEGIN"), TEXT("VCALENDAR")));
	vo->addProperty(new VProperty(TEXT("BEGIN"), TEXT("VTODO")));
	
	pTask->get_Subject(&element);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("SUMMARY"), element));

	pTask->get_Body(&element);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("DESCRIPTION"), element));
	
	pTask->get_StartDate(&startdate);
	dateValue = new wchar_t[20];
	doubleToSystemTime(dateValue, startdate);
	vo->addProperty(new VProperty(TEXT("DTSTART"), dateValue));
	delete [] dateValue; dateValue = NULL;

	pTask->get_DueDate(&duedate);
	dateValue = new wchar_t[20];
	doubleToSystemTime(dateValue, duedate);
	vo->addProperty(new VProperty(TEXT("DUE"), dateValue));
	delete [] dateValue; dateValue = NULL;

	VARIANT_BOOL completed;
	pTask->get_Complete(&completed);

	if(completed) {
		pTask->get_DateCompleted(&completeddate);
		dateValue = new wchar_t[20];
		doubleToSystemTime(dateValue, completeddate);
		vo->addProperty(new VProperty(TEXT("COMPLETED"), dateValue));
	}

	pTask->get_Categories(&element);
	if(wcscmp(element, TEXT("")))
		vo->addProperty(new VProperty(TEXT("CATEGORIES"), element));

	long sensitivity;
	pTask->get_Sensitivity(&sensitivity);
	if(sensitivity == olPrivate)
		vo->addProperty(new VProperty(TEXT("CLASS"), TEXT("PRIVATE")));
	else
		vo->addProperty(new VProperty(TEXT("CLASS"), TEXT("PUBLIC")));

	VARIANT_BOOL bReminder;
	pTask->get_ReminderSet(&bReminder);

	if(bReminder == VARIANT_TRUE) {
		DATE reminder;
		pTask->get_ReminderTime(&reminder);
		
		doubleToSystemTime(dateValue, reminder);
		
		wcscat(dateValue, TEXT(";;0;reminder"));
		vo->addProperty(new VProperty(TEXT("AALARM"), dateValue));
		delete [] dateValue; dateValue = NULL;
	}

	long importance;
	pTask->get_Importance(&importance);
	if(importance == olImportanceLow)
		vo->addProperty(new VProperty(TEXT("PRIORITY"), TEXT("3")));
	else if(importance == olImportanceNormal)
		vo->addProperty(new VProperty(TEXT("PRIORITY"), TEXT("2")));
	else if(importance == olImportanceHigh)
		vo->addProperty(new VProperty(TEXT("PRIORITY"), TEXT("1")));

	VARIANT_BOOL isRec;
	pTask->get_IsRecurring(&isRec);

	if(isRec == VARIANT_TRUE) {
		IRecurrencePattern* pRecPat;
		pTask->GetRecurrencePattern(& pRecPat);
		wchar_t* recRule = NULL;
		recRule = extractRrule(pRecPat);
		if(recRule)
			vo->addProperty(new VProperty(TEXT("RRULE"), recRule));
		pRecPat->Release();
	}	

	vo->addProperty(new VProperty(TEXT("END"), TEXT("VTODO")));
	vo->addProperty(new VProperty(TEXT("END"), TEXT("VCALENDAR")));

	return vo;
}
void VObjectToTask(ITask *pTask, VObject *vo) {
   
	int i = 0;

	pTask->ClearRecurrencePattern ();

	if(vo->containsProperty(TEXT("SUMMARY")) && vo->getProperty(TEXT("SUMMARY"))->getValue()) 
		pTask->put_Subject(vo->getProperty(TEXT("SUMMARY"))->getValue());

	if(vo->containsProperty(TEXT("DESCRIPTION")) && vo->getProperty(TEXT("DESCRIPTION"))->getValue()) 
		pTask->put_Body(vo->getProperty(TEXT("DESCRIPTION"))->getValue());
	
	DATE startdate = NULL;
    DATE duedate = NULL;

	if(vo->containsProperty(TEXT("DTSTART")) && vo->getProperty(TEXT("DTSTART"))->getValue()) {
		
		systemTimeToDouble(vo->getProperty(TEXT("DTSTART"))->getValue(), &startdate, NULL);
		pTask->put_StartDate(startdate);
		}

	if(vo->containsProperty(TEXT("DUE")) && vo->getProperty(TEXT("DUE"))->getValue()) {
		
		systemTimeToDouble(vo->getProperty(TEXT("DUE"))->getValue(), &duedate, NULL);
		pTask->put_DueDate(duedate);
		}

	if(vo->containsProperty(TEXT("CATEGORIES")) && vo->getProperty(TEXT("CATEGORIES"))->getValue()) 
		pTask->put_Categories(vo->getProperty(TEXT("CATEGORIES"))->getValue());


	if(vo->containsProperty(TEXT("CLASS")) && vo->getProperty(TEXT("CLASS"))->getValue()) {
		wchar_t* calClass = new wchar_t[20];
		wcscpy(calClass, vo->getProperty(TEXT("CLASS"))->getValue());
		if(!wcscmp(calClass, TEXT("PRIVATE")) || !wcscmp(calClass, TEXT("CONFIDENTIAL")))
			pTask->put_Sensitivity(olPrivate);
		else
			pTask->put_Sensitivity(olNormal);

		delete [] calClass;
	}
	else
		pTask->put_Sensitivity(olNormal);

	//alarm
	//The value consists of: Run Time, Snooze Time; Repeat Count, Audio Content
	if(vo->containsProperty(TEXT("AALARM")) && vo->getProperty(TEXT("AALARM"))->getValue()) {
		
		DATE runTime;
		systemTimeToDouble(vo->getProperty(TEXT("AALARM"))->getPropComponent(1), &runTime, NULL);

		pTask->put_ReminderSet(VARIANT_TRUE);
		pTask->put_ReminderTime(runTime);

	}
	
	if(vo->containsProperty(TEXT("COMPLETED")) && vo->getProperty(TEXT("COMPLETED"))->getValue()) {
		pTask->put_Complete(VARIANT_TRUE);
	}
	
	if(vo->containsProperty(TEXT("PRIORITY")) && vo->getProperty(TEXT("PRIORITY"))->getValue()) {
		wchar_t* prior = vo->getProperty(TEXT("PRIORITY"))->getValue();
		long importance;

		if(!wcscmp(prior, TEXT("1")))
			importance = olImportanceHigh;
		else if(!wcscmp(prior, TEXT("2")))
			importance = olImportanceNormal;
		else if(!wcscmp(prior, TEXT("3")))
			importance = olImportanceLow;
	}
	
	//recurrence
	if((vo->containsProperty(TEXT("RRULE")) && vo->getProperty(TEXT("RRULE"))->getValue())) {

		pTask->ClearRecurrencePattern();
		
		IRecurrencePattern* pRecPat;
		pTask->GetRecurrencePattern(& pRecPat);
		
		
		DATE startDate;
		pTask->get_StartDate(&startDate);
		bool ret = convertRrule(pRecPat, vo->getProperty(TEXT("RRULE"))->getValue(), startDate);

		if(!ret)
			pTask->ClearRecurrencePattern();
		
		pRecPat->Release();
	
	}

}