/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pim/WindowsSyncSource.h"
#include "spds/SyncItemStatus.h"
#include "base/util/utils.h"
#include "base/Log.h"
#include "pim/POOMClient2Server.h"
#include "pim/POOMServer2Client.h"

static int call;
static int cnew;
static int ckey;
static int cupdated;
static int cdeleted;

WindowsSyncSource::WindowsSyncSource(const wchar_t* name) : SyncSource(name) {
    c = NULL;        
    setPath(TEXT("\\"));
    syncItem = NULL;
}

WindowsSyncSource::~WindowsSyncSource() {
    if (c) 
        delete c; 
    c = NULL;
    if (syncItem)
        delete syncItem;
    syncItem = NULL;
}

void WindowsSyncSource::setType(SourceType t) {
    type = t;
}

SourceType WindowsSyncSource::getType() {
    return type;
}


void WindowsSyncSource::setPath(const wchar_t* p) {
    if (p == NULL)
        path = stringdup(TEXT(""));
    else
        path = stringdup(p);
    
}

wchar_t* WindowsSyncSource::getPath() {
    return path;
}

/*
* Return the first SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
*/

SyncItem* WindowsSyncSource::getFirstItem() {
    
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL) {
        c = new Container();
    }

    setAllItemsPOOM(c, getType()); // olFolderContact = 10
    if (c->getAllItemsSize() == 0) {
        return NULL;
    }        
    call = 0;
    
    ArrayList* list = c->getAllItems();    
    syncItem = (SyncItem*)((SyncItem*)list->get(call))->clone();    
    fillSyncItem(syncItem, getType());
    return syncItem;

}

/*
* Return the next SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
*/

SyncItem* WindowsSyncSource::getNextItem() {        
    
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL)
        return NULL;

    if (c->getAllItemsSize() == 0)
        return NULL;

    call++;
    if (call == c->getAllItemsSize()) {
        return NULL;
    }     
    syncItem = (SyncItem*)((SyncItem*)c->getAllItems()->get(call))->clone();  
    fillSyncItem(syncItem, getType());
    return syncItem;
    
}

/*
* This method is always called by the sync manager. So the setModifiedItemsPOOM is always called
*/

SyncItem* WindowsSyncSource::getFirstNewItem() {
    
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL) {
        c = new Container();
    }

    setModifiedItemsPOOM(c, getType(), getPath()); // olFolderContact = 10
   
    if (c->getNewItemsSize() == 0) {
        return NULL;
    }    
    cnew = 0;
    /*
    ArrayList* list = c->getNewItems();
    syncItem = (SyncItem*)list->get(cnew);    
    fillSyncItem(syncItem, getType());
    return syncItem;
    */
    ArrayList* list = c->getNewItems();
    return (SyncItem*)list->get(cnew);    
    

}

SyncItem* WindowsSyncSource::getFirstItemKey() {

    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL) {
        c = new Container();
    }

    setAllItemsPOOM(c, getType()); // olFolderContact = 10
    if (c->getAllItemsSize() == 0) {
        return NULL;
    }        
    ckey = 0;
    ArrayList* list = c->getAllItems();    
    syncItem = (SyncItem*)((SyncItem*)list->get(ckey))->clone();    
    return syncItem;
}

SyncItem* WindowsSyncSource::getNextItemKey() {
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL)
        return NULL;

    if (c->getAllItemsSize() == 0)
        return NULL;

    ckey++;
    if (ckey == c->getAllItemsSize()) {
        return NULL;
    }     
    syncItem = (SyncItem*)((SyncItem*)c->getAllItems()->get(ckey))->clone();
    return syncItem;
}

SyncItem* WindowsSyncSource::getNextNewItem() {    
    
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL)
        return NULL;

    cnew++;
    if (cnew == c->getNewItemsSize()) {
        return NULL;
    }
    /*
    syncItem = (SyncItem*)c->getNewItems()->get(cnew);    
    fillSyncItem(syncItem, getType());
    return syncItem;  
    */
    return (SyncItem*)c->getNewItems()->get(cnew);    
}

SyncItem* WindowsSyncSource::getFirstUpdatedItem() {
    
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL) {
        c = new Container();
    }

    if (c->getUpdatedItemsSize() == 0) {
        return NULL;
    }    
    cupdated = 0;
    /*
    ArrayList* list = c->getUpdatedItems();
    syncItem = (SyncItem*)list->get(cupdated);          
    fillSyncItem(syncItem, getType());
    return syncItem;
    */

    ArrayList* list = c->getUpdatedItems();
    return (SyncItem*)list->get(cupdated);          
    
}

SyncItem* WindowsSyncSource::getNextUpdatedItem() {    
   
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    
    if (c == NULL)
        return NULL;

    cupdated++;
    if (cupdated == c->getUpdatedItemsSize()) {
        return NULL;
    }        
    /*
    syncItem = (SyncItem*)c->getUpdatedItems()->get(cupdated);         
    fillSyncItem(syncItem, getType());
    return syncItem;
    */
    
    return (SyncItem*)c->getUpdatedItems()->get(cupdated);         

}

SyncItem* WindowsSyncSource::getFirstDeletedItem() {
   
    if (c == NULL) {
        c = new Container();
    }

    if (c->getDeletedItemsSize() == 0) {
        return NULL;
    }    
    cdeleted = 0;
    ArrayList* list = c->getDeletedItems();

    return (SyncItem*)list->get(cdeleted);      
}

SyncItem* WindowsSyncSource::getNextDeletedItem() {    
    if (c == NULL)
        return NULL;

    cdeleted++;
    if (cupdated == c->getDeletedItemsSize()) {
        return NULL;
    }        
    return (SyncItem*)c->getDeletedItems()->get(cdeleted);    
}

void WindowsSyncSource::setItemStatus(const wchar_t* key, int status) {
    wchar_t tmp[256];
    wsprintf(tmp, TEXT("key: %s, status: %i"), key, status);
    LOG.debug(tmp);
}

int WindowsSyncSource::addItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    long oid = manageNewItems(&item, getType());
    
    wchar_t tmp[256];
    if (oid > 0) {        
        ret = STC_ITEM_ADDED;
        wsprintf(tmp, TEXT("Added item: %s LUID: %i"), item.getKey(), oid);
        LOG.debug(tmp);
        wsprintf(tmp, TEXT("%i"), oid);
        item.setKey(tmp);   
    }

    return ret;
}

int WindowsSyncSource::updateItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    long h = manageUpdatedItems(&item, getType());
    
    if (h == 0) {
        ret = STC_OK;
        wchar_t tmp[256];
        wsprintf(tmp, TEXT("updated item: %s"), item.getKey());
        LOG.debug(tmp);
    }
    return ret;
}

int WindowsSyncSource::deleteItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    long h = manageDeletedItems(&item, getType());
    
    if (h == 0) {
        ret = STC_OK;
        wchar_t tmp[256];
         wsprintf(tmp, TEXT("deleted item: %s"), item.getKey());
        LOG.debug(tmp);
    }
    return ret;    
}

int WindowsSyncSource::beginSync() {
    //
    // Nothing to do
    //
    return 0;
}

int WindowsSyncSource::endSync() {
        
    writeCurrentItems(getType(), getPath());
    return 0;
}

void WindowsSyncSource::assign(WindowsSyncSource& s) {
    setSyncMode(s.getSyncMode());
    setLastSync(s.getLastSync());
    setNextSync(s.getNextSync());
    setRemoteURI(s.getRemoteURI());
    setType(s.getType());
    setLastAnchor(s.getLastAnchor());
    setNextAnchor(s.getNextAnchor());
    setErrorHandler(s.getErrorHandler());
    setFilter(s.getFilter());
    setPath(getPath());
    setType(getType());
}