/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pim/mail/MailClient2Server.h"
#include "pim/mail/MailInfo.h"
#include "pim/mail/MailItem.h"
#include "pim/mail/MailClientData.h"
#include "pim/mail/MailSyncSource.h"
#include "pim/mail/MailExtraData.h"


#include "pim/utils.h"
#include "base/util/utils.h"
#include "base/util/ArrayList.h"
#include "base/Log.h"
#include <mapidefs.h>
#include <mapicode.h>
#include <mapitags.h>
#include <mapix.h>
#include <mapiutil.h>
#include <cemapi.h>
#include "wtypes.h"


long mailAll;
long mailNew;
long mailUpdated;
long mailDeleted;

MailClient2Server::MailClient2Server() {
    //mailMessage = NULL;
    mailAll = -2;
    arrayIndex = 0;
    allTot = 0;
    elementIndex = -2;
    
    mailNew = -2;
    mailUpdated = -2;
    mailDeleted = -2;
    folderNumber = 0;
    // deletedMessages = new ArrayList();
    
    entryListArray = NULL;
    folderToSync = NULL;
    newMessagesArray = NULL;
    updatedMessagesArray = NULL;
    deletedMessagesArray = NULL;

}

MailClient2Server::~MailClient2Server() {
    //if (mailMessage) {
    //    delete mailMessage; mailMessage = NULL;
    //}
    if (folderToSync) {
        delete [] folderToSync; folderToSync = NULL;
    }
    if (newMessagesArray) {
        for (int i = 0; i < folderNumber; i++)           
			//MAPIFreeBuffer(newMessagesArray[i].lpbin);
        delete [] newMessagesArray; newMessagesArray = NULL;
    }
    if (updatedMessagesArray) {
        for (int i = 0; i < folderNumber; i++)                    
	        //MAPIFreeBuffer(updatedMessagesArray[i].lpbin);

        delete [] updatedMessagesArray; updatedMessagesArray = NULL;
    }
    if (deletedMessagesArray) {       
        delete [] deletedMessagesArray; deletedMessagesArray = NULL;
    }
    if (entryListArray) {
        for (int i = 0; i < folderNumber; i++)           
			//MAPIFreeBuffer(entryListArray[i].lpbin);

        delete [] entryListArray; entryListArray = NULL;
    }
    
}

enum {
	eeePR_ENTRYID,
	eeePR_MESSAGE_FLAGS,
    eeePR_LAST_MODIFICATION_TIME,
	eeeNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eeeNUM_COLS, endSync) = 
{	
	eeeNUM_COLS,
    PR_ENTRYID,                       // entry id
	PR_MESSAGE_FLAGS,    
    PR_LAST_MODIFICATION_TIME        //

};

 enum {
	ePR_ENTRYID,
	ePR_MESSAGE_FLAGS,
    //MM        
    ePR_LAST_MODIFICATION_TIME,
    ePR_SUBJECT,
    ePR_SUBJECT_PREFIX,
    ePR_MESSAGE_SIZE,    
    ePR_SENDER_NAME,        
    ePR_SENDER_EMAIL_ADDRESS,    
    ePR_HASATTACH,
    ePR_MESSAGE_DELIVERY_TIME,
    //MM
	NUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(NUM_COLS, properties) = 
{	
	NUM_COLS,
    PR_ENTRYID,                         // entry id
	PR_MESSAGE_FLAGS,
    //MM        
    PR_LAST_MODIFICATION_TIME,          //
    PR_SUBJECT,                         // mail Subject
    PR_SUBJECT_PREFIX,
    PR_MESSAGE_SIZE,                    // sum of the size of every property of the object
    PR_SENDER_NAME,                     // "Marco "
    PR_SENDER_EMAIL_ADDRESS,            // From  "Marco <magi@libero.it>    
    PR_HASATTACH,
    PR_MESSAGE_DELIVERY_TIME

};

enum {
	eePR_ENTRYID,
    eePR_LAST_MODIFICATION_TIME,
	eePR_DISPLAY_NAME,     
    eePR_MESSAGE_FLAGS,
    eeNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eeNUM_COLS, Columns) = 
{	
	eeNUM_COLS,
	PR_ENTRYID,
    PR_LAST_MODIFICATION_TIME,
	PR_DISPLAY_NAME,
    PR_MESSAGE_FLAGS
};


/*
* The name of the object has to follow the rule in which the name is
* <FolderName>/<id>
* i.e. I/AAAAABBBCCC   Inbox/AAAAABBBCCC
*
* For update only, the completeObject send all the complete message for Draft folder.
* The others sends only the information extra files
*
*
*/

MailClientData* MailClient2Server::completeObject(IMessage* pmsg, wchar_t folder, BOOL isUpdate) {
    
    ULONG             cmsgValue   =  0;
    SPropValue*       msgprops    =  NULL;
    wchar_t* to = NULL, *cc = NULL, *bcc = NULL;
    wchar_t* body = NULL;
    BodyPart bodyPart;
    wchar_t* ret = NULL;
    wchar_t* r = NULL;
    HRESULT hr; 
    FILETIME ft;    
    bool read = false;
    bool isDraft = false;
    ULONG attachCount = 0;
    BasicTime b;

    MailClientData* mcd = NULL;
    wchar_t time[20];

    hr = pmsg->GetProps ((LPSPropTagArray)&properties, MAPI_UNICODE, &cmsgValue,
       &msgprops);
    EXIT_ON_FAILED (hr);           
        
    if (msgprops == NULL)
        return NULL;
    
    mcd = new MailClientData();
    
    if (msgprops[ePR_ENTRYID].ulPropTag == PR_ENTRYID) {
        ret = convertBinaryToWChar(msgprops[ePR_ENTRYID].Value.bin, folder);
        mcd->getEmailData()->getEmailItem().setEntryID(ret);            
        if (ret) {
            delete [] ret; ret = NULL;
        }
    }
    
    r = c2wc(folder);
    mcd->setParent(r);
    if (r) delete [] r;
    
    //
    // Check if there is a mail into Outbox folder.
    // In this function, if the folder is outbox, the mail has to be in
    //
    if (folder == 'O')
        MailSyncSource::setIsMailInOutbox(TRUE);


    if (msgprops[ePR_LAST_MODIFICATION_TIME].ulPropTag == PR_LAST_MODIFICATION_TIME)
        ft = msgprops[ePR_LAST_MODIFICATION_TIME].Value.ft;  //OK   
    
    FileTime2UTCTimeFormatter(ft, time);
    wchar_t tmp[64];    
    SYSTEMTIME local;
    FileTimeToSystemTime(&ft, &local);

    wsprintf(tmp, TEXT("%i-%i-%iT%i-%i-%i"), local.wYear, local.wMonth, local.wDay, local.wHour, local.wMinute, local.wSecond);
    LOG.debug(tmp);
    mcd->getEmailData()->setModified(time);    
    
    if (folder == 'O') {
        SYSTEMTIME systemTime;
        GetSystemTime(&systemTime);
    
        b.setYear(systemTime.wYear);    
        b.setMonth(systemTime.wMonth);
        b.setWeekday(systemTime.wDayOfWeek);
        b.setDay(systemTime.wDay);
        b.setHour(systemTime.wHour);
        b.setMin(systemTime.wMinute);
        b.setSec(systemTime.wSecond);
    
        mcd->getEmailData()->getEmailItem().setDate(b);    
    }
    // flag

    mcd->getEmailData()->setRead(false);
    mcd->getEmailData()->setForwarded(false);
    mcd->getEmailData()->setReplied(false);
    mcd->getEmailData()->setReceived(false);
    mcd->getEmailData()->setCreated(false);
    mcd->getEmailData()->setDeleted(false);
    mcd->getEmailData()->setFlagged(false);
   
    if (msgprops[ePR_MESSAGE_FLAGS].ulPropTag == PR_MESSAGE_FLAGS) {
        if (msgprops[ePR_MESSAGE_FLAGS].Value.ul & MSGFLAG_READ) {
            read = TRUE;
        }
        else {
            read = FALSE;
        }
    }
    
    mcd->getEmailData()->setRead(read);    

    //
    // TBD: to be developed
    //

    //if (isUpdate)
    //    goto FuncExit;

    if (msgprops[ePR_SUBJECT].ulPropTag == PR_SUBJECT)
        mcd->getEmailData()->getEmailItem().setSubject(msgprops[ePR_SUBJECT].Value.lpszW);        
    
    if (msgprops[ePR_SENDER_EMAIL_ADDRESS].ulPropTag == PR_SENDER_EMAIL_ADDRESS)
        mcd->getEmailData()->getEmailItem().setFrom(msgprops[ePR_SENDER_EMAIL_ADDRESS].Value.lpszW);               
            
    hr = GetRecipients(pmsg, &to, &cc, &bcc);
    EXIT_ON_FAILED (hr);

    mcd->getEmailData()->getEmailItem().setTo (to);
    mcd->getEmailData()->getEmailItem().setCc (cc);
    mcd->getEmailData()->getEmailItem().setBcc(bcc);
    
    hr = GetBody(pmsg, &body);
    EXIT_ON_FAILED (hr);

    bodyPart = BodyPart();
    bodyPart.setContent(body);
    mcd->getEmailData()->getEmailItem().setBody(bodyPart);    
        
    if (msgprops[ePR_HASATTACH].ulPropTag == PR_HASATTACH) {                
            attachCount = msgprops[ePR_HASATTACH].Value.ul;        
            GetAttachments(pmsg, mcd);
    }
        

    //
    // Property X-Funambol-isDraft
    /*
    if (msgprops[ePR_MESSAGE_FLAGS].ulPropTag == PR_MESSAGE_FLAGS) {
        if (msgprops[ePR_MESSAGE_FLAGS].Value.ul & MSGFLAG_FROMME) {
            isDraft = TRUE;
        }
        else {
            isDraft = FALSE;
        }
    }
    */
    // mcd->getEmailData()->setXFunambolIsDeaft(isDraft);

    /*
    wchar_t* disp = msgprops[ePR_SENDER_NAME].Value.lpszW; //OK
    long msgSize = msgprops[ePR_MESSAGE_SIZE].Value.ul; //OK  
    wchar_t* prefix = msgprops[ePR_SUBJECT_PREFIX].Value.lpszW; //no
   
    */
FuncExit:

    return mcd;
}

IMAPISession* MailClient2Server::logOn() {
    
    IMAPISession *    pSession    =  NULL;
    HRESULT hr;
    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    
    if (hr == S_OK) 
        return pSession;
    else
        return NULL;

}

void MailClient2Server::logOff(IMAPISession* pSession) {
    if (pSession)
        RELEASE_OBJ (pSession);   
}

MailClientData* MailClient2Server::getFirstItem() {       
    allTot = 0;
    mailAll = -2;
    elementIndex = -2;

    for (int i = 0; i < folderNumber; i++) {
        allTot += entryListArray[i].cValues;
    }
    if (allTot == 0)
        return NULL;
    
    if (allTot > 0) {
        mailAll++;       // from -2 -> -1  
        elementIndex++;  // from -2 -> -1
    }
    arrayIndex = 0;
    
    return getNextItem();    

}

MailClientData* MailClient2Server::getNextItem() {
    
    IMAPISession *    pSession    = NULL;
    HRESULT           hr          = NULL;
    IMessage*         pmsg        = NULL;    
    ULONG             returned    = 0;
    int               tot         = 0;

    // if (mailClientData) {
    //    delete mailClientData; mailClientData = NULL;
    // }
    if (allTot == 0)
        return NULL;

    MailClientData* mailClientData = NULL;
    
    mailAll++;
    elementIndex++;
    if (arrayIndex == folderNumber) 
        return NULL;
      // if the current folder has no updated items, increments arrayIndex immediately

    if (entryListArray[arrayIndex].cValues == 0) {
        for (int k = 0; (k < folderNumber && arrayIndex < folderNumber); k++) {
            if (entryListArray[arrayIndex].cValues == 0) 
                arrayIndex++;
             else
                break;        
        }
    }
    pSession = logOn();
    
    if (pSession == NULL)
        goto FuncExit;
   
    if (mailAll < allTot) {                

        hr = pSession->OpenEntry(entryListArray[arrayIndex].lpbin[elementIndex].cb, 
            (ENTRYID*)entryListArray[arrayIndex].lpbin[elementIndex].lpb, 
            NULL, 
            MAPI_BEST_ACCESS, 
            &returned, 
            (IUnknown **) &pmsg);
        EXIT_ON_FAILED (hr);        
        ASSERT (pmsg);   
                
        mailClientData = completeObject(pmsg, folderToSync[arrayIndex]);       

        if ((unsigned long)elementIndex == entryListArray[arrayIndex].cValues -1) {
            arrayIndex++;
            elementIndex = -1;
        }
    
    }
  
    logOff(pSession);

FuncExit:
    return mailClientData;
    /*
    if (mailClientData)
        return (MailMessage*)mailMessage->clone();    
    else
        return NULL;
    */
}

MailClientData* MailClient2Server::getFirstNewItem() {       
    
    allTot = 0;
    mailNew = -2;       // from -2 -> -1  
    elementIndex = -2;  // from -2 -> -1

    for (int i = 0; i < folderNumber; i++) {
        allTot += newMessagesArray[i].cValues;
    }
    if (allTot == 0)
        return NULL;
    
    if (allTot > 0) {
        mailNew++;       // from -2 -> -1  
        elementIndex++;  // from -2 -> -1
    }
    arrayIndex = 0;
       
    return getNextNewItem();             
      
}

MailClientData* MailClient2Server::getNextNewItem() {       
    
    IMAPISession *    pSession    = NULL;
    HRESULT           hr          = NULL;
    IMessage*         pmsg        = NULL;    
    ULONG             returned    = 0;
    MailClientData*   mailClientData = NULL;
    //if (mailMessage) {
    //    delete mailMessage; mailMessage = NULL;
    // }

    if (allTot == 0)
        return NULL;

    mailNew++;
    elementIndex++;
    
     if (arrayIndex == folderNumber) 
        return NULL;
     // if the current folder has no updated items, increments arrayIndex immediately
    if (newMessagesArray[arrayIndex].cValues == 0) {
        for (int k = 0; (k < folderNumber && arrayIndex < folderNumber); k++) {
            if (newMessagesArray[arrayIndex].cValues == 0)
                arrayIndex++;
            else
                break;
        }
    }

    pSession = logOn();
    
    if (pSession == NULL)
        goto FuncExit;
   
    if (mailNew < allTot) {                

        hr = pSession->OpenEntry(newMessagesArray[arrayIndex].lpbin[elementIndex].cb, 
            (ENTRYID*)newMessagesArray[arrayIndex].lpbin[elementIndex].lpb, 
            NULL, 
            MAPI_BEST_ACCESS, 
            &returned, 
            (IUnknown **) &pmsg);
        EXIT_ON_FAILED (hr);        
        ASSERT (pmsg);   
                
        mailClientData = completeObject(pmsg, folderToSync[arrayIndex]);       

        if ((unsigned long)elementIndex == newMessagesArray[arrayIndex].cValues -1) {
            arrayIndex++;
            elementIndex = -1;
        }    
    }
   
    logOff(pSession);

FuncExit:
    return mailClientData;
    /*
    if (mailMessage)
        return (MailMessage*)mailMessage->clone();    
    else
        return NULL;             
    */  
}


MailClientData* MailClient2Server::getFirstUpdatedItem() {       
    
    allTot = 0;
    mailUpdated = -2;       // from -2 -> -1  
    elementIndex = -2;  // from -2 -> -1

    for (int i = 0; i < folderNumber; i++) {
        allTot += updatedMessagesArray[i].cValues;
    }
    if (allTot == 0)
        return NULL;
    
    if (allTot > 0) {
        mailUpdated++;       // from -2 -> -1  
        elementIndex++;  // from -2 -> -1
    }
    arrayIndex = 0;
    
    return getNextUpdatedItem();             
      
}

MailClientData* MailClient2Server::getNextUpdatedItem() {       
    
    IMAPISession *    pSession    = NULL;
    HRESULT           hr          = NULL;
    IMessage*         pmsg        = NULL;    
    ULONG             returned    = 0;
    MailClientData*   mailClientData = NULL;

    // if (mailMessage) {
    //    delete mailMessage; mailMessage = NULL;
    //}

    if (allTot == 0)
        return NULL;
    
    mailUpdated++;
    elementIndex++;
    
     if (arrayIndex == folderNumber) 
        return NULL;
    // if the current folder has no updated items, increments arrayIndex immediately
    if (updatedMessagesArray[arrayIndex].cValues == 0) {
        for (int k = 0; (k < folderNumber && arrayIndex < folderNumber); k++) {
            if (updatedMessagesArray[arrayIndex].cValues == 0)
                arrayIndex++;
            else
                break;
        }
    }
    pSession = logOn();
    
    if (pSession == NULL)
        goto FuncExit;
   
    if (mailUpdated < allTot) {                

        hr = pSession->OpenEntry(updatedMessagesArray[arrayIndex].lpbin[elementIndex].cb, 
            (ENTRYID*)updatedMessagesArray[arrayIndex].lpbin[elementIndex].lpb, 
            NULL, 
            MAPI_BEST_ACCESS, 
            &returned, 
            (IUnknown **) &pmsg);
        EXIT_ON_FAILED (hr);        
        ASSERT (pmsg);   

        //
        // The updated parameter
        //
        mailClientData = completeObject(pmsg, folderToSync[arrayIndex], TRUE);       

        if ((unsigned long)elementIndex == updatedMessagesArray[arrayIndex].cValues -1) {
            arrayIndex++;
            elementIndex = -1;
        }    
    }
   
    logOff(pSession);

FuncExit:
    return mailClientData;
    /*
    if (mailMessage)
        return (MailMessage*)mailMessage->clone();    
    else
        return NULL;         
    */  
}

MailClientData* MailClient2Server::getFirstDeletedItem() {       
    
    allTot = 0;
    mailDeleted  = -2;  // from -2 -> -1  
    elementIndex = -2;  // from -2 -> -1

    for (int i = 0; i < folderNumber; i++) {
        allTot += deletedMessagesArray[i]->size();
    }
    if (allTot == 0)
        return NULL;
    
    if (allTot > 0) {
        mailDeleted++;       // from -2 -> -1  
        elementIndex++;  // from -2 -> -1
    }
    arrayIndex = 0;
    /*    
    if (deletedMessages->size() == 0)
        return NULL;
    
     if (deletedMessages->size() > 0)
        mailDeleted++;
    */
     return getNextDeletedItem();             
      
}

MailClientData* MailClient2Server::getNextDeletedItem() {       
    
    // if (mailMessage) {
    //    delete mailMessage; mailMessage = NULL;
    // }

    if (allTot == 0)
        return NULL;

    MailClientData*   mailClientData = NULL;
    wchar_t* r = NULL;
    mailDeleted++;        
    elementIndex++;
    
    // if arrayIndex is the same as folder number we are at the end of the array
    if (arrayIndex == folderNumber) 
        return NULL;

    // if the current folder has no updated items, increments arrayIndex immediately    
    if (deletedMessagesArray[arrayIndex]->size() == 0) {
        for (int k = 0; (k < folderNumber && arrayIndex < folderNumber); k++) {
            if (deletedMessagesArray[arrayIndex]->size() == 0)
                arrayIndex++;               
            else
                break;
        }
    }

    if (mailDeleted < allTot) {                
        
        MailItem* mailItem = (MailItem*)deletedMessagesArray[arrayIndex]->get(mailDeleted);        
                
        if (mailItem) {
            mailClientData = new MailClientData();
            
            //mailMessage = new MailMessage();
            mailClientData->getEmailData()->getEmailItem().setEntryID(mailItem->getId()); 
            r = c2wc(folderToSync[arrayIndex]);
            mailClientData->setParent(r);
            if (r) {
                delete [] r;
                r = NULL;
            }
        }

        if (elementIndex == deletedMessagesArray[arrayIndex]->size() -1) {
            arrayIndex++;
            elementIndex = -1;
        }    
    }
    return mailClientData;
    /*
    if (mailMessage)
        return (MailMessage*)mailMessage->clone();    
    else
        return NULL;
    */
}

BOOL openTmpFile(wchar_t* filename, FILE** f) {    
	
    BOOL ret = TRUE;
    *f = _wfopen(filename, TEXT("w+b"));	

    if(f == NULL) 
        ret = FALSE;
    
    return ret;
}

void closeTmpFile(FILE* f) {
    
    if (f) {
        fflush(f);
	    fclose(f);
    }
}

int writeToTmpFile(FILE* f, char *ptr, int numBytes) {
        	    
   int numWritten = fwrite(ptr, sizeof(char), numBytes, f);                        
   return numWritten; 

}

HRESULT MailClient2Server::GetAttachments(IMessage* pmsg, MailClientData* mcd) {

    LPMAPITABLE pAttachTable = NULL;
	// for each row in the attach table
	LPSRowSet pRows = NULL;
	int currentRow = 0;	    
    char szBuf[2048];
	ULONG ulNumChars;
	
	HRESULT hr = pmsg->GetAttachmentTable(0, &pAttachTable);
	EXIT_ON_FAILED(hr);

    // iterate all the attachment
	while(TRUE) {

		hr = pAttachTable->QueryRows(1, 0, &pRows);
		if (FAILED(hr) || pRows->cRows != 1) 
            break;
		currentRow++;

		// only one row is returned anyway
		int nProps = pRows->aRow[0].cValues;  
        long num = -1;
        long size = -1;
        wchar_t* filename = NULL, *filenameTmp = NULL;
        long msgstatus = -1;
        IAttach* attach;

        for (int i=0; i<nProps; i++) {
            ULONG ulPT = pRows->aRow[0].lpProps[i].ulPropTag;
            // loop through all the properties			

			// TCHAR* szPropName = ReverseLookup(ulPT);
			switch (ulPT) {
			    case PR_ATTACH_NUM:
				    num = pRows->aRow[0].lpProps[i].Value.ul;
				    break;
			    case PR_ATTACH_SIZE:
				    size = pRows->aRow[0].lpProps[i].Value.ul;
				    break;
			    case PR_ATTACH_FILENAME:
				    filenameTmp = pRows->aRow[0].lpProps[i].Value.lpszW;
                    if (filenameTmp)
                        filename = stringdup(filenameTmp);
				    break;
			    case PR_MSG_STATUS:
				    msgstatus = pRows->aRow[0].lpProps[i].Value.ul;
				    break;
			    case 0x30000003:
				    // pRows->aRow[0].lpProps[ii].Value.ul: sequence number
				    break;
			    default:
				    break;
			}	// switch
		
        }        
        FreeProws(pRows);
        
        pmsg->OpenAttach(num, NULL, 0, &attach);
        // read IStream->Read way to write into an array of char.
        IStream* pStream;
        hr = attach->OpenProperty(PR_ATTACH_DATA_BIN, 0, NULL, ATTACH_BY_VALUE, (LPUNKNOWN *)&pStream);
        EXIT_ON_FAILED(hr);
        
        /*
        LARGE_INTEGER lStart; 
        ULARGE_INTEGER lEnd; 

        lStart.QuadPart = 0;         
        pStream->Seek(lStart, STREAM_SEEK_END, &lEnd); // the end 
        pStream->Seek(lStart, STREAM_SEEK_SET, NULL); // the start        
        */
        wchar_t tmpPath[64];
        wchar_t tmpFileName[MAX_PATH];  // System constants for the path
 
        GetTempPath(64, tmpPath);
        GetTempFileName(tmpPath, TEXT("fun"), 0, tmpFileName);

        // wchar_t* tmpFileName = mkTempFileName(filename);
        FILE* f = NULL;
        f = _wfopen(tmpFileName, TEXT("w+b"));	
        
        if (f == NULL)
            goto FuncExit;
    
        int m = 0;
	    do {
		    pStream->Read(szBuf, 2048, &ulNumChars);
            writeToTmpFile(f, szBuf, ulNumChars);
            m = m + ulNumChars;
            
	    } while (ulNumChars >= 2048);
        
        closeTmpFile(f);
  
	    pStream->Release();
        
        BodyPart bodyPart;
        bodyPart.setFilename(filename);
        bodyPart.setContent(tmpFileName);
        bodyPart.setEncoding(TEXT("base64")); // currently the ppc default
        
        if (wcsstr(filename, TEXT(".wav"))) {
            bodyPart.setMimeType(TEXT("audio/x-wav"));
        } else if (wcsstr(filename, TEXT(".bmp"))) {
            bodyPart.setMimeType(TEXT("image/x-ms-bmp"));
        } else if (wcsstr(filename, TEXT(".gif"))) {
            bodyPart.setMimeType(TEXT("image/gif"));
        } else if (wcsstr(filename, TEXT(".jpg"))) {
            bodyPart.setMimeType(TEXT("image/jpg"));
        } else if (wcsstr(filename, TEXT(".psw"))) {
            bodyPart.setMimeType(TEXT("application/msword"));
        } else if (wcsstr(filename, TEXT(".htm"))) {
            bodyPart.setMimeType(TEXT("text/html"));
        }		

        mcd->getEmailData()->getEmailItem().addAttachment(bodyPart);

        if (filename)
            delete [] filename;
	    	                
    }
FuncExit:
	// Free each element in the pRows buffer
	FreeProws(pRows);
	RELEASE_OBJ(pAttachTable);
	return hr;

}


HRESULT MailClient2Server::GetBody(IMessage* pmsg, wchar_t** body) {

	IStream* pStream;
	char szBuf[255];
	ULONG ulNumChars;
	ULONG ulPos = 0;
	HRESULT	hr;

	// temporary storage for body
	TCHAR szTempBody[4096];
	ZeroMemory(szTempBody, 4096*sizeof(TCHAR));
	
	ULONG nAvail = 4095;		
	int nTempLen = 0;

	// Open the Stream to the body
	hr = pmsg->OpenProperty(PR_BODY, NULL, STGM_READ, NULL,(LPUNKNOWN *) &pStream);
	EXIT_ON_FAILED(hr);

	do {
		pStream->Read(szBuf, 255, &ulNumChars);

		// works even if ulNumChars<=0
		for (ULONG ii=0; ii<ulNumChars; ii++) {
			if (ulPos>nAvail) {		// we should not read too much!
				goto DoneReading;
			}
			if (szBuf[ii]!=0) {
				// only store the non-zero characters (because there are stored as Unicode!!)
				szTempBody[ulPos++] = (TCHAR)szBuf[ii];
			}
		}
	} while (ulNumChars >= 255);

DoneReading:
	// DO NOT handle body >4096
	pStream->Release();
	szTempBody[ulPos] = L'\0';

	// copy back to CommandBody, if any
	nTempLen = wcslen(szTempBody);
	if (nTempLen>0) {
        *body = new wchar_t[wcslen(szTempBody) + 1];
        wcscpy(*body, szTempBody);
		// swprintf(szCmdBody+wcslen(szCmdBody), TEXT("MSG {%d} %s"), nTempLen, szTempBody);
	}
	return S_OK;

FuncExit:
	// swprintf(szCmdBody+wcslen(szCmdBody), TEXT("MSG {%d} %s"), 1, TEXT(" "));
	return -1;
}

void BuildAddress(IN LPTSTR pszDisplayName, IN LPTSTR pszEmailAddress, OUT LPTSTR pszAddress)
{
	if (pszDisplayName==NULL && pszEmailAddress!=NULL) {
		wcscpy(pszAddress, pszEmailAddress);
	} else if (pszDisplayName!=NULL && pszEmailAddress==NULL) {
		swprintf(pszAddress, TEXT("\"%s\""), pszDisplayName);
	} else if (pszDisplayName!=NULL && pszEmailAddress!=NULL) {
		swprintf(pszAddress, TEXT("\"%s\" <%s>"), pszDisplayName, pszEmailAddress);
	} else {
		wcscpy(pszAddress, TEXT("UNKNOWN"));
	}
}

// concatenate the recipients part
HRESULT MailClient2Server::GetRecipients(LPMESSAGE pMessage, wchar_t** to, wchar_t** cc, wchar_t** bcc) {
	// for the recipient table
	LPMAPITABLE pRecipientTable = NULL;
	// for each row in the recipient table
	LPSRowSet pRows = NULL;
	int currentRow  = 0;
	int nTempLen    = 0;

	// temporary storage
	TCHAR szTempTO[256];
	TCHAR szTempCC[256];
	TCHAR szTempBCC[256];
	ZeroMemory(szTempTO,  256*sizeof(TCHAR));
	ZeroMemory(szTempCC,  256*sizeof(TCHAR));
	ZeroMemory(szTempBCC, 256*sizeof(TCHAR));

	// pRecipientTable->SetColumn does not work. Returning result is sth. like "NOT IMPLEMENTED".
	// loop through all rows without limiting the columns
	HRESULT hr = pMessage->GetRecipientTable(0, &pRecipientTable);
	EXIT_ON_FAILED(hr);

	// iterate all the recipients
	while(TRUE) {

		hr = pRecipientTable->QueryRows(1, 0, &pRows);
		if (FAILED(hr) || pRows->cRows != 1) break;
		currentRow++;

		// only one row is returned anyway
		int nProps = pRows->aRow[0].cValues;

		TCHAR *szReceipName, *szReceipEmail;
		ULONG ulReceipType;
		szReceipEmail = NULL;
		szReceipName = NULL;
		ulReceipType = 0;

		// loop through all the properties
		for (int ii=0; ii<nProps; ii++) {
			ULONG ulPT = pRows->aRow[0].lpProps[ii].ulPropTag;

			// TCHAR* szPropName = ReverseLookup(ulPT);
			switch (ulPT) {
			case PR_EMAIL_ADDRESS:
				szReceipEmail = pRows->aRow[0].lpProps[ii].Value.LPSZ;
				break;
			case PR_RECIPIENT_TYPE:
				ulReceipType = pRows->aRow[0].lpProps[ii].Value.ul;
				break;
			case PR_DISPLAY_NAME:
				szReceipName = pRows->aRow[0].lpProps[ii].Value.LPSZ;
				break;
			case PR_ADDRTYPE:
				// pRows->aRow[0].lpProps[ii].Value.LPSZ. Always "SMTP"
				break;
			case 0x30000003:
				// pRows->aRow[0].lpProps[ii].Value.ul: sequence number
				break;
			default:
				break;
			}	// switch
		}

		// now let us build the String
		TCHAR szAddr[256];
		BuildAddress(szReceipName, szReceipEmail, szAddr);
		if (ulReceipType==MAPI_TO) {
			wcscat(szTempTO, szAddr);
			wcscat(szTempTO, TEXT(","));
		} else if (ulReceipType==MAPI_CC) {
			wcscat(szTempCC, szAddr);
			wcscat(szTempCC, TEXT(","));
		} else if (ulReceipType==MAPI_BCC) {
			wcscat(szTempBCC, szAddr);
			wcscat(szTempBCC, TEXT(","));
		}

		// Free each element in the pRows buffer
		FreeProws(pRows);
	}	// end of the loop through all recipients

	// remove the trailing ",", if any
	{
		int nLen = wcslen(szTempTO);
		if (nLen>0 && szTempTO[nLen-1]==L',') szTempTO[nLen-1] = L'\0';
		nLen = wcslen(szTempCC);
		if (nLen>0 && szTempCC[nLen-1]==L',') szTempCC[nLen-1] = L'\0';
		nLen = wcslen(szTempBCC);
		if (nLen>0 && szTempBCC[nLen-1]==L',') szTempBCC[nLen-1] = L'\0';
	}

	// copy back
	nTempLen = wcslen(szTempTO);
	if (nTempLen>0) {
        *to = new wchar_t[wcslen(szTempTO) + 1];
        wcscpy(*to, szTempTO);
		//swprintf(szCmdBody+wcslen(szCmdBody), TEXT("TO {%d} %s "), nTempLen, szTempTO);
	}
	nTempLen = wcslen(szTempCC);
	if (nTempLen>0) {
        *cc = new wchar_t[wcslen(szTempCC) + 1];
        wcscpy(*cc, szTempCC);
		//swprintf(szCmdBody+wcslen(szCmdBody), TEXT("CC {%d} %s "), nTempLen, szTempCC);
	}
	nTempLen = wcslen(szTempBCC);
	if (nTempLen>0) {
        *bcc = new wchar_t[wcslen(szTempBCC) + 1];
        wcscpy(*bcc, szTempBCC);
		// swprintf(szCmdBody+wcslen(szCmdBody), TEXT("BCC {%d} %s "), nTempLen, szTempBCC);
	}

FuncExit:
	// Free each element in the pRows buffer
	FreeProws(pRows);
	RELEASE_OBJ(pRecipientTable);
	return hr;
}

HRESULT MailClient2Server::GetAllMessages(ENTRYLIST& entryList, LPMAPIFOLDER m_pInBoxFolder, 
                                          wchar_t* path, wchar_t folderId, ArrayList* extra) {
	HRESULT		hr;
	UINT		nEntries = 0;
	SBinary		sbEntry;
	LPMAPITABLE pContentsTable = NULL;
	LPSRowSet	pRows = NULL;    
	
    //
    // This is only to populate the extra ArrayList with all needed extra info.
    // The mailInfo object doesn't need. It is immediately deleted
    //
    if (extra) {
        MailInfo*  mailInfo = readPreviousMail(path, folderId, extra);
        if (mailInfo) {
            delete mailInfo; mailInfo = NULL;
        }
    }
    // We define a SPropTagArray array here using the SizedSPropTagArray Macro
	// This enum will allows us to access portions of the array by a name instead of a number.
	// If more tags are added to the array, appropriate constants need to be added to the enum.	

	hr = m_pInBoxFolder->GetContentsTable( 0, &pContentsTable);
	EXIT_ON_FAILED(hr);

	hr = pContentsTable->SetColumns((LPSPropTagArray)&Columns, 0);
	EXIT_ON_FAILED(hr);

	while(!FAILED(hr = pContentsTable->QueryRows(1, 0, &pRows)))
	{
		if(pRows->cRows == 0)
			break;
               			
		sbEntry = pRows->aRow[0].lpProps[eePR_ENTRYID].Value.bin;               
        
		if(entryList.cValues)
		{
			PVOID	pOldBuffer = entryList.lpbin;
			MAPIAllocateBuffer(sizeof(SBinary) * (entryList.cValues + 1), (LPVOID*)&(entryList.lpbin));
			memmove((LPVOID*)entryList.lpbin, pOldBuffer, sizeof(SBinary) * entryList.cValues);
			MAPIFreeBuffer(pOldBuffer);
		}
        else {
			MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(entryList.lpbin));
        }

		entryList.lpbin[entryList.cValues].cb = sbEntry.cb;
		MAPIAllocateBuffer(sbEntry.cb, (LPVOID*)&(entryList.lpbin[entryList.cValues].lpb) );
		memmove(entryList.lpbin[entryList.cValues].lpb, sbEntry.lpb, sbEntry.cb);
		entryList.cValues++;            

		// Free each element in the pRows buffer
		FreeProws(pRows);

		hr = S_OK;
	}
    
FuncExit:
	if(pRows)
	{
		// Free each element in the pRows buffer
		FreeProws(pRows);
	}
	RELEASE_OBJ(pContentsTable);
	return hr;
}


HRESULT MailClient2Server::GetModificatedMessages(ENTRYLIST& newMessages, ENTRYLIST& updatedMessages, 
                                                  ArrayList** deletedMessages, LPMAPIFOLDER m_pInBoxFolder, 
                                                  wchar_t* path, wchar_t folderId, OUT ArrayList* extra) {
	HRESULT		hr;
	UINT		nEntries = 0;
	SBinary		sbEntry;
	LPMAPITABLE pContentsTable = NULL;
	LPSRowSet	pRows = NULL;    
    DATE        lastSync = 0;
    int         numMailItem = 0;
    int         i = 0;
    int         action = 0;
    BOOL        found = FALSE;

    BOOL        flagRead = FALSE;
    FILETIME    ft;
    SYSTEMTIME  st;

    DATE        lastSyncMailItem = 0;
    BOOL        flagReadMailItem = FALSE;
    wchar_t*    idMailItem       = NULL;

    MailInfo*  mailInfo = readPreviousMail(path, folderId, extra);
    MailItem*  mailItem = NULL;
    
    if (mailInfo == NULL)
        goto FuncExit;
    
    lastSync    = mailInfo->getLast();
    numMailItem = mailInfo->getNum();

   
	// We define a SPropTagArray array here using the SizedSPropTagArray Macro
	// This enum will allows us to access portions of the array by a name instead of a number.
	// If more tags are added to the array, appropriate constants need to be added to the enum.	

	hr = m_pInBoxFolder->GetContentsTable( 0, &pContentsTable);
	EXIT_ON_FAILED(hr);

	hr = pContentsTable->SetColumns((LPSPropTagArray)&Columns, 0);
	EXIT_ON_FAILED(hr);

	while(!FAILED(hr = pContentsTable->QueryRows(1, 0, &pRows)))
	{   
        found = FALSE;
        
		if(pRows->cRows == 0)
			break;                
        
        // get the property of the mail in the folder
		sbEntry = pRows->aRow[0].lpProps[eePR_ENTRYID].Value.bin;     
        ft = pRows->aRow[0].lpProps[eePR_LAST_MODIFICATION_TIME].Value.ft;
        FileTimeToSystemTime(&ft, &st);
        SystemTimeToVariantTime(&st, &lastSyncMailItem);   

        if (pRows->aRow[0].lpProps[eePR_MESSAGE_FLAGS].Value.ul & MSGFLAG_READ)
            flagRead = TRUE;
        else
            flagRead = FALSE;
        
        wchar_t* id = convertBinaryToWChar(sbEntry, folderId);
        
        for (i = 0; i < mailInfo->getMailItems()->size(); i++) {
            
            mailItem = (MailItem*)(mailInfo->getMailItems())->get(i);;
            
            idMailItem = mailItem->getId();
            if (wcscmp(id, idMailItem) != 0)
                continue;
            
            // if no continue then it goes on and found the item
            found = TRUE;
            flagReadMailItem = mailItem->getRead();            
            
            if (flagReadMailItem != flagRead) 
                LOG.debug(TEXT("flag read different"));
            else
                LOG.debug(TEXT("flag read equal"));

            if (lastSyncMailItem > lastSync) {
                LOG.debug(TEXT("flag lastSyncMailItem > "));
                wchar_t r[256];
                wsprintf(r, TEXT("lastSyncMailItem %lf"), lastSyncMailItem);
                LOG.debug(r);
                wsprintf(r, TEXT("lastSync %lf"), lastSync);
                LOG.debug(r);
            } else {
                 LOG.debug(TEXT("flag lastSyncMailItem < "));
                wchar_t r[256];
                wsprintf(r, TEXT("lastSyncMailItem %lf"), lastSyncMailItem);
                LOG.debug(r);
                wsprintf(r, TEXT("lastSync %lf"), lastSync);
                LOG.debug(r);
            }

            if (flagReadMailItem != flagRead ||
                lastSyncMailItem > lastSync) {                                
                action = UPDATED_MAIL;
            } else {
                action = NO_ACTION_MAIL;
            }
            mailInfo->getMailItems()->removeElementAt(i);
            break;
            
        }        
        
        if (found == FALSE)
            action = NEW_MAIL;
        
        switch (action) {
            
            case NEW_MAIL: {
                
                if(newMessages.cValues)
		        {
			        PVOID	pOldBuffer = newMessages.lpbin;
			        MAPIAllocateBuffer(sizeof(SBinary) * (newMessages.cValues + 1), (LPVOID*)&(newMessages.lpbin));
			        memmove((LPVOID*)newMessages.lpbin, pOldBuffer, sizeof(SBinary) * newMessages.cValues);
			        MAPIFreeBuffer(pOldBuffer);
		        }
                else {
			        MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(newMessages.lpbin));
                }	            
		        newMessages.lpbin[newMessages.cValues].cb = sbEntry.cb;
		        MAPIAllocateBuffer(sbEntry.cb, (LPVOID*)&(newMessages.lpbin[newMessages.cValues].lpb) );
		        memmove(newMessages.lpbin[newMessages.cValues].lpb, sbEntry.lpb, sbEntry.cb);
		        newMessages.cValues++;            

		        // Free each element in the pRows buffer
		        FreeProws(pRows);		        

            }; break;

            case UPDATED_MAIL: {
                
                if(updatedMessages.cValues)
		        {
			        PVOID	pOldBuffer = updatedMessages.lpbin;
			        MAPIAllocateBuffer(sizeof(SBinary) * (updatedMessages.cValues + 1), (LPVOID*)&(updatedMessages.lpbin));
			        memmove((LPVOID*)updatedMessages.lpbin, pOldBuffer, sizeof(SBinary) * updatedMessages.cValues);
			        MAPIFreeBuffer(pOldBuffer);
		        }
                else {
			        MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(updatedMessages.lpbin));
                }

		        updatedMessages.lpbin[updatedMessages.cValues].cb = sbEntry.cb;
		        MAPIAllocateBuffer(sbEntry.cb, (LPVOID*)&(updatedMessages.lpbin[updatedMessages.cValues].lpb) );
		        memmove(updatedMessages.lpbin[updatedMessages.cValues].lpb, sbEntry.lpb, sbEntry.cb);
		        updatedMessages.cValues++;            

		        // Free each element in the pRows buffer
		        FreeProws(pRows);		 
            }; break;

            case NO_ACTION_MAIL: 
                // No action to do
                break;
        }
        
	}
    if (mailInfo->getMailItems()->size() > 0)
        *deletedMessages = mailInfo->getMailItems()->clone();

FuncExit:
	if(pRows)
	{
		// Free each element in the pRows buffer
		FreeProws(pRows);
	}
    if (mailInfo) {
        delete mailInfo; mailInfo = NULL;
    }

	RELEASE_OBJ(pContentsTable);
	return hr;
}

void MailClient2Server::setFolderToSync(wchar_t* t){
    safeDelete(&folderToSync);
    folderToSync = stringdup(t);
}

wchar_t* MailClient2Server::getFolderToSync() {
    return folderToSync;
}

ArrayList* MailClient2Server::setSync (int mode, wchar_t* path) {
    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    IMAPITable*       pTable      =  NULL;
    SRowSet*          psrs        =  NULL;
    IMsgStore*        pStore      =  NULL;
    int               i           =  0;
    ULONG*   rgTags;
    folderNumber = 0;
    ArrayList* extra = new ArrayList();

    if (folderToSync == NULL) {
        rgTags = new ULONG[2];
        rgTags[0] = 1; 
        rgTags[1] = PR_CE_IPM_INBOX_ENTRYID; 
        folderNumber = 1;
    }
    else {
        folderNumber = wcslen(folderToSync);
        rgTags = new ULONG[folderNumber + 1];         
        rgTags[0] = folderNumber;
        for (int i = 1; i <= folderNumber; i++) 
            rgTags[i] = getFolderId(folderToSync[i-1]);    // PR_CE_IPM_INBOX_ENTRYID; 
                                                      // PR_CE_IPM_DRAFTS_ENTRYID;        
    } 
    
    ULONG             rgMsgTags[] =  { 1, PR_ENTRYID };
    ULONG             cValues     =  0;
    ULONG             returned    =  0;
    
    SPropValue*       rgprops     =  NULL;
    SPropValue*       rgpropsMsg  =  NULL;
    LPMAPIFOLDER      pfldrInbox  =  NULL;

    IMessage*         pmsg        =  NULL;
    ULONG             cmsgValue   =  0;
    SPropValue*       msgprops    =  NULL;
        

    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    EXIT_ON_FAILED (hr);

    // Get the message stores table
    hr = pSession->GetMsgStoresTable (MAPI_UNICODE, &pTable);
    EXIT_ON_FAILED (hr);
   
    hr = pTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);
  
    while (1){
    // Get a row
    hr = pTable->QueryRows (1, 0, &psrs);
    EXIT_ON_FAILED (hr);
    
    // Did we hit the end of the table?
    if (psrs->cRows != 1) {
      break;
    }
    //
    // the display name is the name of the account.
    //
    wchar_t* displayName = psrs->aRow[0].lpProps[eePR_DISPLAY_NAME].Value.lpszW;
    
    //
    // The name of the MessageStore that we use for the own Transport
    //
    if (wcscmp(displayName, ACCOUNT_NAME) == 0) {

        // Make sure we got the props we need
        if ((psrs->aRow[0].cValues < 1) || (psrs->aRow[0].lpProps[0].ulPropTag != PR_ENTRYID))
        {
          MessageBox (NULL, L"Store missing PR_ENTRYID!", L"Doh", MB_OK);
          hr = E_FAIL;
          break;
        }

        // Open this message store
        hr = pSession->OpenMsgStore (NULL, 
                       psrs->aRow[0].lpProps[0].Value.bin.cb, 
                      (ENTRYID *) psrs->aRow[0].lpProps[0].Value. 
                       bin.lpb, NULL, 0, &pStore);
        EXIT_ON_FAILED (hr);
  
        // Now get the Inbox folder.
        hr = pStore->GetProps ((SPropTagArray *) rgTags, MAPI_UNICODE, &cValues, &rgprops);
        EXIT_ON_FAILED (hr);
  
        ASSERT (rgprops);
        
        entryListArray       = new ENTRYLIST[folderNumber];
        newMessagesArray     = new ENTRYLIST[folderNumber];
        updatedMessagesArray = new ENTRYLIST[folderNumber];
        deletedMessagesArray = new ArrayList*[folderNumber];
        
        //
        // watch at the folder that we want to sync
        //
        for (i = 0; i < folderNumber; i++) {
           
            // open the message store
            hr = pStore->OpenEntry (rgprops[i].Value.bin.cb, 
                        (LPENTRYID)rgprops[i].Value.bin.lpb, 
                        NULL, 
                        MAPI_MODIFY, 
                        NULL, 
                        (IUnknown **) &pfldrInbox);
            EXIT_ON_FAILED (hr);
        
            ASSERT (pfldrInbox);  

            entryListArray[i].cValues = 0;
            newMessagesArray[i].cValues = 0;
            updatedMessagesArray[i].cValues = 0;
            deletedMessagesArray[i] = new ArrayList();

            // ENTRYLIST entry;
            switch (mode) {
                
                //
                // dummy sync case used to get only the ListArray of all the mail in the folders
                // it doesn't use the namded extra ArrayList.
                //
                case ONLY_ALL_MESSAGE_LIST:
                    hr = GetAllMessages(entryListArray[i], pfldrInbox, path, folderToSync[i], NULL);                     
                break;
                
                case SYNC_SLOW:                                    
                    hr = GetAllMessages(entryListArray[i], pfldrInbox, path, folderToSync[i], extra);                     
                break;

                case SYNC_TWO_WAY:                                    
                    hr = GetModificatedMessages(newMessagesArray[i], updatedMessagesArray[i], 
                                            &deletedMessagesArray[i], pfldrInbox, path, folderToSync[i], extra); // the folder identification                                 
                break;

                default:
                break;
            }
        }        
    }
    
    // Clean up
    MAPIFreeBuffer (rgprops);    
    MAPIFreeBuffer (rgpropsMsg);  
    FreeProws   (psrs);     
    
    rgprops   =  NULL;  
    rgpropsMsg =  NULL; 
    psrs    =  NULL;    

    RELEASE_OBJ (pmsg);
    RELEASE_OBJ (pfldrInbox);
    RELEASE_OBJ (pStore);
  }

FuncExit:
  
  MAPIFreeBuffer (rgprops);
  MAPIFreeBuffer (rgpropsMsg);
  FreeProws   (psrs);

  RELEASE_OBJ (pmsg);
  RELEASE_OBJ (pfldrInbox);
  RELEASE_OBJ (pStore);
  RELEASE_OBJ (pTable);
  RELEASE_OBJ (pSession);
  
  if (rgTags)
      delete rgTags;

  return extra;
}

/*
* Write all the current mail in the message store to get the id, the last, the read 
*/

void MailClient2Server::writeCurrentMails(wchar_t* path, ArrayList* extra) {             

    SYSTEMTIME local;
    DATE last;
    IMAPISession *    pSession    = NULL;
    HRESULT           hr          = NULL;
    IMessage*         pmsg        = NULL;    
    ULONG             returned    = 0;       
    ULONG             cmsgValue   = 0;
    SPropValue*       msgprops    = NULL;
    wchar_t*          ret         = NULL;
    int j = 0;
    wchar_t wfilename[128];
    char filename [128];
    BOOL usedDefaultChar;
        
    BOOL read = FALSE;
    
    wchar_t tmp[64];
    GetSystemTime (&local);            
    wsprintf(tmp, TEXT("%i-%i-%iT%i-%i-%i"), local.wYear, local.wMonth, local.wDay, local.wHour, local.wMinute, local.wSecond);
    SystemTimeToVariantTime (&local, &last);
    
    for (int i = 0; i < folderNumber; i++) {
        
        int mailNum = entryListArray[i].cValues;

        FILE* f = NULL;

        wsprintf(wfilename, TEXT("%s%s_%c"),path, TEXT("mails"), folderToSync[i]);       
        WideCharToMultiByte (CP_ACP, 0, wfilename, 128 -1, filename, 128 -1, "?", &usedDefaultChar);

        f = fopen(filename, "w+");
        fwprintf(f, TEXT("<mails>\n"));
    
        fwprintf(f, TEXT("<num>%i</num>\n"), mailNum);
        fwprintf(f, TEXT("<last>%f</last>\n"), last);
        fwprintf(f, TEXT("<lastTime>%s</lastTime>\n"), tmp);        
        
        pSession = logOn();
    
        if (pSession == NULL)
            goto FuncExit;
                           
        for (j=0; j<mailNum; j++) {
        
            hr = pSession->OpenEntry(entryListArray[i].lpbin[j].cb, 
                (ENTRYID*)entryListArray[i].lpbin[j].lpb, 
                NULL, 
                MAPI_BEST_ACCESS, 
                &returned, 
                (IUnknown **) &pmsg);
            EXIT_ON_FAILED (hr);        
            ASSERT (pmsg);                            
        
            hr = pmsg->GetProps ((LPSPropTagArray)&endSync, MAPI_UNICODE, &cmsgValue,
                &msgprops);        
            fwprintf(f, TEXT("<item>\n"));

            if (msgprops[eeePR_ENTRYID].ulPropTag == PR_ENTRYID) {
                ret = convertBinaryToWChar(msgprops[eeePR_ENTRYID].Value.bin, folderToSync[i]);           
            }           
            
            fwprintf(f, TEXT("<id>%s</id>\n"), ret);       
                    
            if (msgprops[eeePR_MESSAGE_FLAGS].ulPropTag == PR_MESSAGE_FLAGS) {
                if (msgprops[eeePR_MESSAGE_FLAGS].Value.ul & MSGFLAG_READ) {
                    read = TRUE;
                }
                else {
                    read = FALSE;
                }
            }        
            fwprintf(f, TEXT("<read>%i</read>\n"), read);      
            
            //
            // insert extra data about mail...
            //                           
            
            MailExtraData* m = getMailExtraData(extra, ret);
            
            if (m) {
                fwprintf(f, TEXT("<message-id>%s</message-id>\n"), m->getMessageId()); 
            }

            fwprintf(f, TEXT("</item>\n"));       

            if (ret) {
                delete [] ret; ret = NULL;
            } 
        }   
        fwprintf(f, TEXT("</mails>\n"));        
        fflush(f);
        fclose(f);
    }

FuncExit:
   
    logOff(pSession);    
}

/**
* <mails>
*  <num>2</num>
*  <last>2334.4656</last>
*  <item>
*   <id>AAAAABBRD==</id>
*   <read>0</read>    // if the message is read (1) or not (0)
*  </item>
*  <item>
*   <id>AAAAABBRDFFJJJF==</id>
*   <read>1</read>    
*  </item>
* </mails>
*/

MailInfo* MailClient2Server::readPreviousMail(wchar_t* path, wchar_t folderId, ArrayList* extra) {
    
    wchar_t wfilename[128];
    char filename [128];
    BOOL usedDefaultChar;
    wchar_t* element = NULL;
    wchar_t line[100];
    wchar_t* tmp = NULL;
    FILE* f = NULL;
    ArrayList* list = new ArrayList();    
    MailItem mailItem; 
    int num = -1;
   
    wsprintf(wfilename, TEXT("%s%s_%c"),path, TEXT("mails"), folderId);
       
    WideCharToMultiByte (CP_ACP, 0, wfilename, 128 -1, filename, 128 -1, "?", &usedDefaultChar);
    
    f = fopen(filename, "r");
    
    // no file so a slow sync is perfomed
    if (f == NULL)
        return NULL;
    
    MailInfo* mailInfo = new MailInfo();
    MailExtraData mailExtraData = MailExtraData();
    while(fgetws(line, 99, f) != NULL) {        
         
        if ((element = getElementContent(line, TEXT("num"), NULL)) != NULL) {    
            
            mailInfo->setNum(element);
        }
        else if ((element = getElementContent(line, TEXT("last"), NULL)) != NULL) {                
            mailInfo->setLast(element);
        }
        else if (wcsstr(line, TEXT("<item>")) != NULL) {             
            mailItem = MailItem();        
        }
        else if ((element = getElementContent(line, TEXT("id"), NULL)) != NULL) {                
            mailItem.setId(element);
            mailExtraData.setId(element);
        }
        else if ((element = getElementContent(line, TEXT("read"), NULL)) != NULL) {            
            mailItem.setRead(element);                        
        }
        else if ((element = getElementContent(line, TEXT("message-id"), NULL)) != NULL) {            
            mailExtraData.setMessageId(element);                        
        }
        else if (wcsstr(line, TEXT("</item>")) != NULL) {            
            mailInfo->addMailItem(mailItem);
            extra->add(mailExtraData);
        }
        if (element) {
            delete [] element; element = NULL;
        }
    }
    fflush(f);
    fclose(f);
    
    return mailInfo;
}


 /*
        for (unsigned int i = 0; i < entryList.cValues; i++) {
            
            hr = pSession->OpenEntry(entryList.lpbin[i].cb, 
                (ENTRYID*)entryList.lpbin[i].lpb, 
                NULL, 
                MAPI_BEST_ACCESS, 
                &returned, 
                (IUnknown **) &pmsg);
            EXIT_ON_FAILED (hr);        
            ASSERT (pmsg);   
            // (LPSPropTagArray)&properties
            // (SPropTagArray *) rgTags22
            hr = pmsg->GetProps ((LPSPropTagArray)&properties, 0, &cmsgValue,
               &msgprops);
            EXIT_ON_FAILED (hr);
            
            ASSERT (msgprops);
            FILETIME s = msgprops[ePR_LAST_MODIFICATION_TIME].Value.ft;       
            wchar_t* name = msgprops[ePR_DISPLAY_TO].Value.lpszW;  
            wchar_t* name1 = msgprops[ePR_TRANSPORT_MESSAGE_HEADERS].Value.lpszW;              
            name = msgprops[ePR_SUBJECT].Value.lpszW; 
        

            ////////////////////////////////////////////////////////////////
            
            LPMESSAGE pMessage = NULL; 
            LPSTREAM pStream = NULL; 
            char  szBuf[255]; 
            ULONG  ulNumChars = 0; 
            // CString  strBody; 


             //hr = pmsg->OpenProperty( PR_BODY, &IID_IStream, STGM_READ, NULL, 
             //        (LPUNKNOWN *) &pStream); 
            hr = pmsg->OpenProperty( PR_DISPLAY_TO, NULL, STGM_READ, NULL, 
                     (LPUNKNOWN *) &pStream); 
            hr = pStream->Read(szBuf, 255, &ulNumChars);  
            
            
            hr = pmsg->OpenProperty( PR_BODY, NULL, STGM_READ, NULL, 
                     (LPUNKNOWN *) &pStream); 
               if (hr != S_OK ) 
               { 
                  return 1; 
               } 


             hr = pStream->Read(szBuf, 255, &ulNumChars); 
            //strBody += szBuf;
            
              
            hr = pmsg->OpenProperty( PR_TRANSPORT_MESSAGE_HEADERS, NULL, STGM_READ, NULL, 
                     (LPUNKNOWN *) &pStream); 
            hr = pStream->Read(szBuf, 255, &ulNumChars);  
              
            //////////////////////////////////////////////////////////////////               
              
        }
        */
