/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pim/mail/MailServer2Client.h"
#include "base/util/utils.h"
#include "base/util/BasicTime.h"

#include "base/Log.h"


MailServer2Client::MailServer2Client() {
    server_folderToSync = NULL;
    
}

MailServer2Client::~MailServer2Client() {
    if (server_folderToSync) {
        delete [] server_folderToSync; server_folderToSync = NULL;
    }
}
enum {
	eeePR_ENTRYID,
	eeePR_MESSAGE_FLAGS,
    eeePR_LAST_MODIFICATION_TIME,
	eeeNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eeeNUM_COLS, endSync) = 
{	
	eeeNUM_COLS,
    PR_ENTRYID,                       // entry id
	PR_MESSAGE_FLAGS,    
    PR_LAST_MODIFICATION_TIME        //

};

 enum {
	ePR_ENTRYID,
	ePR_MESSAGE_FLAGS,
    //MM        
    ePR_LAST_MODIFICATION_TIME,
    ePR_SUBJECT,
    ePR_SUBJECT_PREFIX,
    ePR_MESSAGE_SIZE,    
    ePR_SENDER_NAME,        
    ePR_SENDER_EMAIL_ADDRESS,    
    ePR_HASATTACH,
    //MM
	NUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(NUM_COLS, properties) = 
{	
	NUM_COLS,
    PR_ENTRYID,                         // entry id
	PR_MESSAGE_FLAGS,
    //MM        
    PR_LAST_MODIFICATION_TIME,          //
    PR_SUBJECT,                         // mail Subject
    PR_SUBJECT_PREFIX,
    PR_MESSAGE_SIZE,                    // sum of the size of every property of the object
    PR_SENDER_NAME,                     // "Marco "
    PR_SENDER_EMAIL_ADDRESS,            // From  "Marco <magi@libero.it>    
    PR_HASATTACH

};

enum {
	eePR_ENTRYID,
    eePR_LAST_MODIFICATION_TIME,
	eePR_DISPLAY_NAME,     
    eePR_MESSAGE_FLAGS,
    eeNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eeNUM_COLS, Columns) = 
{	
	eeNUM_COLS,
	PR_ENTRYID,
    PR_LAST_MODIFICATION_TIME,
	PR_DISPLAY_NAME,
    PR_MESSAGE_FLAGS
};
ULONG CountRecipsInString(LPCWSTR psz)
{
  ULONG cRecips = 0;

  if (psz != NULL)
  {
    if (*psz != '\0')
      ++cRecips;

    while (*psz != L'\0')
    {
      if (*psz == L';' || *psz == L',') 
        ++cRecips;
      ++psz;
    }
  }

  return cRecips;
}

/*
* Check if the t char starts with I/, O/, D/, T/, S/
*/
BOOL startWithFolder(wchar_t* t) {
    BOOL ret = FALSE;
    if (wcsstr(t, TEXT("I/")) != NULL ||
            wcsstr(t, TEXT("O/")) != NULL ||
            wcsstr(t, TEXT("D/")) != NULL ||
            wcsstr(t, TEXT("S/")) != NULL ||
            wcsstr(t, TEXT("T/")) != NULL )
        
        ret = TRUE;
    
    return ret;

}


HRESULT MailServer2Client::AddAttachment(LPMESSAGE pmsg, MailClientData* m)
{
	HRESULT hr = S_OK;
    BodyPart* bodyPart = NULL;
    bodyPart = m->getEmailData()->getEmailItem().getFirstAttachment();

    do {
        if (bodyPart == NULL)
            break;
        
        wchar_t* tmpFileName = (wchar_t*)bodyPart->getContent();
        wchar_t* filename    = (wchar_t*)bodyPart->getFilename();

        SPropValue  prop[3];
		IAttach*    pAttachment = NULL;
		ULONG       ulAttachmentNum = 0;
		FILE*       fileAttach = NULL;;
        
        memset(prop, 0, sizeof(prop));
        fileAttach = _wfopen(tmpFileName, TEXT("rb"));	
        
        if (fileAttach == NULL) {
            hr = -1;
            break;
        }
        
        hr = pmsg->CreateAttach(NULL, 0, &ulAttachmentNum, &pAttachment);
		EXIT_ON_FAILED(hr);

		// Set the initial properties on the attachment - no data actually there yet
		//
		prop[0].ulPropTag = PR_ATTACH_METHOD;
		prop[0].Value.ul = ATTACH_BY_VALUE;

		prop[1].ulPropTag = PR_ATTACH_DATA_BIN;
        
        prop[2].ulPropTag = PR_ATTACH_FILENAME;
		prop[2].Value.LPSZ = filename;
        
        hr = pAttachment->SetProps(3, prop, NULL);
		EXIT_ON_FAILED(hr);

		// Now get the data property and fill it in
		//
		IStream* pStream;

		hr = pAttachment->OpenProperty (PR_ATTACH_DATA_BIN,
									 NULL,
									 0,
									 MAPI_MODIFY | MAPI_CREATE,
									 (LPUNKNOWN *)&pStream);
		EXIT_ON_FAILED(hr);
        
        //DWORD cbFile = GetFileSize(fileAttach, NULL);
        DWORD cbFile = fgetsize(fileAttach);
        if (cbFile == 0xFFFFFFFF) // error
            break;
        
		ULONG        cbWritten;
		ULONG        cbReadSize;
		const ULONG  ChunkSize = 4096;
		BYTE         Buffer[ChunkSize];

		do {
			// cbReadSize = fileAttach.Read(Buffer, ChunkSize);
            cbReadSize = fread(Buffer, sizeof(BYTE), ChunkSize, fileAttach ); 
                
			if (cbReadSize > 0)
			{
				hr = pStream->Write(Buffer, cbReadSize, &cbWritten);
			}
		} while (!FAILED(hr) && (cbReadSize == ChunkSize));

		fflush(fileAttach);
        fclose(fileAttach);
		pStream->Commit(STGC_DEFAULT);
		pStream->Release();

		// Set the size property
		//
		prop[0].ulPropTag = PR_ATTACH_SIZE;
		prop[0].Value.ul = cbFile + sizeof(prop[2]);
		hr = pAttachment->SetProps(1, prop, NULL);                
        
        if (tmpFileName != NULL) {
            DeleteFile(tmpFileName);
        }


    } while ((bodyPart = m->getEmailData()->getEmailItem().getNextAttachment()) != NULL);

FuncExit:    
    return hr;   
}



WCHAR szSMTP[] = L"SMTP";
void AddRecipients(LPWSTR pszSrc, ULONG ulType, ADRLIST* plst, LPWSTR& pszLocalRecips, LPSPropValue& pval)
{
  if (pszSrc == NULL)
    return;

  LPWSTR psz = pszSrc;
  while (*psz != L'\0')
  {
    while (*psz == L' ' && *psz != '\0')
      ++psz;

    if (*psz == L'\0')
      break;

    LPWSTR pszEnd = psz;
    // while (*pszEnd != L' ' && (*pszEnd != ';' || *pszEnd != ',') && *pszEnd != '\0')
    while (*pszEnd != ',' && *pszEnd != '\0')
      ++pszEnd;

    int cch = pszEnd - psz;

    wcsncpy(pszLocalRecips, psz, cch);
    *(pszLocalRecips + cch) = L'\0';

    plst->aEntries[plst->cEntries].cValues = 3;
    plst->aEntries[plst->cEntries].rgPropVals = pval;

    // Set the type (To, Cc, Bcc)...
    pval->ulPropTag = PR_RECIPIENT_TYPE;
    pval->Value.ul = ulType;
    ++pval;
    
    // Set the address type (we only do SMTP)...
    pval->ulPropTag = PR_ADDRTYPE;
    pval->Value.lpszW = szSMTP;
    ++pval;
    
    // Set the address...
    pval->ulPropTag = PR_EMAIL_ADDRESS;
    pval->Value.lpszW = pszLocalRecips;
    ++pval;

    ++plst->cEntries;

    pszLocalRecips += wcslen(pszLocalRecips) + 1;

    if (*pszEnd != L'\0')
      ++pszEnd;

    psz = pszEnd;
  }
}

HRESULT MailServer2Client::SetMessageRecipients (LPMESSAGE pmsg, MailClientData* m)
{
  ULONG cRecipients = 0;
  ULONG cchTotalRecips = 0;

  ADRLIST* plst = NULL;
  LPSPropValue pval = NULL;
  BYTE* pb = NULL;
  LPWSTR pszLocalRecips;
  HRESULT hr;
  
  if (m->getEmailData()->getEmailItem().getTo())
  {
    cRecipients += CountRecipsInString(m->getEmailData()->getEmailItem().getTo());
    cchTotalRecips += wcslen(m->getEmailData()->getEmailItem().getTo()) + 3;
  }

  if (m->getEmailData()->getEmailItem().getCc())
  {
    cRecipients += CountRecipsInString(m->getEmailData()->getEmailItem().getCc());
    cchTotalRecips += wcslen(m->getEmailData()->getEmailItem().getCc()) + 3;
  }

  if (m->getEmailData()->getEmailItem().getBcc())
  {
    cRecipients += CountRecipsInString(m->getEmailData()->getEmailItem().getBcc());
    cchTotalRecips += wcslen(m->getEmailData()->getEmailItem().getBcc()) + 3;
  }
  
//
  // Allocate one big block of memory to hold all the strings.
  // The block is arranged as follows:
  //
  // ADRLIST        SPropValue's     Copy of addresses
  // |---------------------|--------------------|-------------------|
  //

  DWORD cb = sizeof(ADRLIST) + ((sizeof(ADRENTRY) + sizeof(SPropValue) * 3) * cRecipients);

  pb = new BYTE[cb + (cchTotalRecips * sizeof(WCHAR))];
  if (pb == NULL)
    return E_OUTOFMEMORY;

  ZeroMemory(pb, cb + (cchTotalRecips * sizeof(WCHAR)));

  plst = (ADRLIST*) pb;

  pszLocalRecips = (LPWSTR) (pb + cb);

  pb += sizeof(ADRLIST) + (sizeof(ADRENTRY) * cRecipients);

  pval = (SPropValue*) pb;

  AddRecipients((wchar_t*)m->getEmailData()->getEmailItem().getTo(), MAPI_TO, plst, pszLocalRecips, pval);
  AddRecipients((wchar_t*)m->getEmailData()->getEmailItem().getCc(), MAPI_CC, plst, pszLocalRecips, pval);
  AddRecipients((wchar_t*)m->getEmailData()->getEmailItem().getBcc(), MAPI_BCC, plst, pszLocalRecips, pval);

  hr = pmsg->ModifyRecipients(MODRECIP_ADD, plst);

  delete[] (BYTE*) plst;
  
  return hr;
}

HRESULT MailServer2Client::SetMessageProps(LPMESSAGE pmsg, MailClientData* m) {
  
  SPropValue rgprops[6] = {0};
  ULONG    cProps   = 0;
  HRESULT   hr;
  BodyPart bp;
  BasicTime b;

  LPSTREAM  pstm    = NULL;
  ULONG messageFlag = (ULONG) 0x00000000;

  // Set the recipients up.
  hr = SetMessageRecipients(pmsg, m);
  EXIT_ON_FAILED(hr);

  messageFlag = messageFlag | MSGFLAG_UNSENT;

  if (m->getEmailData()->getRead())
    messageFlag = messageFlag | MSGFLAG_READ;
  
  //
  // To be an editable message the MSGFLAG_FROMME has to be set.
  // surely the mail into draft folder has to be flagged MSGFLAG_FROMME.
  // X-Funambol-isDraft
  
  if (wcscmp(m->getParent(), c2wc('D')) == 0)
      messageFlag = messageFlag | MSGFLAG_FROMME;
 
  // Set the flags and a subject if they exist.
  rgprops[cProps].ulPropTag = PR_MESSAGE_FLAGS;
  rgprops[cProps].Value.ul = messageFlag;  
  //rgprops[cProps].Value.ul = MSGFLAG_FROMME | MSGFLAG_UNSENT;  
  // rgprops[cProps].Value.ul = MSGFLAG_READ;
  ++cProps;
  
  rgprops[cProps].ulPropTag = PR_MSG_STATUS;
  rgprops[cProps].Value.ul = MSGSTATUS_RECTYPE_SMTP;
  ++cProps;

  if (m->getEmailData()->getEmailItem().getSubject() != NULL)
  {
    rgprops[cProps].ulPropTag = PR_SUBJECT;
    rgprops[cProps].Value.lpszW = (wchar_t*)m->getEmailData()->getEmailItem().getSubject();
    ++cProps;
  }
  if (m->getEmailData()->getEmailItem().getFrom() != NULL)
  {
    rgprops[cProps].ulPropTag = PR_SENDER_EMAIL_ADDRESS;
    rgprops[cProps].Value.lpszW = (wchar_t*)m->getEmailData()->getEmailItem().getFrom();
    ++cProps;
  }
  
    b = m->getEmailData()->getEmailItem().getDate();    
    SYSTEMTIME local;
    FILETIME f;

    local.wYear         = b.getYear();
    local.wMonth        = b.getMonth();
    local.wDayOfWeek    = b.getWeekday();
    local.wDay          = b.getDay();
    local.wHour         = b.getHour();
    local.wMinute       = b.getMin();
    local.wSecond       = b.getSec();

    SystemTimeToFileTime(&local, &f);    
    rgprops[cProps].ulPropTag = PR_MESSAGE_DELIVERY_TIME;
    rgprops[cProps].Value.ft = f;
    ++cProps;
  

  // MODIFY THE  rgprops TOO!|!!!!!!!!
  //
  
  if (m->getEmailData()->getEmailItem().attachmentCount() > 0)
  {
    rgprops[cProps].ulPropTag = PR_HASATTACH;
    rgprops[cProps].Value.ul = m->getEmailData()->getEmailItem().attachmentCount();
    ++cProps;
  }
  
  // Body  
  bp = m->getEmailData()->getEmailItem().getBody();      
  if (bp.getContent()) {
      // Stream the body in...
      hr = pmsg->OpenProperty(PR_BODY, NULL, 0, MAPI_MODIFY | MAPI_CREATE, 
          (LPUNKNOWN*)&pstm);
      EXIT_ON_FAILED(hr);

      pstm->Write(m->getEmailData()->getEmailItem().getBody().getContent(), (wcslen (m->getEmailData()->getEmailItem().getBody().getContent()) + 1) * sizeof (WCHAR), NULL);
      pstm->Release();
    
  }
  
  // Attachment
  AddAttachment(pmsg, m);

  hr = pmsg->SetProps(cProps, rgprops, NULL);
  EXIT_ON_FAILED(hr);

FuncExit:
  return hr;
}

void MailServer2Client::setFolderToSync(wchar_t* t){
    if (server_folderToSync) {
        delete [] server_folderToSync; server_folderToSync = NULL;
    }
    server_folderToSync = stringdup(t);
}

wchar_t* MailServer2Client::getFolderToSync() {
    return server_folderToSync;
}

/*
* Return the ID of the inserted message
*/

wchar_t* MailServer2Client::addMessage(MailClientData* m) {
    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    IMAPITable*       pTable      =  NULL;
    SRowSet*          psrs        =  NULL;
    IMsgStore*        pStore      =  NULL;
    int               i           =  0;
    ULONG*   rgTags;
    folderNumber = 0;
       
    if (server_folderToSync == NULL) {
        rgTags = new ULONG[2];
        rgTags[0] = 1; 
        rgTags[1] = PR_CE_IPM_INBOX_ENTRYID; 
        folderNumber = 1;
    }
    else {
        folderNumber = wcslen(server_folderToSync);
        rgTags = new ULONG[folderNumber + 1];         
        rgTags[0] = folderNumber;
        for (int i = 1; i <= folderNumber; i++) 
            rgTags[i] = getFolderId(server_folderToSync[i-1]);    // PR_CE_IPM_INBOX_ENTRYID; 
                                                      // PR_CE_IPM_DRAFTS_ENTRYID;        
    }
    
    ULONG             rgMsgTags[] =  { 1, PR_ENTRYID };
    ULONG             cValues     =  0;
    ULONG             returned    =  0;
    
    SPropValue*       rgprops     =  NULL;
    SPropValue*       rgpropsMsg  =  NULL;
    LPMAPIFOLDER      pfldrInbox  =  NULL;

    IMessage*         pmsg        =  NULL;
    ULONG             cmsgValue   =  0;
    SPropValue*       msgprops    =  NULL;
            
	LPSRowSet	pRows = NULL;    
    SPropValue*       rgprops2     =  NULL;
    ULONG             cValues2     =  0;
    wchar_t* ret = NULL;
	       
    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    EXIT_ON_FAILED (hr);

    // Get the message stores table
    hr = pSession->GetMsgStoresTable (MAPI_UNICODE, &pTable);
    EXIT_ON_FAILED (hr);
   
    hr = pTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);
  
    while (1){
    // Get a row
    hr = pTable->QueryRows (1, 0, &psrs);
    EXIT_ON_FAILED (hr);
    
    // Did we hit the end of the table?
    if (psrs->cRows != 1) {
      break;
    }
    //
    // the display name is the name of the account.
    //
    wchar_t* displayName = psrs->aRow[0].lpProps[eePR_DISPLAY_NAME].Value.lpszW;
    
    //
    // The name of the MessageStore that we use for the own Transport
    //
    if (wcscmp(displayName, ACCOUNT_NAME) == 0) {

        // Make sure we got the props we need
        if ((psrs->aRow[0].cValues < 1) || (psrs->aRow[0].lpProps[0].ulPropTag != PR_ENTRYID))
        {
          MessageBox (NULL, L"Store missing PR_ENTRYID!", L"Doh", MB_OK);
          hr = E_FAIL;
          break;
        }

        // Open this message store
        hr = pSession->OpenMsgStore (NULL, 
                       psrs->aRow[0].lpProps[0].Value.bin.cb, 
                      (ENTRYID *) psrs->aRow[0].lpProps[0].Value. 
                       bin.lpb, NULL, 0, &pStore);
        EXIT_ON_FAILED (hr);
  
        // Now get the Inbox folder.
        hr = pStore->GetProps ((SPropTagArray *) rgTags, MAPI_UNICODE, &cValues, &rgprops);
        EXIT_ON_FAILED (hr);
  
        ASSERT (rgprops);
               
        //
        // watch at the folder that we want to sync
        //
        for (i = 0; i < folderNumber; i++) {
            wchar_t* tmp = c2wc(server_folderToSync[i]);

            if (wcscmp(tmp, m->getParent()) != 0) {
                if (tmp) delete [] tmp; tmp = NULL;
                continue;  }
        
            // open the message store
            hr = pStore->OpenEntry (rgprops[i].Value.bin.cb, 
                        (LPENTRYID)rgprops[i].Value.bin.lpb, 
                        NULL, 
                        MAPI_MODIFY, 
                        NULL, 
                        (IUnknown **) &pfldrInbox);
            EXIT_ON_FAILED (hr);
        
            ASSERT (pfldrInbox);  
            
            hr = pfldrInbox->CreateMessage (NULL, 0, &pmsg);
            EXIT_ON_FAILED (hr);
  
            ASSERT (pmsg);
                            
            // Now set the props on the message....
            hr = SetMessageProps (pmsg, m);
            EXIT_ON_FAILED(hr);
            
            //
            // Get the ID of the new message
            //
            hr = pmsg->GetProps ((SPropTagArray *) rgMsgTags, MAPI_UNICODE, &cValues2, &rgprops2);
            EXIT_ON_FAILED(hr);
            
            if (rgprops2[eePR_ENTRYID].ulPropTag == PR_ENTRYID) {
                ret = convertBinaryToWChar(rgprops2[eePR_ENTRYID].Value.bin, server_folderToSync[i]);           
            }
            
        }        
    }
    if (displayName) {
        delete [] displayName; displayName = NULL;
    }
    
    // Clean up
    MAPIFreeBuffer (rgprops);    
    MAPIFreeBuffer (rgprops2);    
    MAPIFreeBuffer (rgpropsMsg);  
    FreeProws   (psrs);     
    
    rgprops   =  NULL;  
    rgpropsMsg =  NULL; 
    psrs    =  NULL;    

    RELEASE_OBJ (pmsg);
    RELEASE_OBJ (pfldrInbox);
    RELEASE_OBJ (pStore);
  }

FuncExit:
  
  MAPIFreeBuffer (rgprops);
  MAPIFreeBuffer (rgpropsMsg);
  FreeProws   (psrs);

  RELEASE_OBJ (pmsg);
  RELEASE_OBJ (pfldrInbox);
  RELEASE_OBJ (pStore);
  RELEASE_OBJ (pTable);
  RELEASE_OBJ (pSession);
  
  if (rgTags)
      delete rgTags;

  return ret;
}


/*
* Return the ID of the inserted message
*/

HRESULT MailServer2Client::updateMessage(MailClientData* m) {
    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    IMAPITable*       pTable      =  NULL;
    SRowSet*          psrs        =  NULL;
    IMsgStore*        pStore      =  NULL;
    int               i           =  0;
    ULONG*   rgTags;
    folderNumber = 0;
       
    if (server_folderToSync == NULL) {
        rgTags = new ULONG[2];
        rgTags[0] = 1; 
        rgTags[1] = PR_CE_IPM_INBOX_ENTRYID; 
        folderNumber = 1;
    }
    else {
        folderNumber = wcslen(server_folderToSync);
        rgTags = new ULONG[folderNumber + 1];         
        rgTags[0] = folderNumber;
        for (int i = 1; i <= folderNumber; i++) 
            rgTags[i] = getFolderId(server_folderToSync[i-1]);    // PR_CE_IPM_INBOX_ENTRYID; 
                                                      // PR_CE_IPM_DRAFTS_ENTRYID;        
    }
    
    ULONG             rgMsgTags[] =  { 1, PR_ENTRYID };
    ULONG             cValues     =  0;
    ULONG             returned    =  0;
    
    SPropValue*       rgprops     =  NULL;
    SPropValue*       rgpropsMsg  =  NULL;
    LPMAPIFOLDER      pfldrInbox  =  NULL;

    IMessage*         pmsg        =  NULL;
    ULONG             cmsgValue   =  0;
    SPropValue*       msgprops    =  NULL;
            
	LPSRowSet	      pRows        = NULL;    
    SPropValue*       rgprops2     =  NULL;
    ULONG             cValues2     =  0;
    wchar_t*          ret         = NULL;
    wchar_t*          displayName  = NULL;
	       
    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    EXIT_ON_FAILED (hr);

    // Get the message stores table
    hr = pSession->GetMsgStoresTable (MAPI_UNICODE, &pTable);
    EXIT_ON_FAILED (hr);
   
    hr = pTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);
  
    while (1){
    // Get a row
    hr = pTable->QueryRows (1, 0, &psrs);
    EXIT_ON_FAILED (hr);
    
    // Did we hit the end of the table?
    if (psrs->cRows != 1) {
      break;
    }
    //
    // the display name is the name of the account.
    //
    displayName = psrs->aRow[0].lpProps[eePR_DISPLAY_NAME].Value.lpszW;
    
    //
    // The name of the MessageStore that we use for the own Transport
    //
    if (wcscmp(displayName, ACCOUNT_NAME) == 0) {

        // Make sure we got the props we need
        if ((psrs->aRow[0].cValues < 1) || (psrs->aRow[0].lpProps[0].ulPropTag != PR_ENTRYID))
        {
          MessageBox (NULL, L"Store missing PR_ENTRYID!", L"Doh", MB_OK);
          hr = E_FAIL;
          break;
        }

        // Open this message store
        hr = pSession->OpenMsgStore (NULL, 
                       psrs->aRow[0].lpProps[0].Value.bin.cb, 
                      (ENTRYID *) psrs->aRow[0].lpProps[0].Value. 
                       bin.lpb, NULL, 0, &pStore);
        EXIT_ON_FAILED (hr);
  
        // Now get the Inbox folder.
        hr = pStore->GetProps ((SPropTagArray *) rgTags, MAPI_UNICODE, &cValues, &rgprops);
        EXIT_ON_FAILED (hr);
  
        ASSERT (rgprops);
               
        //
        // watch at the folder that we want to sync
        //
        for (i = 0; i < folderNumber; i++) {
            wchar_t* t = c2wc(server_folderToSync[i]);

            if (wcscmp(t, m->getParent()) != 0) {
                if (t) {
                    delete [] t; t = NULL; 
                }
                continue; 
            }
        
            // open the message store
            hr = pStore->OpenEntry (rgprops[i].Value.bin.cb, 
                        (LPENTRYID)rgprops[i].Value.bin.lpb, 
                        NULL, 
                        MAPI_MODIFY, 
                        NULL, 
                        (IUnknown **) &pfldrInbox);
            EXIT_ON_FAILED (hr);
        
            ASSERT (pfldrInbox);  
                        
            wchar_t* tmpID = (wchar_t*)m->getEmailData()->getEmailItem().getEntryID();
            wchar_t* id = NULL;

            if (startWithFolder(tmpID))
                id = stringdup(&tmpID[2]);
            else
                id = stringdup(tmpID);
            
            char*    tmp = NULL;
            int      len = 0;          

            tmp = wc2utf8(id);

            len = b64_decode(tmp, tmp);
            tmp[len] = 0;                                                
            
            ENTRYLIST entryList;
            
            entryList.cValues = 0;
			MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(entryList.lpbin));
            entryList.lpbin[entryList.cValues].cb = len;
		    MAPIAllocateBuffer(len, (LPVOID*)&(entryList.lpbin[entryList.cValues].lpb) );
		    memmove(entryList.lpbin[entryList.cValues].lpb, tmp, len);
		    entryList.cValues++;
            
            hr = pSession->OpenEntry(entryList.lpbin[0].cb, 
                (ENTRYID*)entryList.lpbin[0].lpb, 
                NULL, 
                MAPI_BEST_ACCESS, 
                &returned, 
                (IUnknown **) &pmsg);

            EXIT_ON_FAILED (hr);        
            ASSERT (pmsg);     

            if (tmp) {
                delete [] tmp; tmp = NULL;
            }
            if (id) {
                delete [] tmp; tmp = NULL;
            }
            
            // Now set the props on the message....
            hr = SetMessageProps (pmsg, m);
            EXIT_ON_FAILED(hr);                       
           
            MAPIFreeBuffer (entryList.lpbin);    
        }        
    }
    
    // Clean up
    MAPIFreeBuffer (rgprops);    
    MAPIFreeBuffer (rgprops2);    
    MAPIFreeBuffer (rgpropsMsg);  
    FreeProws   (psrs);     
    
    rgprops   =  NULL;  
    rgpropsMsg =  NULL; 
    psrs    =  NULL;    

    RELEASE_OBJ (pmsg);
    RELEASE_OBJ (pfldrInbox);
    RELEASE_OBJ (pStore);
  }

FuncExit:
  
  MAPIFreeBuffer (rgprops);
  MAPIFreeBuffer (rgpropsMsg);
  FreeProws   (psrs);

  RELEASE_OBJ (pmsg);
  RELEASE_OBJ (pfldrInbox);
  RELEASE_OBJ (pStore);
  RELEASE_OBJ (pTable);
  RELEASE_OBJ (pSession);
  
  if (rgTags)
      delete rgTags;

  return hr;
}



/*
* Return the ID of the inserted message
*/

HRESULT MailServer2Client::deleteMessage(MailClientData* m) {
    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    IMAPITable*       pTable      =  NULL;
    SRowSet*          psrs        =  NULL;
    IMsgStore*        pStore      =  NULL;
    int               i           =  0;
    ULONG*   rgTags;
    folderNumber = 0;
       
    if (server_folderToSync == NULL) {
        rgTags = new ULONG[2];
        rgTags[0] = 1; 
        rgTags[1] = PR_CE_IPM_INBOX_ENTRYID; 
        folderNumber = 1;
    }
    else {
        folderNumber = wcslen(server_folderToSync);
        rgTags = new ULONG[folderNumber + 1];         
        rgTags[0] = folderNumber;
        for (int i = 1; i <= folderNumber; i++) 
            rgTags[i] = getFolderId(server_folderToSync[i-1]);    // PR_CE_IPM_INBOX_ENTRYID; 
                                                      // PR_CE_IPM_DRAFTS_ENTRYID;        
    }
    
    ULONG             rgMsgTags[] =  { 1, PR_ENTRYID };
    ULONG             cValues     =  0;
    ULONG             returned    =  0;
    
    SPropValue*       rgprops     =  NULL;
    SPropValue*       rgpropsMsg  =  NULL;
    LPMAPIFOLDER      pfldrInbox  =  NULL;

    IMessage*         pmsg        =  NULL;
    ULONG             cmsgValue   =  0;
    SPropValue*       msgprops    =  NULL;
            
	LPSRowSet	      pRows        = NULL;    
    SPropValue*       rgprops2     =  NULL;
    ULONG             cValues2     =  0;
    wchar_t*          ret         = NULL;
    wchar_t*          displayName  = NULL;
	       
    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    EXIT_ON_FAILED (hr);

    // Get the message stores table
    hr = pSession->GetMsgStoresTable (MAPI_UNICODE, &pTable);
    EXIT_ON_FAILED (hr);
   
    hr = pTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);
  
    while (1){
    // Get a row
    hr = pTable->QueryRows (1, 0, &psrs);
    EXIT_ON_FAILED (hr);
    
    // Did we hit the end of the table?
    if (psrs->cRows != 1) {
      break;
    }
    //
    // the display name is the name of the account.
    //
    displayName = psrs->aRow[0].lpProps[eePR_DISPLAY_NAME].Value.lpszW;
    
    //
    // The name of the MessageStore that we use for the own Transport
    //
    if (wcscmp(displayName, ACCOUNT_NAME) == 0) {

        // Make sure we got the props we need
        if ((psrs->aRow[0].cValues < 1) || (psrs->aRow[0].lpProps[0].ulPropTag != PR_ENTRYID))
        {
          MessageBox (NULL, L"Store missing PR_ENTRYID!", L"Doh", MB_OK);
          hr = E_FAIL;
          break;
        }

        // Open this message store
        hr = pSession->OpenMsgStore (NULL, 
                       psrs->aRow[0].lpProps[0].Value.bin.cb, 
                      (ENTRYID *) psrs->aRow[0].lpProps[0].Value. 
                       bin.lpb, NULL, 0, &pStore);
        EXIT_ON_FAILED (hr);
  
        // Now get the Inbox folder.
        hr = pStore->GetProps ((SPropTagArray *) rgTags, MAPI_UNICODE, &cValues, &rgprops);
        EXIT_ON_FAILED (hr);
  
        ASSERT (rgprops);
               
        //
        // watch at the folder that we want to sync
        //
        for (i = 0; i < folderNumber; i++) {
            wchar_t* t = c2wc(server_folderToSync[i]);
          
            if (wcscmp(t, m->getParent()) != 0) {
                if (t) {
                    delete [] t; t = NULL; 
                }
                continue; 
            }
        
            // open the message store
            hr = pStore->OpenEntry (rgprops[i].Value.bin.cb, 
                        (LPENTRYID)rgprops[i].Value.bin.lpb, 
                        NULL, 
                        MAPI_MODIFY, 
                        NULL, 
                        (IUnknown **) &pfldrInbox);
            EXIT_ON_FAILED (hr);
        
            ASSERT (pfldrInbox);  
                        
            wchar_t* tmpID = (wchar_t*)m->getEmailData()->getEmailItem().getEntryID();
            wchar_t* id = NULL;
            
            if (startWithFolder(tmpID))
                id = stringdup(&tmpID[2]);
            else
                id = stringdup(tmpID);            
            
            char*    tmp = NULL;
            int      len = 0;          

            tmp = wc2utf8(id);

            len = b64_decode(tmp, tmp);
            tmp[len] = 0;                                                
            
            ENTRYLIST entryList;
            
            entryList.cValues = 0;
			MAPIAllocateBuffer(sizeof(SBinary), (LPVOID*)&(entryList.lpbin));
            entryList.lpbin[entryList.cValues].cb = len;
		    MAPIAllocateBuffer(len, (LPVOID*)&(entryList.lpbin[entryList.cValues].lpb) );
		    memmove(entryList.lpbin[entryList.cValues].lpb, tmp, len);
		    entryList.cValues++;
            
            if (tmp) {
                delete [] tmp; tmp = NULL;
            }
            if (id) {
                delete [] tmp; tmp = NULL;
            }
            hr = pfldrInbox->DeleteMessages (&entryList, 0, NULL, 0);
            EXIT_ON_FAILED (hr);
           
            MAPIFreeBuffer (entryList.lpbin);    
        }        
    }
    
    // Clean up
    MAPIFreeBuffer (rgprops);    
    MAPIFreeBuffer (rgprops2);    
    MAPIFreeBuffer (rgpropsMsg);  
    FreeProws   (psrs);     
    
    rgprops   =  NULL;  
    rgpropsMsg =  NULL; 
    psrs    =  NULL;    

    RELEASE_OBJ (pmsg);
    RELEASE_OBJ (pfldrInbox);
    RELEASE_OBJ (pStore);
  }

FuncExit:
  
  MAPIFreeBuffer (rgprops);
  MAPIFreeBuffer (rgpropsMsg);
  FreeProws   (psrs);

  RELEASE_OBJ (pmsg);
  RELEASE_OBJ (pfldrInbox);
  RELEASE_OBJ (pStore);
  RELEASE_OBJ (pTable);
  RELEASE_OBJ (pSession);
  
  if (rgTags)
      delete rgTags;

  return hr;
}