/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pim/mail/MailSyncSource.h"
#include "spds/SyncItemStatus.h"
#include "spds/MailMessage.h"
#include "spds/constants.h"
#include "base/util/utils.h"
#include "base/util/StringBuffer.h"
#include "base/Log.h"

static int call;
static int cnew;
static int cupdated;
static int cdeleted;

BOOL MailSyncSource::isMailInOutbox = 0;
BOOL MailSyncSource::isMailInInbox = 0;


MailSyncSource::MailSyncSource(const wchar_t* name) : SyncSource(name), mailClient2Server(), 
                                                      mailClient2ServerEnd(), mailServer2Client() {    
    setPath(TEXT("\\"));
    syncItem = NULL;
    s_folderToSync = NULL;   
    setIsMailInOutbox(FALSE);
    setIsMailInInbox(FALSE);
    extras = new ArrayList();
}

MailSyncSource::~MailSyncSource() {    
    if (syncItem) {
        delete syncItem; syncItem = NULL;
    }
    if (s_folderToSync) {
        delete [] s_folderToSync; s_folderToSync = NULL;
    }
    if (path) {
        delete [] path; path = NULL;
    }
    if (extras) {
        delete extras;
    }
}

void MailSyncSource::setPath(wchar_t* p) {
    if (p == NULL)
        path = stringdup(TEXT(""));
    else
        path = stringdup(p);
}

wchar_t* MailSyncSource::getPath() {
    return path;
}

void MailSyncSource::setFolderToSync(wchar_t* t){
    safeDelete(&s_folderToSync);
    s_folderToSync = stringdup(t);
}

wchar_t* MailSyncSource::getFolderToSync() {
    return s_folderToSync;
}

BOOL MailSyncSource::isInFolderToSync(wchar_t* toCheck) {
    return wcsstr(s_folderToSync, toCheck) != NULL ? TRUE : FALSE;
}

void MailSyncSource::setIsMailInOutbox(BOOL value){
    isMailInOutbox = value;
    
}

BOOL MailSyncSource::getIsMailInOutbox() {
    return isMailInOutbox;
}

void MailSyncSource::setIsMailInInbox(BOOL value){
    isMailInInbox = value;
    
}

BOOL MailSyncSource::getIsMailInInbox() {
    return isMailInInbox;
}
/*
* Return the first SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
*/

SyncItem* MailSyncSource::getFirstItem() {    
    if (syncItem) {
        delete syncItem; syncItem = NULL;   
    } 
    //
    // initialize the flag
    //
    setIsMailInOutbox(FALSE);

    mailClient2Server.setFolderToSync(s_folderToSync);
	// FIXME    
    ArrayList* l = new ArrayList();
    l = mailClient2Server.setSync(SYNC_SLOW, getPath());
    
    extras = l->clone();
    
    MailClientData* m = mailClient2Server.getFirstItem();
    
    if (m) {              
        
        BOOL created = FALSE;
        //
        // get the MailExtraData object to retrieve i.e. message-id... It is only the 
        // pointer. don't destroy it!
        //
        MailExtraData* med = getMailExtraData(extras, 
                                (wchar_t*)m->getEmailData()->getEmailItem().getEntryID());
        
        // extra is empty because the previous file doesn't exist or is empty, or is a slow after previous
        // fast

        if (!med) {
            created = TRUE;
                    
            med = new MailExtraData();
            med->setId((wchar_t*)m->getEmailData()->getEmailItem().getEntryID());
            med->setMessageId(createMessageID((wchar_t*)m->getEmailData()->getEmailItem().getEntryID()));                           
            extras->add(*med);

        }
        
        m->getEmailData()->getEmailItem().setMessageId(med->getMessageId());
                
        if (created) {
            delete med; med = NULL;
        }

        syncItem = new SyncItem(m->getEmailData()->getEmailItem().getEntryID());        

        wchar_t* s = m->getEmailData()->format();
        LOG.debug(s);
        syncItem->setData(s, wcslen(s) * sizeof(wchar_t));                    
        syncItem->setState(SYNC_STATE_NONE);
        syncItem->setSourceParent(m->getParent());
        
        if (s) {delete [] s; s = NULL; }
        delete m; m = NULL;
    }
    
    return syncItem;    
}

/*
* Return the next SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
*/

SyncItem* MailSyncSource::getNextItem() {    
    if (syncItem) {
        delete syncItem; syncItem = NULL;   
    }
    
    MailClientData* m = mailClient2Server.getNextItem();   
    if (m) {      
                
        //
        // get the MailExtraData object to retrieve i.e. message-id...
        // It must exist after getFirstItem

        MailExtraData* med = getMailExtraData(extras, (wchar_t*)m->getEmailData()->getEmailItem().getEntryID());                
                
        if (med)
            m->getEmailData()->getEmailItem().setMessageId(med->getMessageId());               
        
        syncItem = new SyncItem(m->getEmailData()->getEmailItem().getEntryID());
        wchar_t* s = m->getEmailData()->format();
        LOG.debug(s);
        syncItem->setData(s, wcslen(s) * sizeof(wchar_t));            
        syncItem->setState(SYNC_STATE_NONE);
        syncItem->setSourceParent((wchar_t*)m->getParent());
        
        if (s) {delete [] s; s = NULL; }
        delete m; m = NULL;
    }

    return syncItem;  

}

SyncItem* MailSyncSource::getFirstItemKey() {
    
    if (syncItem) {
        delete syncItem; syncItem = NULL;   
    } 
    //
    // initialize the flag
    //
    setIsMailInOutbox(FALSE);
    
    mailClient2Server.setFolderToSync(s_folderToSync);
    mailClient2Server.setSync(SYNC_SLOW, getPath());
    
    MailClientData* m = mailClient2Server.getFirstItem();
    
    if (m) {      
        syncItem = new SyncItem(m->getEmailData()->getEmailItem().getEntryID());
        delete m; m = NULL;
    }
    
    return syncItem;    
    
}

SyncItem* MailSyncSource::getNextItemKey() {
    
    if (syncItem) {
        delete syncItem; syncItem = NULL;   
    }
    
    MailClientData* m = mailClient2Server.getNextItem();   
    if (m) {      
        syncItem = new SyncItem(m->getEmailData()->getEmailItem().getEntryID());
        delete m; m = NULL;
    }

    return syncItem;  
}

/*
* This method is always called by the sync manager. So the setModifiedItemsPOOM is always called
*/

SyncItem* MailSyncSource::getFirstNewItem() {

    if (syncItem) {
        delete syncItem; syncItem = NULL;   
    }
    //
    // initialize the flag
    //
    setIsMailInOutbox(FALSE);
    
    mailClient2Server.setFolderToSync(s_folderToSync);
    // FIXME
    ArrayList* l = new ArrayList();
    l = mailClient2Server.setSync(SYNC_TWO_WAY, getPath());
    extras = l->clone();
    
    MailClientData* m = mailClient2Server.getFirstNewItem();   
    
    if (m) {      

        BOOL created = FALSE;
        //
        // get the MailExtraData object to retrieve i.e. message-id...
        //
        MailExtraData* med = getMailExtraData(extras, (wchar_t*)m->getEmailData()->getEmailItem().getEntryID());
        
        if (!med) {
            created = TRUE;
                    
            med = new MailExtraData();
            med->setId((wchar_t*)m->getEmailData()->getEmailItem().getEntryID());
            med->setMessageId(createMessageID((wchar_t*)m->getEmailData()->getEmailItem().getEntryID()));    
            extras->add(*med);
        }
        
        m->getEmailData()->getEmailItem().setMessageId(med->getMessageId());

        if (created) {
            delete med; med = NULL;
        }

        syncItem = new SyncItem(m->getEmailData()->getEmailItem().getEntryID());
        
        wchar_t* s = m->getEmailData()->format();
        LOG.debug(s);
        syncItem->setData(s, wcslen(s) * sizeof(wchar_t));      
        syncItem->setState(SYNC_STATE_NEW);
        syncItem->setSourceParent((wchar_t*)m->getParent());
        
        if (s) {delete [] s; s = NULL; }
        delete m; m = NULL;
    }

    return syncItem;  
}

SyncItem* MailSyncSource::getNextNewItem() {    
    if (syncItem) {
        delete syncItem; syncItem = NULL;   
    }
    
    MailClientData* m = mailClient2Server.getNextNewItem();   
    
    if (m) {      

        MailExtraData* med = getMailExtraData(extras, (wchar_t*)m->getEmailData()->getEmailItem().getEntryID());                
                
        if (med)
            m->getEmailData()->getEmailItem().setMessageId(med->getMessageId());               
        
        syncItem = new SyncItem(m->getEmailData()->getEmailItem().getEntryID());
        
        wchar_t* s = m->getEmailData()->format();
        LOG.debug(s);
        syncItem->setData(s, wcslen(s) * sizeof(wchar_t));              
        syncItem->setState(SYNC_STATE_NEW);
        syncItem->setSourceParent((wchar_t*)m->getParent());
        if (s) {delete [] s; s = NULL; }
        delete m; m = NULL;
    }

    return syncItem;  
}

SyncItem* MailSyncSource::getFirstUpdatedItem() {
    if (syncItem) {
        delete syncItem; syncItem = NULL;   
    }  
        
    MailClientData* m = mailClient2Server.getFirstUpdatedItem();   
    
    if (m) {      
        syncItem = new SyncItem(m->getEmailData()->getEmailItem().getEntryID());
        
        wchar_t* s = m->getEmailData()->format();
        LOG.debug(s);
        syncItem->setData(s, wcslen(s) * sizeof(wchar_t));       
        syncItem->setState(SYNC_STATE_UPDATED);
        syncItem->setSourceParent((wchar_t*)m->getParent());
        if (s) {delete [] s; s = NULL; }
        delete m; m = NULL;
    }

    return syncItem;  
}

SyncItem* MailSyncSource::getNextUpdatedItem() {    
    if (syncItem) {
        delete syncItem; syncItem = NULL;   
    }  
    
    MailClientData* m = mailClient2Server.getNextUpdatedItem();   
    
    if (m) {      
        syncItem = new SyncItem(m->getEmailData()->getEmailItem().getEntryID());
        
        wchar_t* s = m->getEmailData()->format();
        LOG.debug(s);
        syncItem->setData(s, wcslen(s) * sizeof(wchar_t));       
        syncItem->setState(SYNC_STATE_UPDATED);
        syncItem->setSourceParent((wchar_t*)m->getParent());
        if (s) {delete [] s; s = NULL; }
        delete m; m = NULL;
    }

    return syncItem;  
}

SyncItem* MailSyncSource::getFirstDeletedItem() {
    if (syncItem) {
        delete syncItem; syncItem = NULL;   
    }  
    
    MailClientData* m = mailClient2Server.getFirstDeletedItem();   
    
    if (m) {      
        syncItem = new SyncItem(m->getEmailData()->getEmailItem().getEntryID());      
        syncItem->setState(SYNC_STATE_DELETED);
        syncItem->setSourceParent((wchar_t*)m->getParent());

        delete m; m = NULL;
    }

    return syncItem;  
}

SyncItem* MailSyncSource::getNextDeletedItem() {    
    if (syncItem) {
        delete syncItem; syncItem = NULL;   
    }  
    
    MailClientData* m = mailClient2Server.getNextDeletedItem();   
    
    if (m) {      
        syncItem = new SyncItem(m->getEmailData()->getEmailItem().getEntryID());     
        syncItem->setState(SYNC_STATE_DELETED);
        syncItem->setSourceParent((wchar_t*)m->getParent());

        delete m; m = NULL;
    }

    return syncItem;  
}

void MailSyncSource::setItemStatus(const wchar_t* key, int status) {
    wchar_t tmp[256];
    wsprintf(tmp, TEXT("key: %s, status: %i"), key, status);
    LOG.debug(tmp);
}

int MailSyncSource::addItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    MailClientData m = MailClientData();    
    MailExtraData med = MailExtraData();
    wchar_t* id = NULL;
    wchar_t tt[3];
    wchar_t *d = NULL;
    int errorCode = 0;
    
    mailServer2Client.setFolderToSync(s_folderToSync);
    
    if (item.getTargetParent() == NULL) {
        wsprintf(tt, TEXT("%c"), item.getKey()[0]);
    }  else {
        wsprintf(tt, TEXT("%s"), item.getTargetParent());    
    }
    
    if (!isInFolderToSync(tt))
        goto finally;      
    
    if (isMailInInbox == FALSE && wcscmp(tt, TEXT("I")) == 0) 
        setIsMailInInbox(TRUE);

    errorCode = m.emailData->parse((wchar_t*)item.getData(), item.getDataSize());

    if (errorCode != 0) {
        // failure
        goto finally;
    }    
    m.setParent(tt);
    
    id = mailServer2Client.addMessage(&m);
    
    if (id != NULL) {        
        ret = STC_ITEM_ADDED;        
        item.setKey(id);           
    }
    
    //
    // set all extra data
    //
    
    med.setId(id);
    med.setMessageId((wchar_t*)(m.emailData->getEmailItem().getMessageId()));    
    
    extras->add(med);

finally:
    
    if (d) {
        delete [] d; d = NULL;
    }
    return ret;
}

int MailSyncSource::updateItem(SyncItem& item) {
    
    int ret = STC_COMMAND_FAILED; // command Failed;
    wchar_t tt[3];
    MailClientData m = MailClientData(); 
    wchar_t *d = NULL;
    int errorCode = 0;
    HRESULT hr = S_OK;

    mailServer2Client.setFolderToSync(s_folderToSync);
          
     if (!isInFolderToSync(tt))
        goto finally;     
    
    errorCode = m.emailData->parse((wchar_t*)item.getData(), item.getDataSize());        

    if (errorCode != 0) {
        // failure
        goto finally;
    }
    
    m.setParent(tt);
           
    hr = mailServer2Client.updateMessage(&m);
    
    if (hr == S_OK) {        
        ret = STC_OK;                        
    }    
    
finally:
    
    if (d) {
        delete [] d; d = NULL;
    }
    return ret;
}

int MailSyncSource::deleteItem(SyncItem& item) {
    
   int ret = STC_COMMAND_FAILED; // command Failed;
    
    wchar_t* id = NULL;
    wchar_t tt[3];

    mailServer2Client.setFolderToSync(s_folderToSync);
    MailClientData m = MailClientData();    
    
    if (item.getTargetParent() == NULL) {
        wsprintf(tt, TEXT("%c"), item.getKey()[0]);
    }  else {
        wsprintf(tt, TEXT("%s"), item.getTargetParent());    
    }

    m.setParent(tt);

    m.emailData->getEmailItem().setEntryID(item.getKey());
    
    HRESULT hr = mailServer2Client.deleteMessage(&m);
    
    if (hr == S_OK) {        
        ret = STC_OK;                        
    }
        
    return ret;
}

int MailSyncSource::beginSync() {
    //
    // Nothing to do
    //
    return 0;
}

int MailSyncSource::endSync() {
    
    // mailClient2Server = MailClient2Server();     
    mailClient2ServerEnd.setFolderToSync(s_folderToSync);
    mailClient2ServerEnd.setSync(ONLY_ALL_MESSAGE_LIST, getPath());    
    mailClient2ServerEnd.writeCurrentMails(getPath(), extras);

    return 0;
}

void MailSyncSource::assign(MailSyncSource& s) {
    setSyncMode(s.getSyncMode());
    setLastSync(s.getLastSync());
    setNextSync(s.getNextSync());
    setRemoteURI(s.getRemoteURI());
    setType(s.getType());
    setLastAnchor(s.getLastAnchor());
    setNextAnchor(s.getNextAnchor());
    setErrorHandler(s.getErrorHandler());
    setFilter(s.getFilter());
    setPath(getPath());
    setFolderToSync(s.getFolderToSync());
    setIsMailInOutbox(s.getIsMailInOutbox());
    setIsMailInInbox(s.getIsMailInInbox());
    extras = s.extras->clone();
}

ArrayElement* MailSyncSource::clone() {
    MailSyncSource* s = new MailSyncSource(getName());
    s->setPath(getPath());
    
    s->assign(*this);

    return s;
}



