/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "notify/AddressChangeSyncSource.h"
#include "base/util/utils.h"
#include "base/Log.h"
#include "pim/checknet.h"

static int all   = -1;
static int allkeys = -1;
static ArrayList items;

//--------------------------------------------------------------- Static members
static wchar_t msisdn[40] = TEXT("") ;

// Set a new MS-ISDN number
void setMSIsdn(const wchar_t *val) {
    wsprintf(msisdn, TEXT("%s"), val);
}

// Get the current MS-ISDN number
const wchar_t * getMSIsdn() { return msisdn; }

//------------------------------------------------------------------ Constructor
AddressChangeSyncSource::AddressChangeSyncSource(const wchar_t* name)
    : SyncSource(name) {}


//--------------------------------------------------------------- Public Methods
SyncItem* AddressChangeSyncSource::getFirstItem() {
	SyncItem item;
    
	if ( !items.isEmpty() ) {
        items.clear();
    }    
    wchar_t ipaddress[64];

    getIPAddress(ipaddress);

	item.setKey(TEXT("ip"));
	item.setData( ipaddress, wcslen(ipaddress)*sizeof(wchar_t) );
	items.add(item);

    wchar_t dbg[128];
    wsprintf(dbg, TEXT("Local IP: %s"), ipaddress );
    LOG.debug(dbg);

	item.setKey(TEXT("msisdn"));
	item.setData( msisdn, wcslen(msisdn)*sizeof(wchar_t) );
	items.add(item);
	
    all = 0;
    return (SyncItem*)items[all];    
}

SyncItem* AddressChangeSyncSource::getNextItem() {
    if( all == -1)
        return NULL;
    all++;
    if (all == items.size()) {
        return NULL;
    }        
    return (SyncItem*)items[all];  
}

SyncItem* AddressChangeSyncSource::getFirstNewItem() {
    return getFirstItem();    
}

SyncItem* AddressChangeSyncSource::getNextNewItem() {    
    return getNextItem();
}

SyncItem* AddressChangeSyncSource::getFirstUpdatedItem() {
    return getFirstItem();    
}

SyncItem* AddressChangeSyncSource::getNextUpdatedItem() {    
    return getNextItem();
}

SyncItem* AddressChangeSyncSource::getFirstDeletedItem() {
    return NULL;
}

SyncItem* AddressChangeSyncSource::getNextDeletedItem() {    
    return NULL;
}

void AddressChangeSyncSource::setItemStatus(const wchar_t* key, int status) {
}

int AddressChangeSyncSource::addItem(SyncItem& item) {
    return 200;
}

int AddressChangeSyncSource::updateItem(SyncItem& item) {
    return 200;
}

int AddressChangeSyncSource::deleteItem(SyncItem& item) {
    return 200;
}

int AddressChangeSyncSource::beginSync() {
    return 0;
}
int AddressChangeSyncSource::endSync() {
    return 0;
}

ArrayElement* AddressChangeSyncSource::clone() {
    return new AddressChangeSyncSource(getName());
}

SyncItem *AddressChangeSyncSource::getFirstItemKey(void) {
    SyncItem item;
    
    if ( !items.isEmpty() ) {
        items.clear();
    }    
    wchar_t ipaddress[64];

    getIPAddress(ipaddress);

	item.setKey(TEXT("ip"));
	items.add(item);

    wchar_t dbg[128];
    wsprintf(dbg, TEXT("Local IP: %s"), ipaddress );
    LOG.debug(dbg);

	item.setKey(TEXT("msisdn"));
	items.add(item);
	
    allkeys = 0;
    return (SyncItem*)items[allkeys];
}

SyncItem *AddressChangeSyncSource::getNextItemKey(void) {
    if( allkeys == -1)
        return NULL;
    allkeys++;
    if (all == items.size()) {
        return NULL;
    }        
    return (SyncItem*)items[allkeys]; 
}
