/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "base/Log.h"
#include "client/DMTClientConfig.h"
#include "spds/SyncManager.h"

#include "notify/checknet.h"
#include "notify/addresschange.h"
#include "notify/AddressChangeSyncSource.h"

//------------------------------------------------------------ Config SubClass

// Address Notificator Configuration
//
// This class reads the same config used for DSClient from the DM,
// and changes some values for the address notification.
// The change to the url is now hard-coded. 
//
class AddressChangeConfig : public DMTClientConfig {
  
  public:
    
	AddressChangeConfig(const wchar_t *context): DMTClientConfig(context) {}

    bool set() {
        // Read access config from DM
        if( !read() )
            return false;

        // Replace SyncURL
        wchar_t *url = new wchar_t[wcslen(accessConfig.getSyncURL())+10];
        wcscpy(url, accessConfig.getSyncURL());
        // Find the last part of the path
        wchar_t *p = wcsrchr(url, CHR('/'));
        if( !p )
            return false;
        wcscpy(p, TEXT("/devinfo"));
        accessConfig.setSyncURL(url);
        delete [] url;

        // Discard source configs
        if (sourceConfigs) {
            delete [] sourceConfigs;
        }
        // Set AddressChangeSyncSource fixed config data
        sourceConfigsCount = 1;
        sourceConfigs = new SyncSourceConfig[1];

        sourceConfigs[0].setName(ADDR_CHANGE_SOURCE_NAME);
        sourceConfigs[0].setURI(TEXT(""));
        sourceConfigs[0].setSync(TEXT("addrchange"));
        sourceConfigs[0].setType(TEXT("text/plain"));
        
        return true;
    }
};

//----------------------------------------------------------- Public Functions

AN_ResponseCode notifyAddressChange(const wchar_t *context)
{
    AN_ResponseCode code = AN_Unspecified ;

    LOG.debug(TEXT("notifyAddressChange START...."));

    if(!checkNetwork())
        return AN_ServerUnreachable;

    AddressChangeSyncSource source(ADDR_CHANGE_SOURCE_NAME);
    
    SyncSource** ssArray = new SyncSource*[2];
    ssArray[0] = &source;
    ssArray[1] = NULL;

    AddressChangeConfig config(context);
    if( config.set() ) {
        LOG.debug(TEXT("notifyAddressChange: config set\n"));

		SyncManager sm(config);

        if (sm.prepareSync(ssArray)) {
            code = AN_AddressNotValid;  // TODO: check status
            wchar_t dbg[256];
			wsprintf(dbg,
                     TEXT("notifyAddressChange: prepareSync failed (%d: %s)\n"),
                     lastErrorCode, lastErrorMsg);
        }
        else {
            code = AN_AddressAccepted;  // Success
			LOG.debug(TEXT("notifyAddressChange: success\n"));
        }
    }
    else {
        code = AN_InvalidConfig ;
        LOG.debug(TEXT("notifyAddressChange: config failed\n"));
    }
    LOG.debug(TEXT("notifyAddressChange END...."));

    delete [] ssArray;

    return code;
}


