/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef INCL_MAIL_SYNC_SOURCE
#define INCL_MAIL_SYNC_SOURCE

#include "base/fscapi.h"
#include "spds/constants.h"
#include "spds/SyncItem.h"
#include "spds/SyncMap.h"
#include "spds/SyncStatus.h"
#include "spds/SyncSource.h"
#include "base/util/ArrayElement.h"
#include "pim/mail/MailClient2Server.h"
#include "pim/mail/MailServer2Client.h"
#include "pim/utils.h"
#include "pim/mail/MailUtils.h"


class MailSyncSource : public SyncSource {

private:    
    /*
    * It is the path in which the application is.
    */
    wchar_t* path;    
    wchar_t* s_folderToSync;    
    SyncItem* syncItem;
    
    static BOOL isMailInOutbox;
    static BOOL isMailInInbox;

    MailClient2Server mailClient2Server;
    MailClient2Server mailClient2ServerEnd;
    MailServer2Client mailServer2Client;
        

public:
    
    ArrayList* extras;
    
    void setPath(wchar_t* p);
    wchar_t* getPath();

    void setFolderToSync(wchar_t* t);
    wchar_t* getFolderToSync();
    BOOL isInFolderToSync(wchar_t* toCheck);
    
    static void setIsMailInOutbox(BOOL value);
    static BOOL getIsMailInOutbox();
    
    static void setIsMailInInbox(BOOL value);
    static BOOL getIsMailInInbox();
       

    /**
     * Constructor: create a SyncSource with the specified name
     *
     * @param name - the name of the SyncSource
     */
    MailSyncSource(const wchar_t* name);

    // TestSyncSource
    ~MailSyncSource();
    

    /*
     * Return the first SyncItem of all.
     * It is used in case of slow or refresh sync 
     * and retrieve the entire data source content.
     */
    SyncItem* getFirstItem();

    /*
     * Return the next SyncItem of all.
     * It is used in case of slow or refresh sync 
     * and retrieve the entire data source content.
     */
    SyncItem* getNextItem();

    /*
     * Return the first SyncItem of new one. It is used in case of fast sync 
     * and retrieve the new data source content.
     */
    SyncItem* getFirstNewItem();

    /*
     * Return the next SyncItem of new one. It is used in case of fast sync 
     * and retrieve the new data source content.
     */
    SyncItem* getNextNewItem();

    /*
     * Return the first SyncItem of updated one. It is used in case of fast sync 
     * and retrieve the new data source content.
     */
    SyncItem* getFirstUpdatedItem() ;

    /*
     * Return the next SyncItem of updated one. It is used in case of fast sync 
     * and retrieve the new data source content.
     */
    SyncItem* getNextUpdatedItem();

    /*
     * Return the first SyncItem of updated one. It is used in case of fast sync 
     * and retrieve the new data source content.
     */
    SyncItem* getFirstDeletedItem();

    /*
     * Return the next SyncItem of updated one. It is used in case of fast sync 
     * and retrieve the new data source content.
     */

    SyncItem* getNextDeletedItem();
    
    void setItemStatus(const wchar_t* key, int status);
    
    int addItem(SyncItem& item);
   
    int updateItem(SyncItem& item);

    int deleteItem(SyncItem& item);
    
    int beginSync();

    int endSync();

    void assign(MailSyncSource& s);

    ArrayElement* clone();

    SyncItem* getFirstItemKey();

    SyncItem* getNextItemKey();

};

#endif
