/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// service.cpp : register and deregister notlstnr service.
//

#include <windows.h>
#include <service.h>
#include <winsock2.h>
#include <ws2tcpip.h>

#include "logutils.h"
#include "notlstnr.h"

#define SERVICE_PREFIX TEXT("S4N")
#define SERVICE_DLL TEXT("notlstnr.dll")
#define SERVICE_PREFIX_INDEX TEXT("S4N0:")
#define SERVICE_INDEX 0
#define SERVICE_INFO 745

typedef HANDLE (*PFN_REGISTER_SERVICE)(
  LPCWSTR lpszType,
  DWORD dwIndex,
  LPCWSTR lpszLib,
  DWORD dwInfo
);

typedef BOOL (*PFN_DEREGISTER_SERVICE) (
  HANDLE hDevice
);

typedef BOOL (*PFN_SERVICE_IO_CONTROL) (
  HANDLE hService,
  DWORD dwIoControlCode,
  LPVOID lpInBuf,
  DWORD nInBufSize,
  LPVOID lpOutBuf,
  DWORD nOutBufSize,
  LPDWORD lpBytesReturned,
  LPOVERLAPPED lpOverlapped
);

typedef BOOL (*PFN_SERVICE_ADD_PORT) (
  HANDLE hService,
  SOCKADDR* pSockAddr,
  INT cbSockAddr,
  INT iProtocol,
  WCHAR szRegWritePath
);

typedef HANDLE (*PFN_GET_SERVICE_HANDLE) (
  LPWSTR szPrefix,
  LPWSTR szDllName,
  DWORD pdwDllBuf
);

int registerService() {
    PFN_REGISTER_SERVICE pfnRegister = NULL;
    PFN_SERVICE_IO_CONTROL pfnIOControl = NULL;
    PFN_SERVICE_ADD_PORT pfnAddPort = NULL;
    SOCKADDR_IN SockAddr;
    HANDLE s = 0;
    
    int ret = 0;

    HMODULE h = LoadLibrary(L"COREDLL.dll");
    if (h == NULL) {
        DebugPrint(TEXT("Library not found"));
        goto finally;
    }
    pfnRegister = 
        (PFN_REGISTER_SERVICE)GetProcAddress(h, L"RegisterService");

    if (pfnRegister == NULL) {
        DebugPrint(TEXT("Function RegisterService not found"));
        goto finally;
    }

    pfnIOControl =
        (PFN_SERVICE_IO_CONTROL)GetProcAddress(h, L"ServiceIoControl");

    if (pfnIOControl == NULL) {
        DebugPrint(TEXT("Function ServiceIoControl not found"));
        goto finally;
    }

    pfnAddPort =
        (PFN_SERVICE_ADD_PORT)GetProcAddress(h, L"ServiceAddPort");

    if (pfnAddPort == NULL) {
        DebugPrint(TEXT("Function ServiceAddPort not found"));
        goto finally;
    }


    s = (pfnRegister)(SERVICE_PREFIX, SERVICE_INDEX, SERVICE_DLL, SERVICE_INFO);

    if (s == NULL) {
		LPVOID lpMsgBuf;
		FormatMessage( 
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			GetLastError(),
			0, // Default language
			(LPTSTR) &lpMsgBuf,
			0,
			NULL 
		);
        DebugPrint(TEXT("Registering error: %s"), (LPCTSTR)lpMsgBuf);
        if (GetLastError() == 120) {
            DebugPrint(TEXT("Service arleady registered: %d"), GetLastError());
        }
        ret = GetLastError();
		// Free the buffer.
		LocalFree( lpMsgBuf );
        goto finally;
    }

    //
    // Now we register the super service...
    //
    memset(&SockAddr,0,sizeof(SockAddr));

    if(!(pfnIOControl)(s, IOCTL_SERVICE_REGISTER_SOCKADDR,0,0,0,0,0,0)) {
        DebugPrint(TEXT("The service does not support socked address registering"));
        goto finally; //does not support super services
    }

    SockAddr.sin_family = AF_INET;
    SockAddr.sin_port=htons(745);
    if (! (pfnAddPort)(s,(SOCKADDR*)&SockAddr, sizeof(SockAddr), 0,0)) {
        DebugPrint(TEXT("I cannot listen on port 745"));
        goto finally;
    }

    //When all sockets have been bound, the service must be informed that it
    //is ready to start.
    if(!(pfnIOControl)(s, IOCTL_SERVICE_STARTED,0,0,0,0,0,0)) {
        DebugPrint(TEXT("The service could not start...."));
        goto finally; //if the service cannot start
    }

    DebugPrint(TEXT("Service sucessfully registerd"));

finally:
    if (h) {
        FreeLibrary(h);
    }
    return ret;
}

int deregisterService() {
    HMODULE h = NULL;
    HANDLE  s = NULL; // service handle
    int ret = 0;

    PFN_GET_SERVICE_HANDLE pfnGetHandle = NULL;
    PFN_DEREGISTER_SERVICE pfnDeregister = NULL;

    h = LoadLibrary(L"COREDLL.dll");
    if (h == NULL) {
        DebugPrint(TEXT("Library not found"));
        goto finally;
    }
    pfnDeregister = 
        (PFN_DEREGISTER_SERVICE)GetProcAddress(h, L"DeregisterService");

    if (pfnDeregister == NULL) {
        DebugPrint(TEXT("Function DeregisterService not found"));
        goto finally;
    }

    pfnGetHandle = 
        (PFN_GET_SERVICE_HANDLE)GetProcAddress(h, L"GetServiceHandle");

    if (pfnGetHandle == NULL) {
        DebugPrint(TEXT("Function GetServiceHandle not found"));
        goto finally;
    }
    
    s = (pfnGetHandle)(SERVICE_PREFIX_INDEX, NULL, 0);

    if (s == INVALID_HANDLE_VALUE) {
        DebugPrint(TEXT("Service not found: %ld"), GetLastError());
        ret = GetLastError();
        goto finally;
    }

    if ((pfnDeregister)(s) == 0) {
        DebugPrint(TEXT("Deregistering error: %ld"), GetLastError());
        ret = GetLastError();
        goto finally;
    }

    s = NULL;
    
    DebugPrint(L"Service sucessfully deregisterd");

finally:
    if (h) {
        FreeLibrary(h);
    }
    return ret;
}

void activateService() {

}
